@@ -1,7 +1,7 @@
 /*
  * auxsys.c - Auxiliary system functions
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,17 +30,27 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: auxsys.c,v 1.8 2007/03/28 09:32:33 shirok Exp $
+ *  $Id: auxsys.c,v 1.12 2008-05-10 13:35:32 shirok Exp $
  */
 
 #include <gauche.h>
 #include <gauche/extend.h>
 #include "auxsysconf.h"
 
-#ifndef __MINGW32__
+#if !defined(GAUCHE_WINDOWS)
 #  include <sys/times.h>
+/* POSIX defines environ, and ISO C defines __environ.
+   Modern C seems to have the latter declared in unistd.h */
+extern char **environ;
 #endif
 
+#if defined(_MSC_VER)
+/* This is a kludge to patch the defect of auxsysconf.h misconfiguration. */
+#undef HAVE_SETENV
+#undef HAVE_UNSETENV
+#undef HAVE_GETPGID
+#endif /*_MSC_VER*/
+
 /*===============================================================
  * Environment
  */
@@ -74,8 +84,8 @@ void Scm_SetEnv(const char *name, const char *value, int overwrite)
         /* check the existence of NAME first. */
         if (getenv(name) != NULL) return;
     }
-    nlen = strlen(name);
-    vlen = strlen(value);
+    nlen = (int)strlen(name);
+    vlen = (int)strlen(value);
     /* we need malloc, since the pointer will be owned by the system */
     nameval = (char*)malloc(nlen+vlen+2);
     if (nameval == NULL) {
@@ -94,21 +104,52 @@ void Scm_SetEnv(const char *name, const char *value, int overwrite)
 #endif
 }
 
+/* Returns the system's environment table as a list of strings.
+   Each string is in the format of "key=value". */
+ScmObj Scm_Environ(void)
+{
+#if !defined(GAUCHE_WINDOWS)
+    if (environ == NULL) return SCM_NIL;
+    else return Scm_CStringArrayToList((const char**)environ, -1,
+                                       SCM_STRING_COPYING);
+#else  /*GAUCHE_WINDOWS*/
+#define ENV_BUFSIZ 64
+    LPVOID ss = GetEnvironmentStrings();
+    ScmObj h = SCM_NIL, t = SCM_NIL;
+    TCHAR *cp = (TCHAR*)ss, *pp;
+    TCHAR sbuf[ENV_BUFSIZ], *buf=sbuf;
+    int bsize = ENV_BUFSIZ, size;
+    
+    do {
+        for (pp=cp; *pp; pp++) /*proceed ptr*/;
+        size = (int)(pp - cp) + 1;
+        if (size >= bsize) {
+            buf = SCM_NEW_ATOMIC_ARRAY(TCHAR, size);
+            bsize = size;
+        }
+        memcpy(buf, cp, size*sizeof(TCHAR));
+        SCM_APPEND1(h, t, SCM_MAKE_STR_COPYING(SCM_WCS2MBS(buf)));
+        cp = pp+1;
+    } while (pp[1] != 0);
+    FreeEnvironmentStrings(ss);
+    return h;
+#endif /*GAUCHE_WINDOWS*/
+}
 
 /*==============================================================
- * Emulation stuff for Windows/MinGW
+ * Emulation stuff for Windows
  */
-#ifdef __MINGW32__
+#if defined(GAUCHE_WINDOWS)
 
 const char *getlogin(void)
 {
-    static char buf[256]; /* this isn't thread-safe, but getlogin() is
-			     inherently thread-unsafe call anyway */
-    DWORD size = sizeof(buf);
+    static TCHAR buf[256]; /* this isn't thread-safe, but getlogin() is
+                              inherently thread-unsafe call anyway */
+    DWORD size = sizeof(buf)/sizeof(TCHAR);
     BOOL r;
     r = GetUserName(buf, &size);
     if (r) {
-	return buf;
+	return SCM_WCS2MBS(buf);
     } else {
 	return NULL;
     }
@@ -134,7 +175,7 @@ clock_t times(struct tms *info)
     info->tms_cutime = 0;
     return 0;
 }
-#endif /*__MINGW32__*/
+#endif /*GAUCHE_WINDOWS*/
 
 
 /*
@@ -143,7 +184,7 @@ clock_t times(struct tms *info)
 
 extern void Scm_Init_auxsyslib(ScmModule *mod);
 
-void Scm_Init_auxsys(void)
+SCM_EXTENSION_ENTRY void Scm_Init_auxsys(void)
 {
     ScmModule *mod;
 
