@@ -1,7 +1,7 @@
 /*
  * jconv.c - alternative japanese code conversion routines
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: jconv.c,v 1.21 2007/04/16 12:52:32 shirok Exp $
+ *  $Id: jconv.c,v 1.24 2008-05-10 13:35:37 shirok Exp $
  */
 
 /* Some iconv() implementations don't support japanese character encodings,
@@ -49,8 +49,8 @@
 #include <ctype.h>
 #include "charconv.h"
 
-#define INCHK(n)   do{if (inroom < (n)) return INPUT_NOT_ENOUGH;}while(0)
-#define OUTCHK(n)  do{if (outroom < (n)) return OUTPUT_NOT_ENOUGH;}while(0)
+#define INCHK(n)   do{if ((int)inroom < (n)) return INPUT_NOT_ENOUGH;}while(0)
+#define OUTCHK(n)  do{if ((int)outroom < (n)) return OUTPUT_NOT_ENOUGH;}while(0)
 
 #define ERRP(n)    ((n)==INPUT_NOT_ENOUGH||(n)==OUTPUT_NOT_ENOUGH||(n)==ILLEGAL_SEQUENCE)
 
@@ -975,7 +975,8 @@ static size_t jis2eucj(ScmConvInfo *cinfo, const char *inptr, size_t inroom,
             *outchars = 1;
             return 1+inoffset;
         default:
-            Scm_Error("internal state of ISO2022-JP -> EUC_JP got messed up (%d).  Implementation error?", cinfo->istate);
+            /* Can't be here */
+            Scm_Panic("internal state of ISO2022-JP -> EUC_JP got messed up (%d).  Implementation error?", cinfo->istate);
         }
     }
     return ILLEGAL_SEQUENCE;
@@ -1010,7 +1011,8 @@ static size_t jis_ensure_state(ScmConvInfo *cinfo, int newstate, size_t outbytes
     case JIS_0212:
         escseq = "\033$(D"; esclen = 4; break;
     default:
-        Scm_Error("something wrong in jis_ensure_state: implementation error?");
+        /* Can't be here */
+        Scm_Panic("something wrong in jis_ensure_state: implementation error?");
         return 0;               /* dummy */
     }
     OUTCHK(esclen + outbytes);
@@ -1235,7 +1237,7 @@ static size_t jconv_1tier(ScmConvInfo *info, const char **iptr,
     ScmConvProc cvt = info->convproc[0];
     const char *inp = *iptr;
     char *outp = *optr;
-    int inr = *iroom, outr = *oroom; 
+    int inr = (int)*iroom, outr = (int)*oroom; 
     size_t outchars, inchars, converted = 0;
 
 #ifdef JCONV_DEBUG
@@ -1250,9 +1252,9 @@ static size_t jconv_1tier(ScmConvInfo *info, const char **iptr,
         } else {
             converted += inchars;
             inp += inchars;
-            inr -= inchars;
+            inr -= (int)inchars;
             outp += outchars;
-            outr -= outchars;
+            outr -= (int)outchars;
         }
     }
     *iptr = inp;
@@ -1272,7 +1274,7 @@ static size_t jconv_2tier(ScmConvInfo *info, const char **iptr, size_t *iroom,
     ScmConvProc ocvt = info->convproc[1];
     const char *inp = *iptr;
     char *outp = *optr;
-    int inr = *iroom, outr = *oroom;
+    int inr = (int)*iroom, outr = (int)*oroom;
     size_t outchars, inchars, bufchars, converted = 0;
 
 #ifdef JCONV_DEBUG
@@ -1295,9 +1297,9 @@ static size_t jconv_2tier(ScmConvInfo *info, const char **iptr, size_t *iroom,
         }
         converted += inchars;
         inp += inchars;
-        inr -= inchars;
+        inr -= (int)inchars;
         outp += outchars;
-        outr -= outchars;
+        outr -= (int)outchars;
     }
     *iptr = inp;
     *iroom = inr;
