@@ -196,13 +196,43 @@
        (find even? '#(3 1 7 5 4 8 7)))
 (test* "find (string)" #\a
        (find char-lower-case? "YAEUB4309aBrnar"))
-(test* "for-each (hash-table)" '(b . "b")
+(test* "find (hash-table)" '(b . "b")
        (find (lambda (x) (string=? "b" (cdr x)))
              (hash-table 'eq? '(a . "a") '(b . "b") '(c . "c"))))
 (test* "find (custom)" "zoo"
        (find (lambda (s) (= (size-of s) 3))
              (sseq 'najr 'ej 'zoo 'bunr)))
 
+(define (test-find-minmax msg ex-min ex-max coll . args)
+  (test* #`"find-min (,|msg|)" ex-min (apply find-min coll args))
+  (test* #`"find-max (,|msg|)" ex-max (apply find-max coll args))
+  (test* #`"find-min&max (,|msg|)" (list ex-min ex-max)
+         (receive r (apply find-min&max coll args) r)))
+
+
+(test-find-minmax "list 0" -1 9
+                  '(3 9 -1 6))
+(test-find-minmax "list 1" '(c . -1) '(b . 9)
+                  '((a . 3) (b . 9) (c . -1) (d . 6)) :key cdr)
+(test-find-minmax "list 2" '(c . -1) '(b . 9)
+                  '((c . -1) (a . 3) (b . 9) (d . 6)) :key cdr)
+(test-find-minmax "list 3" '(c . -1) '(c . -1)
+                  '((c . -1)) :key cdr)
+(test-find-minmax "list 4" #f #f
+                  '() :key cdr)
+(test-find-minmax "list 5" 'foo 'foo
+                  '() :key cdr :default 'foo)
+(test-find-minmax "list 6" '("a" . 3) '("d" . 6)
+                  '(("c" . -1) ("b" . 9) ("a" . 3) ("d" . 6))
+                  :key car :compare string<?)
+(test-find-minmax "vector" '(c . -1) '(b . 9)
+                  '#((a . 3) (b . 9) (c . -1) (d . 6)) :key cdr)
+(test-find-minmax "string" '#\a '#\Z
+                  "IBaenRuZgaeKG" :compare char-ci<?)
+(test-find-minmax "hash-table" '(c . -1) '(b . 9)
+                  (hash-table 'eq? '(a . 3) '(b . 9) '(c . -1) '(d . 6))
+                  :key cdr)
+
 (test* "filter (list)" '(2 4 6)
        (filter even? '(1 2 3 4 5 6 7)))
 (test* "filter (vector)" '(2 4 6)
@@ -554,4 +584,52 @@
                        :test (lambda (x y)
                                (= (modulo x 2) (modulo y 2)))))
 
+(define (permute-tester msg expected source order . fallback)
+  (define (unit type elt-coercer order-type)
+    (test* #`"permute,msg ,type by ,order-type"
+           (map-to type elt-coercer expected)
+           (apply permute
+                  (map-to type elt-coercer source)
+                  (coerce-to order-type order)
+                  (map elt-coercer fallback)))
+    (test* #`"permute!,msg ,type by ,order-type"
+           (if (= (size-of source) (size-of expected))
+             (map-to type elt-coercer expected)
+             *test-error*)
+           (let1 imp (map-to type elt-coercer source)
+             (apply permute!
+                    imp
+                    (coerce-to order-type order)
+                    (map elt-coercer fallback))
+             imp)))
+  (dolist (order-type `(,<list> ,<vector>))
+    (dolist (t `((,<list> ,values)
+                 (,<vector> ,values)
+                 (,<string> ,(lambda (elt) (string-ref (x->string elt) 0)))))
+      (unit (car t) (cadr t) order-type)))
+  )
+
+(permute-tester "" '(d a c b) '(a b c d) '(3 0 2 1))
+(permute-tester " (short)" '(d a) '(a b c d) '(3 0))
+(permute-tester " (long)"  '(d a z c b) '(a b c d) '(3 0 4 2 1) 'z)
+
+(define (shuffle-tester source)
+  (define (cmp a b)
+    (lset= eqv? (coerce-to <list> a) (coerce-to <list> b)))
+  (define (unit type)
+    (test* #`"shuffle ,type"
+           (coerce-to type source)
+           (shuffle (coerce-to type source))
+           cmp)
+    (test* #`"shuffle! ,type"
+           (coerce-to type source)
+           (let1 imp (coerce-to type source)
+             (shuffle! imp)
+             imp)
+           cmp))
+  (unit <list>) (unit <vector>) (unit <string>))
+
+(shuffle-tester '(#\a #\b #\c #\d #\e #\f #\g))
+(shuffle-tester '())
+
 (test-end)
