@@ -1,7 +1,7 @@
 /*
  * addr.c - socket address
  *
- *  Copyright (c) 2001-2007  Shiro Kawai  <shiro@acm.org>
+ *  Copyright (c) 2001-2008  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: addr.c,v 1.30 2007/03/24 09:02:55 shirok Exp $
+ *  $Id: addr.c,v 1.34 2008-05-10 13:35:41 shirok Exp $
  */
 
 #include "gauche/net.h"
@@ -187,11 +187,11 @@ static ScmObj sockaddr_in_allocate(ScmClass *klass, ScmObj initargs)
     addr->addr.sin_len = sizeof(struct sockaddr_in);
 #endif
     addr->addr.sin_family = AF_INET;
-    addr->addr.sin_port = htons(SCM_INT_VALUE(port));
+    addr->addr.sin_port = htons((short)SCM_INT_VALUE(port));
     if (SCM_STRINGP(host)) {
         const char *hname = Scm_GetStringConst(SCM_STRING(host));
         /* First, see if host is dotted number notation. */
-        if (inet_aton(hname, &addr->addr.sin_addr) == 0) {
+        if (inet_pton(AF_INET, hname, &addr->addr.sin_addr) <= 0) {
             /* Now, we need to look up the host name.
                Call MT-safe Scm_GetHostByName */
             ScmObj ap, hent = Scm_GetHostByName(hname);
@@ -203,7 +203,7 @@ static ScmObj sockaddr_in_allocate(ScmClass *klass, ScmObj initargs)
                 Scm_Error("host have unknown address type: %S", host);
             }
             hname = Scm_GetStringConst(SCM_STRING(SCM_CAR(ap)));
-            if (inet_aton(hname, &addr->addr.sin_addr) == 0) {
+            if (inet_pton(AF_INET, hname, &addr->addr.sin_addr) == 0) {
                 Scm_Error("host name lookup failure: %S", host);
             }
         }
