@@ -1,7 +1,7 @@
 /*
  * net.c - network interface
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,11 +30,10 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: net.c,v 1.53 2007/03/15 22:43:14 shirok Exp $
+ *  $Id: net.c,v 1.60 2008-05-10 13:35:41 shirok Exp $
  */
 
 #include "gauche/net.h"
-#include <unistd.h>
 #include <fcntl.h>
 #include <gauche/extend.h>
 #include <gauche/uvector.h>
@@ -107,10 +106,10 @@ ScmSocket *make_socket(Socket fd, int type)
 
 ScmObj Scm_MakeSocket(int domain, int type, int protocol)
 {
-    int sock; 
+    intptr_t sock; 
     SCM_SYSCALL(sock, socket(domain, type, protocol));
     if (SOCKET_INVALID(sock)) Scm_SysError("couldn't create socket");
-    return SCM_OBJ(make_socket(sock, type));
+    return SCM_OBJ(make_socket((Socket)sock, type));
 }
 
 ScmObj Scm_SocketShutdown(ScmSocket *s, int how)
@@ -165,11 +164,11 @@ ScmObj Scm_SocketInputPort(ScmSocket *sock, int buffering)
             sock->status < SCM_SOCKET_STATUS_CONNECTED) {
             sockport_err(sock, "input");
         }
-#ifndef __MINGW32__
+#ifndef GAUCHE_WINDOWS
 	infd = sock->fd;
-#else  /*__MINGW32__*/
+#else  /*GAUCHE_WINDOWS*/
 	infd = _open_osfhandle(sock->fd, O_RDONLY);
-#endif /*__MINGW32__*/
+#endif /*GAUCHE_WINDOWS*/
         if (infd == INVALID_SOCKET) sockport_err(sock, "input");
         
         /* NB: I keep the socket itself in the port name, in order to avoid
@@ -192,11 +191,11 @@ ScmObj Scm_SocketOutputPort(ScmSocket *sock, int buffering)
             sock->status < SCM_SOCKET_STATUS_CONNECTED) {
             sockport_err(sock, "output");
         }
-#ifndef __MINGW32__
+#ifndef GAUCHE_WINDOWS
 	outfd = sock->fd;
-#else  /*__MINGW32__*/
+#else  /*GAUCHE_WINDOWS*/
 	outfd = _open_osfhandle(sock->fd, 0);
-#endif /*__MINGW32__*/
+#endif /*GAUCHE_WINDOWS*/
         if (outfd == INVALID_SOCKET) sockport_err(sock, "output");
 
         /* NB: I keep the socket itself in the port name, in order to avoid
@@ -257,7 +256,7 @@ ScmObj Scm_SocketListen(ScmSocket *sock, int backlog)
 
 ScmObj Scm_SocketAccept(ScmSocket *sock)
 {
-    int newfd;
+    Socket newfd;
     struct sockaddr_storage addrbuf;
     socklen_t addrlen = sizeof(addrbuf);
     ScmSocket *newsock;
@@ -510,48 +509,73 @@ ScmObj Scm_SocketGetOpt(ScmSocket *s, int level, int option, int rsize)
 /*==================================================================
  * Windows/MinGW compatibility layer
  */
-#ifdef __MINGW32__
+#if defined(GAUCHE_WINDOWS)
 
-/* 
- * I should use WSAStringToAddress, but just for the time being...
- */
-int inet_aton(const char *cp, struct in_addr *inp)
+int inet_pton(int af, const char *src, void *dst)
 {
-    unsigned long r = inet_addr(cp);
-    if (r == (unsigned long)-1) {
-	return 0;
-    } else {
-	inp->s_addr = r;
-	return 1;
+    TCHAR *str = SCM_MBS2WCS(src);
+    int r;
+    INT addrsize;
+    struct sockaddr_in sa;
+    struct sockaddr_in6 sa6;
+    
+    switch (af) {
+    case AF_INET:
+        addrsize = (INT)sizeof(sa);
+        r = WSAStringToAddress(str, af, NULL, (LPSOCKADDR)&sa, &addrsize);
+        if (r != 0) return -1;
+        memcpy(dst, &sa.sin_addr, sizeof(struct in_addr));
+        return 1;
+    case AF_INET6:
+        addrsize = (INT)sizeof(sa6);
+        r = WSAStringToAddress(str, af, NULL, (LPSOCKADDR)&sa6, &addrsize);
+        if (r != 0) return -1;
+        memcpy(dst, &sa6.sin6_addr, sizeof(struct in6_addr));
+        return 1;
     }
+    return -1;
 }
 
-/* winsock requires some obscure initialization */
-static WSADATA wsaData;
-
-static void init_winwock(void)
+const char *inet_ntop(int af, const void *src, char *dst, socklen_t size)
 {
-    int opt;
-    int r = WSAStartup(MAKEWORD(2,2), &wsaData);
-    if (r != 0) {
-        SetLastError(r);
-        Scm_SysError("WSAStartup failed");
+#define ADDR_MAXLEN 64
+    struct sockaddr_in sa;
+    struct sockaddr_in6 sa6;
+    TCHAR buf[ADDR_MAXLEN];
+    int r;
+    size_t ressize;
+    DWORD tressize = ADDR_MAXLEN-1;
+    const char *res;
+    
+    switch (af) {
+    case AF_INET:
+        memset(&sa, 0, sizeof(sa));
+        sa.sin_family = AF_INET;
+        memcpy(&sa.sin_addr, src, sizeof(struct in_addr));
+        r = WSAAddressToString((LPSOCKADDR)&sa, (DWORD)sizeof(sa), NULL,
+                               buf, &tressize);
+        break;
+    case AF_INET6:
+        memset(&sa6, 0, sizeof(sa6));
+        sa6.sin6_family = AF_INET6;
+        memcpy(&sa6.sin6_addr, src, sizeof(struct in6_addr));
+        r = WSAAddressToString((LPSOCKADDR)&sa6, (DWORD)sizeof(sa6), NULL,
+                               buf, &tressize);
+        break;
+    default:
+        return NULL;
     }
-    /* windows voodoo to make _open_osfhandle magic work */
-    opt = SO_SYNCHRONOUS_NONALERT;
-    r = setsockopt(INVALID_SOCKET, SOL_SOCKET,
-                   SO_OPENTYPE, (char*)&opt, sizeof(opt));
-    if (r == SOCKET_ERROR) {
-        Scm_SysError("winsock initialization failed");
-    }
-}
 
-static void fini_winsock(void *data)
-{
-    (void)WSACleanup();
+    if (r != 0) return NULL;
+    buf[tressize] = 0;
+    res = SCM_WCS2MBS(buf);
+    ressize = strlen(res);
+    if (size <= (int)ressize) return NULL;
+    memcpy(dst, res, ressize+1);
+    return dst;
+#undef ADDR_MAXLEN
 }
-
-#endif /*__MINGW32__*/
+#endif /*GAUCHE_WINDOWS*/
                           
 /*==================================================================
  * Initialization
@@ -565,16 +589,12 @@ extern void Scm_Init_netaux(void);
 
 
 
-void Scm_Init_libnet(void)
+SCM_EXTENSION_ENTRY void Scm_Init_libnet(void)
 {
     ScmModule *mod;
 
     SCM_INIT_EXTENSION(net);
     mod = SCM_FIND_MODULE("gauche.net", SCM_FIND_MODULE_CREATE);
-#ifdef __MINGW32__
-    init_winsock();
-    Scm_AddCleanupHandler(fini_winsock, NULL);
-#endif /*__MINGW32__*/
 #ifdef HAVE_IPV6
     Scm_AddFeature("gauche.net.ipv6", NULL);
 #endif
