@@ -1,7 +1,7 @@
 /*
  * netdb.c - obtain information about network
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: netdb.c,v 1.16 2007/03/02 07:39:06 shirok Exp $
+ *  $Id: netdb.c,v 1.19 2008-05-10 13:35:41 shirok Exp $
  */
 
 #include "gauche/net.h"
@@ -139,7 +139,7 @@ ScmObj Scm_GetHostByAddr(const char *addr, int type)
     if (type != AF_INET) {
         Scm_Error("unsupported address type: %d", type);
     }
-    if (!inet_aton(addr, &iaddr)) {
+    if (inet_pton(AF_INET, addr, &iaddr) <= 0) {
         Scm_Error("bad inet address format: %s", addr);
     }
     
@@ -211,7 +211,7 @@ static ScmClassStaticSlotSpec hostent_slots[] = {
     SCM_CLASS_SLOT_SPEC("name", hostent_name, NULL),
     SCM_CLASS_SLOT_SPEC("aliases", hostent_aliases, NULL),
     SCM_CLASS_SLOT_SPEC("addresses", hostent_addresses, NULL),
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 /*-------------------------------------------------------------
@@ -331,7 +331,7 @@ static ScmClassStaticSlotSpec protoent_slots[] = {
     SCM_CLASS_SLOT_SPEC("name", protoent_name, NULL),
     SCM_CLASS_SLOT_SPEC("aliases", protoent_aliases, NULL),
     SCM_CLASS_SLOT_SPEC("proto", protoent_proto, NULL),
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 /*-------------------------------------------------------------
@@ -460,7 +460,7 @@ static ScmClassStaticSlotSpec servent_slots[] = {
     SCM_CLASS_SLOT_SPEC("aliases", servent_aliases, NULL),
     SCM_CLASS_SLOT_SPEC("port", servent_port, NULL),
     SCM_CLASS_SLOT_SPEC("proto", servent_proto, NULL),
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 /*-------------------------------------------------------------
