@@ -66,6 +66,9 @@
 (test* "list=" #t (list= eq? '(a)))
 (test* "list=" #t (list= eq? '(a) '(a)))
 (test* "list=" #f (list= eq? '(a) '(a b)))
+(test* "list=" #t (list= eq? '(a b) '(a b) '(a b)))
+(test* "list=" #f (list= eq? '(a b) '(a b) '(a b c)))
+(test* "list=" #f (list= eq? '(a b) '(a b) '(a)))
 (test* "list=" #t (list= char-ci=? '(#\a #\b #\z) '(#\A #\B #\Z)))
 (test* "list=" #f (list= char-ci=? '(#\a #\b #\z) '(#\A #\B)))
 (test* "first"  1 (first '(1 2 3 4 5 6 7 8 9 10)))
@@ -617,6 +620,14 @@
        (string-downcase "SPEAK SOFTLY"))
 (test* "string-downcase" "peak"
        (string-downcase "SPEAK SOFTLY" 1 5))
+(test* "string-downcase!" "speak softly"
+       (let ((s (string-copy "SPEAK SOFTLY")))
+         (string-downcase! s)
+         s))
+(test* "string-downcase!" "Speak softly"
+       (let ((s (string-copy "SPEAK SOFTLY")))
+         (string-downcase! s 1)
+         s))
 (test* "string-downcase!" "Speak SOFTLY"
        (let ((s (string-copy "SPEAK SOFTLY")))
          (string-downcase! s 1 5)
@@ -975,6 +986,11 @@
                (time=? t3 (date->time-tai (time-tai->date t3)))
                )))
 
+(test* "date past 2038/1/19" (date->time-utc (make-date 0 0 0 0 1 1 3000 0))
+       (julian-day->time-utc
+        (+ (date->julian-day (make-date 0 0 0 0 1 1 2000 0))
+           (+ (* 365 1000) 243))))
+
 ;; NB: in Gauche, the round-trip conversion from time -> julian-day -> time
 ;; can't be guaranteed because of the limited precision of julian-day
 ;; calcularion.   We round the nanosecond range.
@@ -1010,6 +1026,13 @@
        "2002/05/15 01:23:34.001234567 -1000 3"
        (date->string (make-date 1234567 34 23 1 15 5 2002 -36000)
                      "~Y/~m/~d ~H:~M:~S.~N ~z ~w"))
+(test* "date->string (past 2038)"
+       "2100/05/15 01:23:34.001234567 -1000 6"
+       (date->string
+        (time-utc->date
+         (date->time-utc
+          (make-date 1234567 34 23 1 15 5 2100 -36000)) -36000)
+        "~Y/~m/~d ~H:~M:~S.~N ~z ~w"))
 
 (test* "date->string"
        "02/05/ 1| 2|14"
@@ -1020,7 +1043,7 @@
        '(2002 5 15 12 34 56 -36000)
        (let1 d (string->date "2002/5/15 12:34:56 (-1000)"
                              "~Y/~m/~d ~H:~M:~S (~z)")
-         (map (lambda (s) (slot-ref d s))
+         (map (cut slot-ref d <>)
               '(year month day hour minute second zone-offset))))
 
 ;; NB: this test will fail when locale-dependent date name is supported.
@@ -1028,7 +1051,7 @@
        '(2002 11 2 7 14 11 32400)
        (let1 d (string->date "02/Nov/2002:07:14:11 +0900"
                              "~d~b~Y~H~M~S~z")
-         (map (lambda (s) (slot-ref d s))
+         (map (cut slot-ref d <>)
               '(year month day hour minute second zone-offset))))
 
 ;;
