@@ -1,7 +1,8 @@
 ;;;
 ;;; Array operations.  This is superset of SRFI-25.
 ;;;
-;;;  Copyright(C) 2002 by Shiro Kawai (shiro@acm.org)
+;;;  Copyright(C) 2002-2007 by Shiro Kawai (shiro@acm.org)
+;;;  Copyright(C) 2004      by Alex Shinn  (foof@synthcode.com)
 ;;;
 ;;;  Permission to use, copy, modify, distribute this software and
 ;;;  accompanying documentation for any purpose is hereby granted,
@@ -12,7 +13,7 @@
 ;;;  warranty.  In no circumstances the author(s) shall be liable
 ;;;  for any damages arising out of the use of this software.
 ;;;
-;;;  $Id: array.scm,v 1.12 2004/04/02 01:16:37 foof Exp $
+;;;  $Id: array.scm,v 1.18 2008-02-25 08:42:56 shirok Exp $
 ;;;
 
 ;; Conceptually, an array is a backing storage and a procedure to
@@ -25,6 +26,8 @@
   (use gauche.collection)
   (use gauche.sequence)
   (export <array-meta> <array>
+          <u8array> <s8array> <u16array> <s16array> <u32array> <s32array>
+          <u64array> <s64array> <f16array> <f32array> <f64array>
           array? make-array shape array array-rank
           array-start array-end array-ref array-set!
           share-array subarray array-equal?
@@ -37,7 +40,7 @@
           array-map array-map! array->vector array->list
           make-u8array make-s8array make-u16array make-s16array
           make-u32array make-s32array make-u64array make-s64array
-          make-f32array make-f64array
+          make-f16array make-f32array make-f64array
           array-concatenate array-transpose array-rotate-90 array-flip array-flip!
           identity-array array-inverse determinant determinant! array-mul array-expt
           array-div-left array-div-right array-add-elements array-add-elements!
@@ -73,7 +76,7 @@
       (define-method write-object ((self class) port)
         (format port "#,(~A ~S" name (array->list (array-shape self)))
         (array-for-each (cut format port " ~S" <>) self)
-        (display ")" port))
+        (format port ")"))
       (define-reader-ctor name
         (lambda (sh . inits)
           (list-fill-array! (make-array-internal class (apply shape sh)) inits))))))
@@ -149,20 +152,6 @@
   :backing-storage-setter s32vector-set!
   :backing-storage-length s32vector-length)
 
-(define-class <u32array> (<array-base>)
-  ()
-  :backing-storage-creator make-u32vector
-  :backing-storage-getter u32vector-ref
-  :backing-storage-setter u32vector-set!
-  :backing-storage-length u32vector-length)
-
-(define-class <s32array> (<array-base>)
-  ()
-  :backing-storage-creator make-s32vector
-  :backing-storage-getter s32vector-ref
-  :backing-storage-setter s32vector-set!
-  :backing-storage-length s32vector-length)
-
 (define-class <u64array> (<array-base>)
   ()
   :backing-storage-creator make-u64vector
@@ -177,6 +166,13 @@
   :backing-storage-setter s64vector-set!
   :backing-storage-length s64vector-length)
 
+(define-class <f16array> (<array-base>)
+  ()
+  :backing-storage-creator make-f16vector
+  :backing-storage-getter f16vector-ref
+  :backing-storage-setter f16vector-set!
+  :backing-storage-length f16vector-length)
+
 (define-class <f32array> (<array-base>)
   ()
   :backing-storage-creator make-f32vector
@@ -217,6 +213,8 @@
   (u64vector-copy self))
 (define-method copy-object ((self <s64vector>))
   (s64vector-copy self))
+(define-method copy-object ((self <f16vector>))
+  (f16vector-copy self))
 (define-method copy-object ((self <f32vector>))
   (f32vector-copy self))
 (define-method copy-object ((self <f64vector>))
@@ -319,14 +317,13 @@
         (unless (and (exact? (cadr l)) (integer? (cadr l)))
           (error "exact integer required for shape, but got" (cadr l)))
         (unless (<= (car l) (cadr l))
-          (error "beginning index ~s is larger than ending index ~s in shape argument: ~s" (car l) (cadr l) args)))
+          (errorf "beginning index ~s is larger than ending index ~s in shape argument: ~s" (car l) (cadr l) args)))
       ;; make array.
       (make <array>
         :start-vector (s32vector 0 0)
         :end-vector (s32vector rank 2)
         :mapper (lambda (Vi) (s32vector-dot (s32vector 2 1) Vi))
-        :backing-storage (list->vector args))
-      )))
+        :backing-storage (list->vector args)))))
 
 (define (shape->start/end-vector shape)
   (let* ((rank (array-end shape 0))
@@ -373,6 +370,8 @@
   (apply make-array-internal <u64array> shape opt))
 (define (make-s64array shape . opt)
   (apply make-array-internal <s64array> shape opt))
+(define (make-f16array shape . opt)
+  (apply make-array-internal <f16array> shape opt))
 (define (make-f32array shape . opt)
   (apply make-array-internal <f32array> shape opt))
 (define (make-f64array shape . opt)
@@ -499,7 +498,7 @@
                 ((constants coeffs)
                  (affine-proc->coeffs proc (size-of Vb)))
                 )
-    (make <array>
+    (make (class-of array)
       :start-vector Vb
       :end-vector   Ve
       :mapper (generate-shared-map (mapper-of array) constants coeffs)
@@ -741,9 +740,8 @@
   (apply array-map proc ar0 more))
 
 (define-method array-map ((proc <procedure>) ar0 . more)
-  (let1 target (make-array (array-shape ar0))
-    (apply array-map! target proc ar0 more)
-    target))
+  (rlet1 target (make-array (array-shape ar0))
+    (apply array-map! target proc ar0 more)))
 
 (define (array->vector ar)
   (with-builder (<vector> add! get :size (array-size ar))
