@@ -1,7 +1,7 @@
 /*
  * uvinit.c - initialize routine for uvector extension
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,11 +30,12 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: uvinit.c,v 1.13 2007/03/02 07:39:07 shirok Exp $
+ *  $Id: uvinit.c,v 1.19 2008-05-10 13:35:45 shirok Exp $
  */
 
 #include <gauche.h>
 #include <gauche/extend.h>
+#define EXTUVECTOR_EXPORTS
 #include "gauche/uvector.h"
 #include "uvectorP.h"
 
@@ -74,10 +75,8 @@ static ScmObj read_uvector(ScmPort *port, const char *tag,
  */
 extern void Scm_Init_uvlib(ScmModule *);
 extern void Scm_Init_uvseq(void);
-SCM_EXTERN ScmObj (*Scm_ReadUvectorHook)(ScmPort *port, const char *tag,
-                                         ScmReadContext *ctx);
  
-void Scm_Init_libgauche_uvector(void)
+SCM_EXTENSION_ENTRY void Scm_Init_libgauche_uvector(void)
 {
     ScmModule *m;
 
@@ -98,5 +97,5 @@ void Scm_Init_libgauche_uvector(void)
 
     Scm_Init_uvlib(m);
     Scm_Init_uvseq();
-    Scm_ReadUvectorHook = read_uvector;
+    Scm__InstallReadUvectorHook(read_uvector);
 }
