@@ -1,7 +1,7 @@
 ;;;
 ;;; parseopt.scm - yet another command-line argument parser
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: parseopt.scm,v 1.11 2007/03/02 07:39:08 shirok Exp $
+;;;  $Id: parseopt.scm,v 1.14 2008-05-10 13:35:56 shirok Exp $
 ;;;
 
 (define-module gauche.parseopt
@@ -43,18 +43,19 @@
 
 ;; Represents each option info
 (define-class <option-spec> ()
-  ((name :init-keyword :name) ;; option name
-   (args :init-keyword :args) ;; option agrspecs (list of chars)
-   (arg-optional? :init-keyword :arg-optional?) ;; option's arg optional?
-   (handler :init-keyword :handler) ;; handler closure
-   (optspec :init-keyword :optspec) ;; original <optspec> string
-   (help :init-keyword :help) ;; help string
+  ((name :init-keyword :name)        ; option name
+   (args :init-keyword :args)        ; option agrspecs (list of chars)
+   (arg-optional? :init-keyword :arg-optional?) ; option's arg optional?
+   (handler :init-keyword :handler)  ; handler closure
+   (plural? :init-keyword :plural?)  ; accept multiple options?
+   (optspec :init-keyword :optspec)  ; original <optspec> string
+   (help    :init-keyword :help)     ; help string
    ))
 
 ;; Helper functions
 
 ;; Parse optspec clause, and returns
-;;  ((option-string argument-specs optional? handler) ...)
+;;  (<option-spec> ...)
 ;; <a-spec> is (<optspec> <handler>) or
 ;; ((<optspec> <help-string>) <handler)
 (define (compose-entry a-spec)
@@ -65,13 +66,14 @@
         (handler (cadr a-spec)))
     (unless (string? optspec)
       (error "option spec must be a string, but got" optspec))
-    (rxmatch-if (rxmatch #/^-*([-+\w|]+)(?:([=:])(.+))?$/ optspec)
-        (#f optnames optional?  argspec)
+    (rxmatch-if (rxmatch #/^-*([-+\w|]+)(\*)?(?:([=:])(.+))?$/ optspec)
+        (#f optnames plural? optional?  argspec)
       (map (cute make <option-spec>
                  :name <>
                  :args (if argspec (string->list argspec) '())
                  :arg-optional? (equal? optional? ":")
                  :handler handler
+                 :plural? plural?
                  :optspec optspec
                  :help helpstr)
            (string-split optnames #\|))
@@ -80,16 +82,13 @@
 ;; From the args given at the command line, get a next option.
 ;; Returns option string and rest args.
 (define (next-option args)
-  (if (null? args)
-      (values #f '())
-      (rxmatch-case (car args)
-        (test (lambda (opt) (string=? opt "--"))
-              (values #f (cdr args)))
-        (#/^--?(\w[-+\w]*)(=(.*))?$/
-          (#f opt #f maybe-arg)
-          (values opt (if maybe-arg (cons maybe-arg (cdr args)) (cdr args))))
-        (else
-         (values #f args)))))
+  (cond
+   [(null? args) (values #f '())]
+   [(string=? (car args) "--") (values #f (cdr args))]
+   [(#/^--?(\w[-+\w]*)(=)?/ (car args))
+    => (lambda (m)
+         (values (m 1) (if (m 2) (cons (m'after) (cdr args)) (cdr args))))]
+   [else (values #f args)]))
 
 ;; From the list of optarg spec and given command line arguments,
 ;; get a list of optargs.  Returns optargs and unconsumed args.
@@ -99,43 +98,43 @@
         (errorf "a number is required for option ~a, but got ~a"
                 (ref optspec 'name) arg)))
   (define (get-real arg)
-    (or (and-let* ((num (string->number arg))
-                   ((real? num)))
-          num)
+    (or (and-let* ([num (string->number arg)]
+                   [ (real? num) ])
+          (exact->inexact num))
         (errorf "a real number is required for option ~a, but got ~a"
                 (ref optspec 'name) arg)))
   (define (get-integer arg)
-    (or (and-let* ((num (string->number arg))
-                   ((exact? num)))
+    (or (and-let* ([num (string->number arg)]
+                   [ (exact? num) ])
           num)
         (errorf "an integer is required for option ~a, but got ~a"
                 (ref optspec 'name) arg)))
   (define (get-sexp arg)
-    (guard (e ((<read-error> e)
+    (guard (e [(<read-error> e)
                (errorf "the argument for option ~a is not valid sexp: ~s"
-                       (ref optspec 'name) arg)))
+                       (ref optspec 'name) arg)])
       (read-from-string arg)))
   (define (process-args)
     (let loop ((spec (ref optspec 'args))
                (args args)
                (optargs '()))
-      (cond ((null? spec) (values (reverse! optargs) args))
-            ((null? args) (error "running out the arguments for option"
-                                 (ref optspec 'name)))
-            (else
+      (cond [(null? spec) (values (reverse! optargs) args)]
+            [(null? args) (error "running out the arguments for option"
+                                 (ref optspec 'name))]
+            [else
              (case (car spec)
-               ((#\s) (loop (cdr spec) (cdr args) (cons (car args) optargs)))
-               ((#\n) (loop (cdr spec) (cdr args)
-                            (cons (get-number (car args)) optargs)))
-               ((#\f) (loop (cdr spec) (cdr args)
-                            (cons (get-real (car args)) optargs)))
-               ((#\i) (loop (cdr spec) (cdr args)
-                            (cons (get-integer (car args)) optargs)))
-               ((#\e) (loop (cdr spec) (cdr args)
-                            (cons (get-sexp (car args)) optargs)))
-               ((#\y) (loop (cdr spec) (cdr args)
-                            (cons (string->symbol (car args)) optargs)))
-               (else (error "unknown option argument spec:" (car spec))))))
+               [(#\s) (loop (cdr spec) (cdr args) (cons (car args) optargs))]
+               [(#\n) (loop (cdr spec) (cdr args)
+                            (cons (get-number (car args)) optargs))]
+               [(#\f) (loop (cdr spec) (cdr args)
+                            (cons (get-real (car args)) optargs))]
+               [(#\i) (loop (cdr spec) (cdr args)
+                            (cons (get-integer (car args)) optargs))]
+               [(#\e) (loop (cdr spec) (cdr args)
+                            (cons (get-sexp (car args)) optargs))]
+               [(#\y) (loop (cdr spec) (cdr args)
+                            (cons (string->symbol (car args)) optargs))]
+               [else (error "unknown option argument spec:" (car spec))])])
       ))
 
   (if (ref optspec 'arg-optional?)
@@ -151,14 +150,14 @@
   (let loop ((args args))
     (receive (option nextargs) (next-option args)
       (if option
-          (cond ((find (lambda (e) (equal? option (ref e 'name))) speclist)
-                 => (lambda (entry)
-                      (receive (optargs nextargs)
-                          (get-optargs entry nextargs)
-                        (apply (ref entry 'handler) optargs)
-                        (loop nextargs))))
-                (else (fallback option nextargs loop)))
-          nextargs))))
+        (cond [(find (lambda (e) (equal? option (ref e 'name))) speclist)
+               => (lambda (entry)
+                    (receive (optargs nextargs)
+                        (get-optargs entry nextargs)
+                      (apply (ref entry 'handler) optargs)
+                      (loop nextargs)))]
+              [else (fallback option nextargs loop)])
+        nextargs))))
 
 ;; Build 
 (define (build-option-parser spec fallback)
