@@ -1,7 +1,7 @@
 ;;;
 ;;; process.scm - process interface
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,9 +30,11 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: process.scm,v 1.27 2007/03/05 09:21:35 shirok Exp $
+;;;  $Id: process.scm,v 1.33 2008-05-10 13:35:56 shirok Exp $
 ;;;
 
+#!no-fold-case
+
 ;; process interface, mostly compatible with STk's, but implemented
 ;; as an object on top of basic system interface.
 
@@ -40,6 +42,7 @@
   (use srfi-1)
   (use srfi-2)
   (use srfi-13)
+  (use srfi-14)
   (export <process> <process-abnormal-exit>
           run-process process? process-alive? process-pid
           process-command process-input process-output process-error
@@ -52,6 +55,8 @@
           with-input-from-process   with-output-to-process
           call-with-process-io
           process-output->string    process-output->string-list
+          ;; shell utilities
+          shell-escape-string
           ))
 (select-module gauche.process)
 
@@ -80,6 +85,10 @@
               "active"
               "inactive")))
 
+;; null device
+(define *nulldev*
+  (cond-expand (gauche.os.windows "NUL") (else "/dev/null")))
+
 ;; create process and run.
 (define (run-process command . args)
   (if (not (list? command))
@@ -198,24 +207,24 @@
   (let1 status (ref process 'status)
     (unless (and (sys-wait-exited? status)
                  (zero? (sys-wait-exit-status status)))
-      (cond ((sys-wait-exited? status)
+      (cond [(sys-wait-exited? status)
              (errorf <process-abnormal-exit> :process process
                      "~s exitted abnormally with exit code ~a"
-                     process (sys-wait-exit-status status)))
-            ((sys-wait-signaled? status)
+                     process (sys-wait-exit-status status))]
+            [(sys-wait-signaled? status)
              (errorf <process-abnormal-exit> :process process
                      "~s exitted by signal ~s"
-                     process (sys-signal-name (sys-wait-termsig status))))
-            (else ;; we shouldn't be here, but just in case...
+                     process (sys-signal-name (sys-wait-termsig status)))]
+            [else ;; we shouldn't be here, but just in case...
              (errorf <process-abnormal-exit> :process process
                      "~s exitted abnormally with status ~s"
-                     process status))))))
+                     process status)]))))
 
 ;; other basic interfaces
 (define (process? obj) (is-a? obj <process>))
 (define (process-alive? process)
   (and (not (process-exit-status process))
-       (>= (process-pid process) 0)))
+       (process-pid process)))
 (define (process-list) (class-slot-ref <process> 'processes))
 
 ;;-----------------------------------------------------------------
@@ -226,7 +235,7 @@
     (let-optionals* args ((nohang? #f)
                           (raise-error? #f))
       (receive (p code) (sys-waitpid (process-pid process) :nohang nohang?)
-        (and (not (zero? p))
+        (and (not (eqv? p 0))
              (begin
                (slot-set! process 'status code)
                (slot-set! process 'processes
@@ -240,11 +249,12 @@
                         (raise-error? #f))
     (and (not (null? (process-list)))
          (receive (pid status) (sys-waitpid -1 :nohang nohang?)
-           (and pid
-                (and-let* ((p (find (lambda (pp) (= (process-pid pp) pid))
+           (and (not (eqv? pid 0))
+                (and-let* ((p (find (lambda (pp) (eqv? (process-pid pp) pid))
                                     (process-list))))
-                  (slot-set! p 'status status)
                   (update! (ref p 'processes) (cut delete p <>))
+                  (set! (ref p 'status) status)
+                  (set! (ref p 'pid) #f)
                   (when raise-error? (%check-normal-exit p))
                   p))))))
 
@@ -252,9 +262,15 @@
 (define (process-send-signal process signal)
   (when (process-alive? process)
     (sys-kill (process-pid process) signal)))
-(define (process-kill process) (process-send-signal process |SIGKILL|))
-(define (process-stop process) (process-send-signal process |SIGSTOP|))
-(define (process-continue process) (process-send-signal process |SIGCONT|))
+(define (process-kill process) (process-send-signal process SIGKILL))
+(define (process-stop process)
+  (cond-expand
+   (gauche.os.windows (undefined))
+   (else (process-send-signal process SIGSTOP))))
+(define (process-continue process)
+  (cond-expand
+   (gauche.os.windows (undefined))
+   (else (process-send-signal process SIGCONT))))
 
 ;;===================================================================
 ;; Process ports
@@ -268,7 +284,7 @@
 ;;   :on-abmormal-exit - :error, :ignore, or a handler (called w/ process)
 
 (define (open-input-process-port command . opts)
-   (let-keywords opts ((input "/dev/null")
+   (let-keywords opts ((input *nulldev*)
                        (err :error #f)
                        (host #f)
                        . rest)
@@ -276,7 +292,7 @@
        (values (wrap-input-process-port p rest) p))))
 
 (define (call-with-input-process command proc . opts)
-  (let-keywords opts ((input "/dev/null")
+  (let-keywords opts ((input *nulldev*)
                       (err :error #f)
                       (host #f)
                       (on-abnormal-exit :error)
@@ -295,7 +311,7 @@
          opts))
 
 (define (open-output-process-port command . opts)
-  (let-keywords opts ((output "/dev/null")
+  (let-keywords opts ((output *nulldev*)
                       (err :error #f)
                       (host #f)
                       . rest)
@@ -303,7 +319,7 @@
       (values (wrap-output-process-port p rest) p))))
 
 (define (call-with-output-process command proc . opts)
-  (let-keywords opts ((output "/dev/null")
+  (let-keywords opts ((output *nulldev*)
                       (err :error #f)
                       (host #f)
                       (on-abnormal-exit :error)
@@ -351,14 +367,49 @@
 (define (process-output->string-list command . opts)
   (apply call-with-input-process command port->string-list opts))
 
+;;---------------------------------------------------------------------
+;; Shell utility
+;;
+
+(define (shell-escape-string str)
+  (cond-expand
+   [gauche.os.windows
+    ;; There seems no reliable way to escape command line arguments on
+    ;; windows, since the parsing is up to every application.  However,
+    ;; the standard C runtime seems to obey that (a) whitespaces can be
+    ;; embedded if the argument is surrounded by double quotes, and (b)
+    ;; within double-quotes, consecutive two double-quotes are replaced
+    ;; for one double-quote.
+    (cond [(string-null? str) "\"\""]
+          [(string-index str #[\s\"])
+           (string-append "\"" (regexp-replace-all #/\"/ str "\"\"") "\"")]
+          [else str])]
+   [else
+    ;; We follow standard unix shell convention: if STR contains special
+    ;; chars, we quote the entire STR by single-quotes.  If STR contains
+    ;; a single quote, we replace it with '"'"'.
+    (cond [(string-null? str) "''"]
+          [(string-index str #[\s\\\"\'*?$<>!\[\](){}])
+           (string-append "'" (regexp-replace-all #/'/ str "'\"'\"'") "'")]
+          [else str])]))
+
 ;;----------------------------------------------------------------------
 ;; Internal utilities for process ports
 ;;
 
 ;; If the given command is a string, return an argv to use /bin/sh.
+;; NB: on Windows we need to use cmd.exe.  But its command-line parsing
+;; rule is too broken to use reliably.  Another possibility is to implement
+;; much of high-level /bin/sh functionalities in Scheme, so that we can
+;; provide consistent behavior.  Something to think about.
 (define (%apply-run-process command stdin stdout stderr host)
   (apply run-process
-         (cond ((string? command) `("/bin/sh" "-c" ,command))
+         (cond ((string? command)
+                (cond-expand
+                 (gauche.os.windows
+                  `("cmd" "/c" ,command))
+                 (else
+                  `("/bin/sh" "-c" ,command))))
                ((list? command) command)
                (else (error "Bad command spec" command)))
          :input stdin :output stdout :host host
@@ -386,6 +437,7 @@
   (case on-abnormal-exit
     ((:error) (%check-normal-exit process))
     ((:ignore))
-    (else (on-abnormal-exit process))))
+    (else (unless (zero? (process-exit-status process))
+            (on-abnormal-exit process)))))
 
 (provide "gauche/process")
