@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.test - test framework
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,9 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: test.scm,v 1.25 2007/04/08 01:03:45 shirok Exp $
+;;;  $Id: test.scm,v 1.29 2008-05-10 13:35:56 shirok Exp $
+
+#!no-fold-case
 
 ;; Writing your own test
 ;;
@@ -126,7 +128,7 @@
           (vector-ref *test-counts* 3)))
 (define (read-summary)
   (when (and (string? *test-record-file*)
-             (file-exists? *test-record-file*))
+             (sys-access *test-record-file* F_OK)) ; avoid file-exists? to trigger autoload
     (with-input-from-file *test-record-file*
       (lambda ()
         (let ((m (rxmatch #/Total:\s+(\d+)\s+tests,\s+(\d+)\s+passed,\s+(\d+)\s+failed,\s+(\d+)\s+aborted/ (read-line))))
@@ -284,12 +286,14 @@
   (define code->list (with-module gauche.internal vm-code->list))
   (let loop ((r '())
              (code (code->list (closure-code closure))))
-    (cond ((null? code) r)
-          ((identifier? (car code))
-           (loop (cons (car code) r) (cdr code)))
-          ((is-a? (car code) <compiled-code>)
-           (loop (loop r (code->list (car code))) (cdr code)))
-          (else (loop r (cdr code))))))
+    (cond [(null? code) r]
+          [(identifier? (car code))
+           (loop (cons (car code) r) (cdr code))]
+          [(is-a? (car code) <compiled-code>)
+           (loop (loop r (code->list (car code))) (cdr code))]
+          [(list? (car code)) ; for operand of LOCAL-ENV-CLOSURES
+           (loop (loop r (car code)) (cdr code))]
+          [else (loop r (cdr code))])))
 
 (define (dangling-gref? ident closure)
   (and (not ((with-module gauche.internal find-binding)
@@ -340,6 +344,8 @@
 
     (when *test-record-file*
       (write-summary))
-    ))
+
+    ;; Returns the number of failed tests.
+    (length *discrepancy-list*)))
 
 (provide "gauche/test")
