@@ -1,7 +1,7 @@
 ;;;
 ;;; file/util.scm - filesystem utility functions
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: util.scm,v 1.6 2007/03/02 07:39:12 shirok Exp $
+;;;  $Id: util.scm,v 1.14 2008-05-11 08:30:53 shirok Exp $
 ;;;
 
 ;;; This module provides convenient utility functions to handle
@@ -38,15 +38,18 @@
 ;;; different names for compatibility with existing Scheme
 ;;; implementations.
 
+#!no-fold-case
+
 (define-module file.util
   (use srfi-1)
-  (use srfi-2)
   (use srfi-11)
   (use srfi-13)
   (use util.list)
+  (use util.match)
   (export current-directory directory-list directory-list2 directory-fold
           home-directory temporary-directory
           make-directory* create-directory* remove-directory* delete-directory*
+          copy-directory*
           build-path resolve-path expand-path simplify-path decompose-path
           absolute-path? relative-path? find-file-in-paths
 	  path-separator
@@ -59,52 +62,61 @@
           file-mtime=? file-mtime<? file-mtime<=? file-mtime>? file-mtime>=?
           file-atime=? file-atime<? file-atime<=? file-atime>? file-atime>=?
           file-ctime=? file-ctime<? file-ctime<=? file-ctime>? file-ctime>=?
-          touch-file copy-file move-file
+          touch-file copy-file move-file 
+          ;copy-files move-files
+          remove-files delete-files
           file->string file->string-list file->list file->sexp-list
           ))
 (select-module file.util)
 
-;; common util
+;; Common util.  Returns #f if PATH does not exist.
+
+(define (safe-stat path follow-link?)
+  (if (cond-expand [gauche.os.windows #t]
+                   [else follow-link?])
+    (and (sys-access path F_OK) (sys-stat path))
+    ;; When we don't follow symlink, we can't use sys-access since it returns
+    ;; #f if PATH is a dangling symlink.
+    (guard (e [(and (<system-error> e) (eq? (ref e 'errno) ENOENT)) #f])
+      (sys-lstat path))))
+
 (define (%stat path follow-link?)
-  (let1 stat (if follow-link? sys-stat sys-lstat)
-    (stat path)))
+  ((if follow-link? sys-stat sys-lstat) path))
 
 ;;;=============================================================
 ;;; Directory entries
 
 ;; "current-directory" is found in ChezScheme, MzScheme, etc.
-(define (current-directory . maybe-newdir)
-  (cond ((null? maybe-newdir) (sys-getcwd))
-        ((null? (cdr maybe-newdir))
-         (if (string? (car maybe-newdir))
-             (sys-chdir (car maybe-newdir))
-             (error "directory name should be a string" (car maybe-newdir))))
-        (else
-         (error "too many arguments for current-directory" maybe-newdir))))
+(define (current-directory . opts)
+  (let-optionals* opts ((newdir #f))
+    (match newdir
+      [#f (sys-getcwd)]
+      [(? string?) (sys-chdir newdir)]
+      [_ (error "directory name should be a string" newdir)])))
 
 (define (home-directory . maybe-user)
   (let-optionals* maybe-user ((user (sys-getuid)))
-    (and-let* ((ent (cond ((integer? user) (sys-getpwuid user))
+    (and-let* ([ent (cond ((integer? user) (sys-getpwuid user))
                           ((string? user)  (sys-getpwnam user))
-                          (else (error "bad user" user)))))
+                          (else (error "bad user" user)))])
       (slot-ref ent 'dir))))
 
 (define (temporary-directory)
-  "/tmp")  ;; would be more smarter
+  (or (sys-getenv "TMPDIR") "/tmp"))
 
 ;; utility for directory-list and directory-list2
 (define (%directory-filter dir pred filter-add-path?)
   (if filter-add-path?
-      (filter (lambda (e) (pred (build-path dir e))) (sys-readdir dir))
-      (filter pred (sys-readdir dir))))
+    (filter (lambda (e) (pred (build-path dir e))) (sys-readdir dir))
+    (filter pred (sys-readdir dir))))
 
 (define (%directory-filter-compose opts)
   (let-keywords opts ((children? #f)
                       (filter #f))
     (apply every-pred
-           (cond-list (children?
-                       (lambda (e) (not (member (sys-basename e) '("." "..")))))
-                      (filter)))))
+           (cond-list
+            [children? (lambda (e) (not (member (sys-basename e) '("." ".."))))]
+            [filter]))))
 
 ;; directory-list DIR &keyword ADD-PATH? FILTER-ADD-PATH? CHILDREN? FILTER
 (define (directory-list dir . opts)
@@ -115,8 +127,8 @@
         (sort (%directory-filter dir (%directory-filter-compose rest)
                                  filter-add-path?))
       (if add-path?
-          (map (cut build-path dir <>) entries)
-          entries))))
+        (map (cut build-path dir <>) entries)
+        entries))))
 
 ;; directory-list2 DIR &optional ADD-DIR? FILTER-ADD-PATH? CHILDREN? FILTER FOLLOW-LINK?
 (define (directory-list2 dir . opts)
@@ -126,24 +138,21 @@
                       . rest)
     (let* ((filters (%directory-filter-compose rest))
            (selector (lambda (e)
-                       (and (file-exists? e)
-                            (eq? (slot-ref (%stat e follow-link?) 'type)
-                                 'directory))))
+                       (and-let* ([s (safe-stat e follow-link?)])
+                         (eq? (slot-ref s 'type) 'directory))))
            (entries (sort (%directory-filter dir filters filter-add-path?))))
       (if add-path?
-          (partition selector
-                     (map (cut build-path dir <>) entries))
-          (partition (lambda (e) (selector (build-path dir e)))
-                     entries)))))
+        (partition selector (map (cut build-path dir <>) entries))
+        (partition (lambda (e) (selector (build-path dir e)))
+                   entries)))))
 
 ;; directory-fold DIR PROC KNIL &keyword LISTER FOLDER FOLLOW-LINK?
 (define (directory-fold dir proc knil . opts)
   (let-keywords opts ((lister (lambda (path knil)
-                                (values
-                                 (directory-list path
-                                                 :add-path? #t
-                                                 :children? #t)
-                                 knil)))
+                                (values (directory-list path
+                                                        :add-path? #t
+                                                        :children? #t)
+                                        knil)))
                       (folder fold)
                       (follow-link? #t))
     (define (selector e)
@@ -194,6 +203,70 @@
 
 (define delete-directory* remove-directory*)
 
+;; cp -r
+;;   if-exists  - :error :supersede :backup :overwrite #f
+;;   backup-suffix
+;;   safe
+;;   follow-link? - follow symlink.  the default is #f
+;;   keep-timestamp
+;;   keep-mode
+(define (copy-directory* src dst . opts)
+  (let-keywords opts ((if-exists :error)
+                      (backsfx :backup-suffix ".orig")
+                      (safe #f)
+                      (follow-link? #f)
+                      (keeptime :keep-timestamp #f)
+                      (keepmode :keep-mode #f))
+
+    (define (postprocess src dst)
+      (let1 stat (sys-stat src)
+        (when keeptime (sys-utime dst (ref stat'atime) (ref stat'mtime)))
+        (when keepmode (sys-chmod dst (ref stat'perm)))))
+
+    (define (do-copy src dst)
+      (copy-file src dst :if-exists if-exists :backup-suffix backsfx
+                 :follow-link? follow-link?
+                 :keep-timestamp keeptime :keep-mode keepmode))
+    
+    (define (rec src dst)
+      (cond [(file-is-directory? src)
+             (let1 exists? (file-exists? dst)
+               (if (not exists?)
+                 (sys-mkdir dst #o777)
+                 (case if-exists
+                   [(:error) (error "destination file exists" dst)]
+                   [(:supersede)
+                    (remove-files dst)
+                    (sys-mkdir dst #o777)]
+                   [(:overwrite)
+                    (unless (file-is-directory? dst)
+                      (remove-files dst))
+                    ;; TODO: if dst directory exists, check the permission
+                    ]
+                   ;; NB: :backup is handled by toplevel
+                   [else (error "unknown if-exists value:" if-exists)]))
+               (for-each (lambda (s)
+                           (rec s (build-path dst (sys-basename s))))
+                         (directory-list src :add-path? #t :children? #t))
+               (unless exists? (postprocess src dst))
+               )]
+            [else (do-copy src dst)]))
+
+    (cond [(file-is-symlink? src) ; we check this first, for src may be a dangling symlink.
+           (do-copy src dst)]
+          [(not (file-exists? src))
+           (error "source file does not exist:" src)]
+          [(file-exists? dst)
+           (case if-exists
+             [(:backup)
+              (sys-rename dst (string-append dst backsfx))
+              (rec src dst)]
+             [(#f) #f]
+             [else  ; other if-exists options are handled later
+              (rec src dst)])]
+          [else (rec src dst)])
+    ))
+
 ;;;=============================================================
 ;;; Pathnames
 
@@ -201,21 +274,20 @@
   (define path-separator-string (string (path-separator)))
   (define (rec base components)
     (if (null? components)
-        base
-        (let1 component
-            (let1 p (car components)
-              (cond ((eq? p 'up) "..")
-                    ((eq? p 'same) ".")
-                    ((absolute-path? p)
-                     (error "can't append absolute path after other path" p))
-                    (else (sys-normalize-pathname p))))
-          (rec (cond ((string-null? base) component)
-                     ((string-null? component) base)
-                     ((#/[\/\\]$/ base)
-                      (string-append base component))
-                     (else
-                      (string-append base path-separator-string component)))
-               (cdr components)))))
+      base
+      (let1 component
+          (match (car components)
+            ['up   ".."]
+            ['same "."]
+            [(? absolute-path? p)
+             (error "can't append absolute path after other path" p)]
+            [p (sys-normalize-pathname p)])
+        (rec (cond [(string-null? base) component]
+                   [(string-null? component) base]
+                   [(#/[\/\\]$/ base) (string-append base component)]
+                   [else
+                    (string-append base path-separator-string component)])
+          (cdr components)))))
   (rec (sys-normalize-pathname base-path) components))
 
 (define (expand-path path)
@@ -231,17 +303,16 @@
                        (values dir pat)
                        (let1 nd (rec dir)
                          (values nd (pathcat nd base))))))
-      (unless (sys-access p |F_OK|)
+      (unless (sys-access p F_OK)
         (error "path doesn't exist" path))
       (let loop ((count 0) (p p))
-        (cond ((>= count 8) ;; arbitrary upper bound to avoid infinite loop
-               (error "possibly looping symlink" pat))
-              ((eq? (file-type p :follow-link? #f) 'symlink)
+        (cond [(>= count 8) ;; arbitrary upper bound to detect infinite loop
+               (error "possibly looping symlink" pat)]
+              [(eq? (file-type p :follow-link? #f) 'symlink)
                (loop (+ count 1)
                      (let1 np (sys-readlink p)
-                       (if (absolute-path? np) np (pathcat ndir np)))))
-              (else p)))
-      ))
+                       (if (absolute-path? np) np (pathcat ndir np))))]
+              [else p]))))
   (rec (expand-path path)))
 
 (define (simplify-path path)
@@ -250,42 +321,39 @@
 (define (decompose-path path)
   (if (#/[\/\\]$/ path)
     (values (string-trim-right path #[\\/]) #f #f)
-    (let* ((dir (sys-dirname path))
-           (base (sys-basename path)))
-      (cond ((string-index-right base #\.)
+    (let ((dir (sys-dirname path))
+          (base (sys-basename path)))
+      (cond [(string-index-right base #\.)
              => (lambda (pos)
                   (if (zero? pos)
                     (values dir base #f)  ; '.' at the beginning doesn't delimit extension
                     (values dir
                             (string-take base pos)
-                            (string-drop base (+ pos 1))))))
-            (else
-             (values dir base #f))))))
+                            (string-drop base (+ pos 1)))))]
+            [else (values dir base #f)]))))
 
 (define (relative-path? path)
   (cond-expand
-   (gauche-windows
-    (not (#/^[\/\\]|^[A-Za-z]:/ path)))
-   (else
-    (not (#/^\// path)))))
+   [gauche.os.windows (not (#/^[\/\\]|^[A-Za-z]:/ path))]
+   [else              (not (#/^\// path))]))
 
 (define (absolute-path? path)
   (not (relative-path? path)))
 
 (define (path-separator)
   (cond-expand
-   (gauche-windows #\\)
-   (else #\/)))
+   [gauche.os.windows #\\]
+   [else #\/]))
 
 (define (path-extension path)
   (receive (dir file ext) (decompose-path path) ext))
 
 (define (path-sans-extension path)
-  (cond ((path-extension path)
+  (cond [(path-extension path)
          => (lambda (ext)
               (substring path 0
-                         (- (string-length path) (string-length ext) 1))))
-        (else path)))
+                         (- (string-length path) (string-length ext) 1)))]
+        [else path]))
 
 (define (path-swap-extension path ext)
   (if ext
@@ -293,14 +361,13 @@
     (path-sans-extension path)))
 
 (define (find-file-in-paths name . opts)
-  (let-keywords opts
-      ((paths (cond ((sys-getenv "PATH")
-                     => (cut string-split <> 
-                             (cond-expand
-                              (gauche-windows #\;)
-                              (else #\:))))
-                    (else '())))
-       (pred file-is-executable?))
+  (let-keywords opts ((paths (cond [(sys-getenv "PATH")
+                                    => (cut string-split <> 
+                                            (cond-expand
+                                             [gauche.os.windows #\;]
+                                             [else #\:]))]
+                                   [else '()]))
+                      (pred file-is-executable?))
     (if (absolute-path? name)
       (and (pred name) name)
       (let loop ((paths paths))
@@ -319,11 +386,11 @@
 ;; accepts keyword argument FOLLOW-LINK? 
 (define-syntax define-stat-accessor
   (syntax-rules ()
-    ((_ name slot)
+    [(_ name slot)
      (define (name path . opts)
        (let-keywords opts ((follow-link? #t))
-         (and (sys-access path |F_OK|)
-              (slot-ref (%stat path follow-link?) slot)))))))
+         (and-let* ([s (safe-stat path follow-link?)])
+           (slot-ref s slot))))]))
 
 (define-stat-accessor file-type 'type)
 (define-stat-accessor file-perm 'perm)
@@ -340,13 +407,13 @@
 (define-stat-accessor file-ctime 'ctime)
 
 ;; file permissions
-(define (file-is-readable? path) (sys-access path |R_OK|))
-(define (file-is-writable? path) (sys-access path |W_OK|))
-(define (file-is-executable? path) (sys-access path |X_OK|))
+(define (file-is-readable? path) (sys-access path R_OK))
+(define (file-is-writable? path) (sys-access path W_OK))
+(define (file-is-executable? path) (sys-access path X_OK))
 
 (define (file-is-symlink? path)
-  (and (file-exists? path)
-       (eq? (slot-ref (sys-lstat path) 'type) 'symlink)))
+  (and-let* ([s (safe-stat path #f)])
+    (eq? (slot-ref s 'type) 'symlink)))
 
 ;; compares two files are identical, in the sense that:
 ;;  file-eq?  - two files (or directories) are the same entity. symbolic
@@ -360,7 +427,7 @@
 ;; an alias.  We should call Windows API to check for sure in future).
 
 (cond-expand
- (gauche-windows
+ [gauche.os.windows
   (define (%stat-compare s1 s2 f1 f2)
     (let ((p1 (sys-normalize-pathname f1 :absolute #t :canonicalize #t))
 	  (p2 (sys-normalize-pathname f2 :absolute #t :canonicalize #t)))
@@ -369,8 +436,8 @@
     (%stat-compare #f #f f1 f2))
   (define (file-eqv? f1 f2)
     (%stat-compare #f #f f1 f2))
-  )
- (else
+  ]
+ [else
   (define (%stat-compare s1 s2 f1 f2)
     (and (eqv? (slot-ref s1 'dev) (slot-ref s2 'dev))
          (eqv? (slot-ref s1 'ino) (slot-ref s2 'ino))))
@@ -378,29 +445,27 @@
     (%stat-compare (sys-lstat f1) (sys-lstat f2) f1 f2))
   (define (file-eqv? f1 f2)
     (%stat-compare (sys-stat f1) (sys-stat f2) f1 f2))
-  ))
+  ])
 
 (define (file-equal? f1 f2)
   (let ((s1 (sys-stat f1))
         (s2 (sys-stat f2)))
-    (cond ((%stat-compare s1 s2 f1 f2))
-          ((not (eq? (slot-ref s1 'type) (slot-ref s2 'type)))
-           #f)
-          ((not (= (slot-ref s1 'size) (slot-ref s2 'size)))
-           #f)
-          ((eq? (slot-ref s1 'type) 'directory)
-           (error "directory comparison is not supported yet" s1))
-          (else
+    (cond [(%stat-compare s1 s2 f1 f2)]
+          [(not (eq? (slot-ref s1 'type) (slot-ref s2 'type))) #f]
+          [(not (= (slot-ref s1 'size) (slot-ref s2 'size)))   #f]
+          [(eq? (slot-ref s1 'type) 'directory)
+           (error "directory comparison is not supported yet" s1)]
+          [else
            (call-with-input-file f1
              (lambda (p1)
                (call-with-input-file f2
                  (lambda (p2)
                    (let loop ((b1 (read-block 8192 p1))
                               (b2 (read-block 8192 p2)))
-                     (cond ((eof-object? b1) #t)
-                           ((string=? b1 b2)
-                            (loop (read-block 8192 p1) (read-block 8192 p2)))
-                           (else #f))))))))
+                     (cond [(eof-object? b1) #t]
+                           [(string=? b1 b2)
+                            (loop (read-block 8192 p1) (read-block 8192 p2))]
+                           [else #f]))))))]
           )))
 
 ;; see if two files or directories exist on the same device.
@@ -411,7 +476,7 @@
 ;; <time>, or number.
 (define-syntax define-time-comparer
   (syntax-rules ()
-    ((_ name slot cmp)
+    [(_ name slot cmp)
      (begin
        (define-method name ((a <sys-stat>) (b <sys-stat>))
          (cmp (slot-ref a slot) (slot-ref b slot)))
@@ -429,7 +494,7 @@
          (name (slot-ref a 'second) b))
        (define-method name (a  (b <time>))
          (name a (slot-ref b 'second)))
-       ))))
+       )]))
 
 (define-time-comparer file-mtime=?  'mtime =)
 (define-time-comparer file-mtime<?  'mtime <)
@@ -453,62 +518,92 @@
 ;;; File operation
 
 (define (touch-file pathname)
-  (if (sys-access pathname |F_OK|)
-      (sys-utime pathname)
-      (close-output-port (open-output-file pathname)))
+  (if (sys-access pathname F_OK)
+    (sys-utime pathname)
+    (close-output-port (open-output-file pathname)))
   (values))
 
+(define (touch-files pathnames)
+  (for-each touch-file pathnames))
+
 ;; copy-file
 ;;  if-exists     - :error :supersede :backup #f
+;;  follow-link?  - follow symlinks (default #t)
 ;;  backup-suffix
 ;;  safe
 ;;  keep-timestamp
+;;  keep-mode
 (define (copy-file src dst . opts)
   (let-keywords opts ((if-exists :error)
                       (backsfx :backup-suffix ".orig")
+                      (follow-link? #t)
                       (safe #f)
-                      (keeptime :keep-timestamp #f))
-    (let* ((backfile  (string-append dst backsfx))
-           (times     '())
-           (tmpfile   #f)
-           (inport    #f)
-           (outport   #f))
+                      (keeptime :keep-timestamp #f)
+                      (keepmode :keep-mode #f))
+    (let ((backfile  (string-append dst backsfx))
+          (times     '())
+          (tmpfile   #f)
+          (inport    #f)
+          (outport   #f)
+          (dst-exists #f)
+          (default-perm (logand #o666 (lognot (sys-umask)))))
       (define (rollback)
-        (cond (inport  => close-input-port))
-        (cond (outport => close-output-port))
-        (cond (tmpfile => sys-unlink)))
+        (cond [inport  => close-input-port])
+        (cond [outport => close-output-port])
+        (cond [tmpfile => sys-unlink]))
       (define (commit)
-        (cond (inport  => close-input-port))
-        (cond (outport => close-output-port))
-        (when tmpfile
-          (when (eq? if-exists :backup) (sys-rename dst backfile))
-          (sys-rename tmpfile dst))
+        (cond [inport  => close-input-port])
+        (cond [outport => close-output-port])
+        (cond [tmpfile
+               (safe-chmod tmpfile (if keepmode (file-perm src) default-perm))
+               (when (eq? if-exists :backup) (sys-rename dst backfile))
+               (sys-rename tmpfile dst)]
+              [keepmode         (safe-chmod dst (file-perm src))]
+              [(not dst-exists) (safe-chmod dst default-perm)])
         (unless (null? times) (apply sys-utime dst times)))
+      (define (safe-chmod path mode)
+        (and (not (file-is-symlink? path)) (sys-chmod path mode)))
       (define (open-destination)
         (if safe
           (cond
-           ((and (eq? if-exists :error) (file-exists? dst))
-            (error "destination file exists" dst))
-           ((and (not if-exists) (file-exists? dst)) #f)
-           (else
-            (set!-values (outport tmpfile) (sys-mkstemp dst)) #t))
+           [(and (eq? if-exists :error) (file-exists? dst))
+            (error "destination file exists" dst)]
+           [(and (not if-exists) (file-exists? dst)) #f]
+           [else
+            (set!-values (outport tmpfile) (sys-mkstemp dst)) #t])
           (cond
-           ((eq? if-exists :error)
-            (set! outport (open-output-file dst :if-exists :error)) #t)
-           ((not if-exists)
-            (set! outport (open-output-file dst :if-exists #f)) outport)
-           ((eq? if-exists :backup)
+           [(eq? if-exists :error)
+            (set! outport (open-output-file dst :if-exists :error)) #t]
+           [(not if-exists)
+            (set! outport (open-output-file dst :if-exists #f)) outport]
+           [(eq? if-exists :backup)
             (when (file-exists? dst) (sys-rename dst backfile))
-            (set! outport (open-output-file dst)) #t)
-           (else
-            (set! outport (open-output-file dst :if-exists :supersede)) #t))
+            (set! outport (open-output-file dst)) #t]
+           [else
+            (set! outport (open-output-file dst :if-exists :supersede)) #t])
           ))
+      (define (get-times stat)
+        (map (cut slot-ref stat <>) '(atime mtime)))
+      (define (do-symlink)
+        (define (doit)
+          (cond-expand
+           [gauche.sys.symlink (sys-symlink (sys-readlink src) dst)]
+           [else #f]));NB: if system doesn't support symlink, we can't be here.
+        (when keeptime (set! times (get-times (sys-lstat src))))
+        (if (file-exists? dst)
+          (case if-exists
+            [(:error) (error <system-error> :errno EEXIST
+                             "destination file exists" dst)]
+            [(#f) #f]
+            [(:backup) (sys-rename dst backfile) (doit)]
+            [else      (sys-unlink dst) (doit)])
+          (doit))
+        (commit))
       (define (do-copy)
         (guard (e (else (rollback) (raise e)))
           (set! inport (open-input-file src))
-          (when keeptime
-            (set! times (let1 stat (sys-fstat inport)
-                          (map (cut slot-ref stat <>) '(atime mtime)))))
+          (when keeptime (set! times (get-times (sys-fstat inport))))
+          (set! dst-exists (file-exists? dst))
           (begin0
            (and (open-destination)
                 (copy-port inport outport :unit 65536)
@@ -518,9 +613,12 @@
       ;; body of copy-file
       (unless (memq if-exists '(#f :error :supersede :backup))
         (error "argument for :if-exists must be either :error, :supersede, :backup or #f, but got" if-exists))
-      (when (and (file-exists? src) (file-exists? dst) (file-eqv? src dst))
+      (when (and (file-exists? src) (file-exists? dst)
+                 ((if follow-link? file-eqv? file-eq?) src dst))
         (errorf "source ~s and destination ~s are the same file" src dst))
-      (do-copy)
+      (if (and (not follow-link?) (file-is-symlink? src))
+        (do-symlink)
+        (do-copy))
       )))
   
 ;; move-file
@@ -545,7 +643,7 @@
     (define (do-copying)
       (and (copy-file src dst :if-exists if-exists
                       :backup-suffix backsfx 
-                      :safe #t :keep-timestamp #t)
+                      :safe #t :keep-timestamp #t :keep-mode #t)
            (begin (sys-unlink src) #t)))
     
     ;; body of move-file
@@ -559,26 +657,56 @@
       (unless (file-exists? dstdir)
         (errorf "can't move to ~s: path does not exist" dst))
       (if (file-device=? src dstdir)
-          (do-rename)
-          (do-copying)))
+        (do-rename)
+        (do-copying)))
     ))
 
-;; copy-directory
-;; move-directory
+;; copy-files & move files
+;; NB: need to think more about argument order.  Should it be
+;;     (copy-files file1 file2 ... destdir)  ;; cp(1) style
+;;   or
+;;     (copy-files destdir file1 file2 ...)
+;; And how shall we specify keyword args to pass to copy-file?
 
+; (define (%multifile-cpmv op)
+;   (lambda (files dst . opts)
+;     (unless (file-exists? dst)
+;       (error "destination does not exist" dst))
+;     (unless (file-is-directory? dst)
+;       (error "destination is not a directory" dst))
+;     (dolist (f files)
+;       (apply op f (build-path dst (sys-basename f)) opts))))
+
+; (define copy-files (%multifile-cpmv copy-file))
+; (define move-files (%multifile-cpmv move-file))
+
+;; remove files
+(define (remove-files . paths)
+  (dolist (p paths)
+    (cond [(list? p) (apply remove-files p)] ; for the convenience
+          [(file-is-directory? p) (remove-directory* p)]
+          [else (sys-unlink p)])))
+
+(define delete-files remove-files)
+  
 ;; file->string, file->list, file->string-list, file->sexp-list
 ;; shortcuts of port->string etc.
+;; NB: call-with-input-file may pass #f to the proc if :if-does-not-exist #f
+;; is given.  We return #f in such case.
+
+(define (%maybe proc)
+  (lambda (maybe-port) (and maybe-port (proc maybe-port))))
 
 (define (file->string file . opts)
-  (apply call-with-input-file file port->string opts))
+  (apply call-with-input-file file (%maybe port->string) opts))
 
 (define (file->list reader file . opts)
-  (apply call-with-input-file file (cut port->list reader <>) opts))
+  (apply call-with-input-file file (%maybe (cut port->list reader <>)) opts))
 
 (define (file->string-list file . opts)
-  (apply call-with-input-file file (cut port->list read-line <>) opts))
+  (apply call-with-input-file file (%maybe (cut port->list read-line <>)) opts))
 
 (define (file->sexp-list file . opts)
-  (apply call-with-input-file file (cut port->list read <>) opts))
+  (apply call-with-input-file file (%maybe (cut port->list read <>)) opts))
 
 (provide "file/util")
