@@ -1,7 +1,7 @@
 /*
  * bignum.c - multiple precision exact integer arithmetic
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: bignum.c,v 1.63 2007/03/02 07:39:12 shirok Exp $
+ *  $Id: bignum.c,v 1.70 2008-05-10 13:36:16 shirok Exp $
  */
 
 /* Bignum library.  Not optimized well yet---I think bignum performance
@@ -71,6 +71,10 @@ char *alloca ();
 # if HAVE_ALLOCA_H
 #  include <alloca.h>
 # endif
+# if HAVE_MALLOC_H
+/* MinGW helds alloca() in "malloc.h" instead of "alloca.h" */
+#  include <malloc.h>
+# endif
 #endif
 
 #include <stdlib.h>
@@ -79,6 +83,8 @@ char *alloca ();
 #define LIBGAUCHE_BODY
 #include "gauche.h"
 #include "gauche/arith.h"
+#include "gauche/bits.h"
+#include "gauche/bignum.h"
 
 #undef min
 #define min(x, y)   (((x) < (y))? (x) : (y))
@@ -98,7 +104,7 @@ static ScmBignum *bignum_2scmpl(ScmBignum *br);
  */
 static ScmBignum *bignum_clear(ScmBignum *b)
 {
-    int i;
+    u_int i;
     for (i=0; i<b->size; i++) b->values[i] = 0;
     return b;
 }
@@ -109,10 +115,7 @@ static ScmBignum *make_bignum(int size)
 {
     ScmBignum *b;
     if (size < 0) Scm_Error("invalid bignum size (internal error): %d", size);
-    if (size > SCM_BIGNUM_MAX_DIGITS) {
-        Scm_Error("too large bignum (> 2^%d-1)",
-                  SCM_BIGNUM_MAX_DIGITS*(SIZEOF_LONG*CHAR_BIT));
-    }
+    if (size > (int)SCM_BIGNUM_MAX_DIGITS) Scm_Error("too large bignum");
     b = SCM_NEW_ATOMIC2(ScmBignum*, BIGNUM_SIZE(size));
     SCM_SET_CLASS(b, SCM_CLASS_INTEGER);
     b->size = size;
@@ -199,6 +202,7 @@ ScmObj Scm_MakeBignumFromDouble(double val)
         Scm_Error("can't convert %lf to an integer", val);
     }
     b = Scm_Ash(mantissa, exponent);
+    if (sign < 0) b = Scm_Negate(b);
     /* always returns bignum */
     if (SCM_INTP(b)) {
         return Scm_MakeBignumFromSI(SCM_INT_VALUE(b));
@@ -209,7 +213,7 @@ ScmObj Scm_MakeBignumFromDouble(double val)
 
 ScmObj Scm_BignumCopy(ScmBignum *b)
 {
-    int i;
+    u_int i;
     ScmBignum *c = make_bignum(b->size);
     c->sign = b->sign;
     for (i=0; i<b->size; i++) c->values[i] = b->values[i];
@@ -236,7 +240,7 @@ ScmObj Scm_NormalizeBignum(ScmBignum *b)
             return SCM_MAKE_INT(b->values[0]);
         }
         if (b->sign < 0 && b->values[0] <= (u_long)-SCM_SMALL_INT_MIN) {
-            return SCM_MAKE_INT(-b->values[0]);
+            return SCM_MAKE_INT(-((signed long)b->values[0]));
         }
     }
     b->size = size;
@@ -341,7 +345,7 @@ ScmInt64 Scm_BignumToSI64(ScmBignum *b, int clamp, int *oor)
             if (!(clamp&SCM_CLAMP_LO)) goto err;
             SCM_SET_INT64_MIN(r);
         } else {
-            r = -(((int64_t)b->values[1] << 32) + (uint64_t)b->values[0]);
+            r = -(int64_t)(((int64_t)b->values[1] << 32) + (uint64_t)b->values[0]);
         }
     }
     return r;
@@ -400,23 +404,84 @@ ScmUInt64 Scm_BignumToUI64(ScmBignum *b, int clamp, int *oor)
 #endif /* SIZEOF_LONG == 4 */
 
 
-/* b must be normalized */
+/* Converts a bignum b to a double.  b must be normalized.
+   We don't rely on double arithmetic, for it may result
+   an error in the LSB from multiple roundings.  Instead we
+   extract bits directly from the bignum values array.
+   (We use ScmBits API by casting b->values to ScmBits*).
+ */
 double Scm_BignumToDouble(ScmBignum *b)
 {
-    double r;
-    switch (b->size) {
-    case 0: r = 0.0; break;
-    case 1: r = (double)b->values[0]; break;
-    case 2:
-        r = ldexp((double)b->values[1], WORD_BITS) + (double)b->values[0];
-        break;
-    default:
-        r = ldexp((double)b->values[b->size-1], WORD_BITS*(b->size-1))
-            + ldexp((double)b->values[b->size-2], WORD_BITS*(b->size-2))
-            + ldexp((double)b->values[b->size-3], WORD_BITS*(b->size-3));
-        break;
+    ScmIEEEDouble dd;
+    ScmBits *bits = (ScmBits*)b->values;
+    ScmBits dst[2];
+    int maxbit, exponent;
+
+    /* first, filter out a special case. */
+    if (b->size == 0) return 0.0;
+
+    maxbit = Scm_BitsHighest1(bits, 0, b->size*WORD_BITS);
+    exponent = maxbit+1023;
+#if SIZEOF_LONG >= 8
+    SCM_ASSERT(maxbit >= 54);   /* because b is normalized */
+    dst[0] = 0;
+    Scm_BitsCopyX(dst, 0, bits, maxbit-52, maxbit);
+    /* Rounding.  We have to round up if maxbit-53 == 1, EXCEPT
+       the special case where maxbit-52==0, maxbit-53==1, and all
+       bits below are 0 (round-to-even rule) */
+    if (SCM_BITS_TEST(bits, maxbit-53)
+        && (dst[0]&1 == 1
+            || (Scm_BitsCount1(bits, 0, maxbit-53) > 0))) {
+        dst[0]++;
+        if (dst[0] >= (1UL<<52)) {
+            /* Overflow.  We mask the hidden bit, then shift. */
+            dst[0] &= ~(1UL<<52);
+            dst[0] >>= 1;
+            exponent++;
+        }
     }
-    return (b->sign < 0)? -r : r;
+    if (exponent > 2046) {
+        dd.components.mant = 0;
+        dd.components.exp = 2047;
+    } else {
+        dd.components.mant = dst[0];
+        dd.components.exp  = exponent;
+    }
+    dd.components.sign = (b->sign < 0);
+#else  /*SIZEOF_LONG == 4 */
+    dst[0] = dst[1] = 0;
+    if (maxbit < 53) {
+        Scm_BitsCopyX(dst, 52-maxbit, bits, 0, maxbit);
+    } else {
+        Scm_BitsCopyX(dst, 0, bits, maxbit-52, maxbit);
+        /* Rounding.  See the above comment. */
+        if (SCM_BITS_TEST(bits, maxbit-53)
+            && ((dst[0]&1) == 1
+                || (maxbit > 53 && Scm_BitsCount1(bits, 0, maxbit-53) > 0))) {
+            u_long inc = dst[0] + 1;
+            if (inc < dst[0]) dst[1]++;
+            dst[0] = inc;
+            if (dst[1] >= (1UL<<(52-32))) {
+                /* Overflow.  We mask the hidden bit, then shift. */
+                dst[1] &= ~(1UL<<(52-32));
+                dst[0] = (dst[0] >> 1) | (dst[1]&1 << 31);
+                dst[1] >>= 1;
+                exponent++;
+            }
+        }
+    }
+    if (exponent > 2046) {
+        dd.components.mant0 = 0;
+        dd.components.mant1 = 0;
+        dd.components.exp = 2047;
+    } else {
+        dd.components.mant0 = dst[1];
+        dd.components.mant1 = dst[0];
+        dd.components.exp = exponent;
+    }
+    dd.components.sign = (b->sign < 0);
+#endif /*SIZEOF_LONG==4*/
+    return dd.d;
 }
 
 /* return -b, normalized */
@@ -441,7 +506,7 @@ int Scm_BignumCmp(ScmBignum *bx, ScmBignum *by)
     if (bx->size < by->size) return (bx->sign > 0) ? -1 : 1;
     if (bx->size > by->size) return (bx->sign > 0) ? 1 : -1;
 
-    for (i=bx->size-1; i>=0; i--) {
+    for (i=(int)bx->size-1; i>=0; i--) {
         if (bx->values[i] < by->values[i]) return (bx->sign > 0) ? -1 : 1;
         if (bx->values[i] > by->values[i]) return (bx->sign > 0) ? 1 : -1;
     }
@@ -455,7 +520,7 @@ int Scm_BignumAbsCmp(ScmBignum *bx, ScmBignum *by)
     
     if (bx->size < by->size) return -1;
     if (bx->size > by->size) return 1;
-    for (i=bx->size-1; i>=0; i--) {
+    for (i=(int)bx->size-1; i>=0; i--) {
         if (bx->values[i] < by->values[i]) return -1;
         if (bx->values[i] > by->values[i]) return 1;
     }
@@ -471,10 +536,12 @@ int Scm_BignumAbsCmp(ScmBignum *bx, ScmBignum *by)
    doing actual bignum addition. */
 int Scm_BignumCmp3U(ScmBignum *bx, ScmBignum *off, ScmBignum *by)
 {
-    int xsize = SCM_BIGNUM_SIZE(bx), ysize = SCM_BIGNUM_SIZE(by);
-    int osize = SCM_BIGNUM_SIZE(off);
-    int tsize, i;
+    u_int xsize = SCM_BIGNUM_SIZE(bx), ysize = SCM_BIGNUM_SIZE(by);
+    u_int osize = SCM_BIGNUM_SIZE(off);
+    u_int tsize;
+    int i;
     ScmBignum *br;
+
     if (xsize > ysize) return 1;
     if (xsize < ysize) {
         if (osize < ysize && by->values[ysize-1] > 1) {
@@ -506,8 +573,8 @@ int Scm_BignumCmp3U(ScmBignum *bx, ScmBignum *off, ScmBignum *by)
     bignum_add_int(br, bx, off);
     
     if (br->size < by->size) return -1;
-    for (i=br->size-1; i>=0; i--) {
-        if (i >= by->size) {
+    for (i=(int)br->size-1; i>=0; i--) {
+        if (i >= (int)by->size) {
             if (br->values[i]) return 1;
             continue;
         }
@@ -653,7 +720,7 @@ static ScmBignum *bignum_sub(ScmBignum *bx, ScmBignum *by)
 static ScmBignum *bignum_add_si(ScmBignum *bx, long y)
 {
     long c;
-    int i, rsize = bx->size+1;
+    u_int i, rsize = bx->size+1;
     u_long yabs = ((y < 0)? -y : y);
     int ysign = ((y < 0)? -1 : 1);
     ScmBignum *br;
@@ -705,21 +772,21 @@ ScmObj Scm_BignumSubSI(ScmBignum *bx, long y)
    has enough size to hold the result.  br and bx can be the same object. */
 static ScmBignum *bignum_rshift(ScmBignum *br, ScmBignum *bx, int amount)
 {
-    int nwords = amount / WORD_BITS;
-    int nbits = amount % WORD_BITS;
+    u_int nwords = amount / WORD_BITS;
+    u_int nbits = amount % WORD_BITS;
     int i;
     
     if (bx->size <= nwords) {
         br->size = 0; br->values[0] = 0;
     } else if (nbits == 0) {
-        for (i = nwords; i < bx->size; i++) {
+        for (i = (int)nwords; i < (int)bx->size; i++) {
             br->values[i-nwords] = bx->values[i];
         }
         br->size = bx->size - nwords;
         br->sign = bx->sign;
     } else {
         u_long x;
-        for (i = nwords; i < bx->size-1; i++) {
+        for (i = (int)nwords; i < (int)bx->size-1; i++) {
             x = (bx->values[i+1]<<(WORD_BITS-nbits))|(bx->values[i]>>nbits);
             br->values[i-nwords] = x;
         }
@@ -741,8 +808,8 @@ static ScmBignum *bignum_lshift(ScmBignum *br, ScmBignum *bx, int amount)
     nbits = amount % WORD_BITS;
     if (nbits == 0) {
         /* short path */
-        for (i = bx->size-1; i>=0; i--) {
-            if (br->size > i+nwords) br->values[i+nwords] = bx->values[i];
+        for (i = (int)bx->size-1; i>=0; i--) {
+            if ((int)br->size > i+nwords) br->values[i+nwords] = bx->values[i];
         }
         for (i = nwords-1; i>=0; i--) br->values[i] = 0;
     } else {
@@ -750,9 +817,9 @@ static ScmBignum *bignum_lshift(ScmBignum *br, ScmBignum *bx, int amount)
             br->values[bx->size+nwords] =
                 bx->values[bx->size-1]>>(WORD_BITS-nbits);
         }
-        for (i = bx->size-1; i > 0; i--) {
+        for (i = (int)bx->size-1; i > 0; i--) {
             x = (bx->values[i]<<nbits)|(bx->values[i-1]>>(WORD_BITS-nbits));
-            if (br->size > i+nwords) br->values[i+nwords] = x;
+            if ((int)br->size > i+nwords) br->values[i+nwords] = x;
         }
         br->values[nwords] = bx->values[0]<<nbits;
         for (i = nwords-1; i>=0; i--) br->values[i] = 0;
@@ -772,7 +839,7 @@ static ScmBignum *bignum_mul_word(ScmBignum *br, ScmBignum *bx,
                                   u_long y, int off)
 {
     u_long hi, lo, x, r0, r1, c;
-    int i,j;
+    u_int i,j;
     
     for (i=0; i<bx->size; i++) {
         x = bx->values[i];
@@ -799,7 +866,7 @@ static ScmBignum *bignum_mul_word(ScmBignum *br, ScmBignum *bx,
 /* returns bx * by.  not normalized */
 static ScmBignum *bignum_mul(ScmBignum *bx, ScmBignum *by)
 {
-    int i;
+    u_int i;
     ScmBignum *br = make_bignum(bx->size + by->size);
     for (i=0; i<by->size; i++) {
         bignum_mul_word(br, bx, by->values[i], i);
@@ -1004,7 +1071,9 @@ ScmObj Scm_BignumDivSI(ScmBignum *dividend, long divisor, long *remainder)
         br = bignum_gdiv(dividend, bv, q);
         rr = br->values[0];
     }
-    if (remainder) *remainder = (dividend->sign < 0)? -rr : rr;
+    if (remainder) {
+        *remainder = ((dividend->sign < 0)? -(signed long)rr : (signed long)rr);
+    }
     q->sign = dividend->sign * d_sign;
     return Scm_NormalizeBignum(q);
 }
@@ -1176,6 +1245,21 @@ ScmObj Scm_BignumLogXor(ScmBignum *x, ScmBignum *y)
     return Scm_LogAnd(xory, Scm_LogNot(xandy));
 }
 
+int Scm_BignumLogCount(ScmBignum *b)
+{
+    ScmBits *bits;
+    ScmBignum *z = (SCM_BIGNUM_SIGN(b)>0)? b : SCM_BIGNUM(Scm_BignumComplement(b));
+    int size = SCM_BIGNUM_SIZE(z) * SCM_WORD_BITS;
+
+    bits = (ScmBits*)z->values;
+    if (b->sign > 0) {
+        return Scm_BitsCount1(bits, 0, size);
+    } else {
+        return Scm_BitsCount0(bits, 0, size);
+    }
+}
+
+
 /*-----------------------------------------------------------------------
  * Printing
  */
@@ -1208,7 +1292,7 @@ int Scm_DumpBignum(ScmBignum *b, ScmPort *out)
     int i;
     Scm_Printf(out, "#<bignum ");
     if (b->sign < 0) SCM_PUTC('-', out);
-    for (i=b->size-1; i>=0; i--) {
+    for (i=(int)b->size-1; i>=0; i--) {
         Scm_Printf(out, "%08x ", b->values[i]);
     }
     SCM_PUTC('>', out);
@@ -1238,7 +1322,7 @@ ScmBignum *Scm_MakeBignumWithSize(int size, u_long init)
 ScmBignum *Scm_BignumAccMultAddUI(ScmBignum *acc, u_long coef, u_long c)
 {
     ScmBignum *r;
-    int rsize = acc->size + 1, i;
+    u_int rsize = acc->size + 1, i;
     ALLOC_TEMP_BIGNUM(r, rsize);
     r->values[0] = c;
     bignum_mul_word(r, acc, coef, 0);
