@@ -1,7 +1,7 @@
 /*
  * char.c - character and character set operations
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: char.c,v 1.53 2007/04/18 02:24:09 shirok Exp $
+ *  $Id: char.c,v 1.62 2008-05-10 13:36:17 shirok Exp $
  */
 
 #include <ctype.h>
@@ -110,9 +110,20 @@ ScmChar Scm_IntToDigit(int n, int radix)
 /*
  * Convert UCS4 code <-> character
  * If the native encoding is not utf-8, gauche.charconv module is loaded.
+ * and these pointers are filled.
  */
-ScmChar (*Scm_UcsToCharHook)(int ucs4) = NULL;  /* filled by ext/charconv */
-int (*Scm_CharToUcsHook)(ScmChar ch) = NULL;    /* filled by ext/charconv */
+static ScmChar (*ucs2char_hook)(int ucs4) = NULL;
+static int     (*char2ucs_hook)(ScmChar ch) = NULL;
+
+/* called by gauche.charconv */
+void Scm__InstallCharconvHooks(ScmChar (*u2c)(int), int (*c2u)(ScmChar))
+{
+    ucs2char_hook = u2c;
+    char2ucs_hook = c2u;
+}
+
+ScmChar (*Scm_UcsToCharHook)(int ucs4) = NULL;
+int (*Scm_CharToUcsHook)(ScmChar ch) = NULL;
 
 ScmChar Scm_UcsToChar(int n)
 {
@@ -121,16 +132,16 @@ ScmChar Scm_UcsToChar(int n)
     return (ScmChar)n;
 #elif defined(GAUCHE_CHAR_ENCODING_EUC_JP) || defined(GAUCHE_CHAR_ENCODING_SJIS)
     if (n < 0x80) return (ScmChar)n; /*ASCII range*/
-    if (Scm_UcsToCharHook == NULL) {
+    if (ucs2char_hook == NULL) {
         /* NB: we don't need mutex here, for the loading of gauche.charconv
            is serialized in Scm_Require. */
         Scm_Require(SCM_MAKE_STR("gauche/charconv"),
                     SCM_LOAD_PROPAGATE_ERROR, NULL);
-        if (Scm_UcsToCharHook == NULL) {
+        if (ucs2char_hook == NULL) {
             Scm_Error("couldn't autoload gauche.charconv");
         }
     }
-    return Scm_UcsToCharHook(n);
+    return ucs2char_hook(n);
 #else
     if (n < 0x100) return (ScmChar)n; /* ISO8859-1 */
     else return SCM_CHAR_INVALID;
@@ -144,16 +155,16 @@ int Scm_CharToUcs(ScmChar ch)
     return (int)ch;
 #elif defined(GAUCHE_CHAR_ENCODING_EUC_JP) || defined(GAUCHE_CHAR_ENCODING_SJIS)
     if (ch < 0x80) return (int)ch; /*ASCII range*/
-    if (Scm_CharToUcsHook == NULL) {
+    if (char2ucs_hook == NULL) {
         /* NB: we don't need mutex here, for the loading of gauche.charconv
            is serialized in Scm_Require. */
         Scm_Require(SCM_MAKE_STR("gauche/charconv"),
                     SCM_LOAD_PROPAGATE_ERROR, NULL);
-        if (Scm_CharToUcsHook == NULL) {
+        if (char2ucs_hook == NULL) {
             Scm_Error("couldn't autoload gauche.charconv");
         }
     }
-    return Scm_CharToUcsHook(ch);
+    return char2ucs_hook(ch);
 #else
     return (int)ch;             /* ISO8859-1 */
 #endif /*!GAUCHE_CHAR_ENCODING_UTF_8*/
@@ -180,11 +191,18 @@ SCM_DEFINE_BUILTIN_CLASS(Scm_CharSetClass,
 /*----------------------------------------------------------------------
  * Printer
  */
+
 static void charset_print_ch(ScmPort *out, ScmChar ch)
 {
-    if (ch < 0x20 || ch == 0x7f) {
+    if (ch == '[' || ch == ']' || ch == '-') {
+        Scm_Printf(out, "\\%c", ch);
+    } else if (ch < 0x20 || ch == 0x7f) {
         Scm_Printf(out, "\\x%02x", ch);
     } else {
+        /* This is a bit tricky.  For maximum portability and read/write
+           invariance, unicode escape would be the best.  However, we also
+           want readability.  At least raw byte sequence here should work
+           as far as the external encoding of reader and writer match. */
         Scm_Putc(ch, out);
     }
 }
@@ -192,12 +210,12 @@ static void charset_print_ch(ScmPort *out, ScmChar ch)
 static void charset_print(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
 {
     int prev, code;
-    ScmCharSet *cs = SCM_CHARSET(obj);
+    ScmCharSet *cs = SCM_CHAR_SET(obj);
     ScmTreeIter iter;
     ScmDictEntry *e;
 
     Scm_Printf(out, "#[");
-    for (prev = -1, code = 0; code < SCM_CHARSET_SMALL_CHARS; code++) {
+    for (prev = -1, code = 0; code < SCM_CHAR_SET_SMALL_CHARS; code++) {
         if (MASK_ISSET(cs, code) && prev < 0) {
             charset_print_ch(out, code);
             prev = code;
@@ -240,9 +258,8 @@ static int cmp(ScmTreeCore *tc, intptr_t a, intptr_t b)
 static ScmCharSet *make_charset(void)
 {
     ScmCharSet *cs = SCM_NEW(ScmCharSet);
-    int i;
     SCM_SET_CLASS(cs, SCM_CLASS_CHARSET);
-    Scm_BitsFill(cs->small, 0, SCM_CHARSET_SMALL_CHARS, 0);
+    Scm_BitsFill(cs->small, 0, SCM_CHAR_SET_SMALL_CHARS, 0);
     Scm_TreeCoreInit(&cs->large, cmp, NULL);
     return cs;
 }
@@ -255,7 +272,7 @@ ScmObj Scm_MakeEmptyCharSet(void)
 ScmObj Scm_CharSetCopy(ScmCharSet *src)
 {
     ScmCharSet *dst = make_charset();
-    Scm_BitsCopyX(dst->small, 0, src->small, 0, SCM_CHARSET_SMALL_CHARS);
+    Scm_BitsCopyX(dst->small, 0, src->small, 0, SCM_CHAR_SET_SMALL_CHARS);
     Scm_TreeCoreCopy(&dst->large, &src->large);
     return SCM_OBJ(dst);
 }
@@ -321,8 +338,8 @@ ScmChar Scm_ReadXdigitsFromPort(ScmPort *port, int ndigits,
  */
 static int charset_compare(ScmObj x, ScmObj y, int equalp)
 {
-    ScmCharSet *xx = SCM_CHARSET(x);
-    ScmCharSet *yy = SCM_CHARSET(y);
+    ScmCharSet *xx = SCM_CHAR_SET(x);
+    ScmCharSet *yy = SCM_CHAR_SET(y);
     
     if (equalp) {
         return (Scm_CharSetEq(xx, yy)? 0 : 1);
@@ -337,7 +354,7 @@ static int charset_compare(ScmObj x, ScmObj y, int equalp)
 
 int Scm_CharSetEq(ScmCharSet *x, ScmCharSet *y)
 {
-    if (!Scm_BitsEqual(x->small, y->small, 0, SCM_CHARSET_SMALL_CHARS))
+    if (!Scm_BitsEqual(x->small, y->small, 0, SCM_CHAR_SET_SMALL_CHARS))
         return FALSE;
     if (!Scm_TreeCoreEq(&x->large, &y->large))
         return FALSE;
@@ -349,7 +366,7 @@ int Scm_CharSetLE(ScmCharSet *x, ScmCharSet *y)
 {
     ScmTreeIter xi;
     ScmDictEntry *xe, *ye, *yl, *yh;
-    if (!Scm_BitsIncludes(y->small, x->small, 0, SCM_CHARSET_SMALL_CHARS))
+    if (!Scm_BitsIncludes(y->small, x->small, 0, SCM_CHAR_SET_SMALL_CHARS))
         return FALSE;
     /* For each range of X, check if it is fully contained by a range of Y.
      *
@@ -381,16 +398,15 @@ int Scm_CharSetLE(ScmCharSet *x, ScmCharSet *y)
 ScmObj Scm_CharSetAddRange(ScmCharSet *cs, ScmChar from, ScmChar to)
 {
     ScmDictEntry *e, *lo, *hi;
-    int lbound;
     
     if (to < from) return SCM_OBJ(cs);
-    if (from < SCM_CHARSET_SMALL_CHARS) {
-        if (to < SCM_CHARSET_SMALL_CHARS) {
+    if (from < SCM_CHAR_SET_SMALL_CHARS) {
+        if (to < SCM_CHAR_SET_SMALL_CHARS) {
             Scm_BitsFill(cs->small, (int)from, (int)to+1, TRUE);
             return SCM_OBJ(cs);
         }
-        Scm_BitsFill(cs->small, (int)from, SCM_CHARSET_SMALL_CHARS, TRUE);
-        from = SCM_CHARSET_SMALL_CHARS;
+        Scm_BitsFill(cs->small, (int)from, SCM_CHAR_SET_SMALL_CHARS, TRUE);
+        from = SCM_CHAR_SET_SMALL_CHARS;
     }
 
     /* Let e have the lower bound. */
@@ -424,29 +440,28 @@ ScmObj Scm_CharSetAddRange(ScmCharSet *cs, ScmChar from, ScmChar to)
 
 ScmObj Scm_CharSetAdd(ScmCharSet *dst, ScmCharSet *src)
 {
-    int i;
     ScmTreeIter iter;
     ScmDictEntry *e;
 
     if (dst == src) return SCM_OBJ(dst);  /* precaution */
     
     Scm_BitsOperate(dst->small, SCM_BIT_IOR, dst->small, src->small,
-                    0, SCM_CHARSET_SMALL_CHARS);
+                    0, SCM_CHAR_SET_SMALL_CHARS);
     Scm_TreeIterInit(&iter, &src->large, NULL);
     while ((e = Scm_TreeIterNext(&iter)) != NULL) {
-        Scm_CharSetAddRange(dst, e->key, e->value);
+        Scm_CharSetAddRange(dst, SCM_CHAR(e->key), SCM_CHAR(e->value));
     }
     return SCM_OBJ(dst);
 }
 
 ScmObj Scm_CharSetComplement(ScmCharSet *cs)
 {
-    int i, last;
+    int last;
     ScmDictEntry *e, *n;
 
     Scm_BitsOperate(cs->small, SCM_BIT_NOT1, cs->small, NULL,
-                    0, SCM_CHARSET_SMALL_CHARS);
-    last = SCM_CHARSET_SMALL_CHARS;
+                    0, SCM_CHAR_SET_SMALL_CHARS);
+    last = SCM_CHAR_SET_SMALL_CHARS;
     /* we can't use treeiter, since we modify the tree while traversing it. */
     while ((e = Scm_TreeCoreNextEntry(&cs->large, last)) != NULL) {
         Scm_TreeCoreSearch(&cs->large, e->key, SCM_DICT_DELETE);
@@ -454,7 +469,7 @@ ScmObj Scm_CharSetComplement(ScmCharSet *cs)
             n = Scm_TreeCoreSearch(&cs->large, last, SCM_DICT_CREATE);
             n->value = e->key-1;
         }
-        last = e->value+1;
+        last = (int)e->value+1;
     }
     if (last < SCM_CHAR_MAX) {
         n = Scm_TreeCoreSearch(&cs->large, last, SCM_DICT_CREATE);
@@ -484,7 +499,7 @@ ScmObj Scm_CharSetCaseFold(ScmCharSet *cs)
 int Scm_CharSetContains(ScmCharSet *cs, ScmChar c)
 {
     if (c < 0) return FALSE;
-    if (c < SCM_CHARSET_SMALL_CHARS) return MASK_ISSET(cs, c);
+    if (c < SCM_CHAR_SET_SMALL_CHARS) return MASK_ISSET(cs, c);
     else {
         ScmDictEntry *e, *l, *h;
         e = Scm_TreeCoreClosestEntries(&cs->large, (int)c, &l, &h);
@@ -505,7 +520,7 @@ ScmObj Scm_CharSetRanges(ScmCharSet *cs)
     ScmTreeIter iter;
     ScmDictEntry *e;
 
-    for (ind = 0; ind < SCM_CHARSET_SMALL_CHARS; ind++) {
+    for (ind = 0; ind < SCM_CHAR_SET_SMALL_CHARS; ind++) {
         int bit = MASK_ISSET(cs, ind);
         if (!prev && bit) begin = ind;
         if (prev && !bit) {
@@ -530,9 +545,8 @@ ScmObj Scm_CharSetRanges(ScmCharSet *cs)
 void Scm_CharSetDump(ScmCharSet *cs, ScmPort *port)
 {
     int i;
-    struct ScmCharSetRange *r;
     Scm_Printf(port, "CharSet %p\nmask:", cs);
-    for (i=0; i<SCM_BITS_NUM_WORDS(SCM_CHARSET_SMALL_CHARS); i++) {
+    for (i=0; i<SCM_BITS_NUM_WORDS(SCM_CHAR_SET_SMALL_CHARS); i++) {
 #if SIZEOF_LONG == 4
         Scm_Printf(port, "[%08lx]", cs->small[i]);
 #else
@@ -594,7 +608,7 @@ static ScmChar read_charset_xdigits(ScmPort *port, int ndigs, int key)
    whether complement character (caret in the beginning) appeared or not.
    In that case, the returned charset is not complemented. */
 
-static ScmObj read_predef_charset(ScmPort*, ScmObj*);
+static ScmObj read_predef_charset(ScmPort*, ScmObj*, int);
 
 ScmObj Scm_CharSetRead(ScmPort *input, int *complement_p,
                        int error_p, int bracket_syntax)
@@ -603,7 +617,7 @@ ScmObj Scm_CharSetRead(ScmPort *input, int *complement_p,
 #define CARET_BEGIN 2
     int begin = REAL_BEGIN, complement = FALSE;
     int lastchar = -1, inrange = FALSE, moreset_complement = FALSE;
-    ScmCharSet *set = SCM_CHARSET(Scm_MakeEmptyCharSet());
+    ScmCharSet *set = SCM_CHAR_SET(Scm_MakeEmptyCharSet());
     ScmObj moreset;
     ScmObj chars = SCM_NIL;
     ScmChar ch = 0;
@@ -663,40 +677,40 @@ ScmObj Scm_CharSetRead(ScmPort *input, int *complement_p,
                 goto ordchar;
             case 'd':
                 moreset_complement = FALSE;
-                moreset = Scm_GetStandardCharSet(SCM_CHARSET_DIGIT);
+                moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_DIGIT);
                 break;
             case 'D':
                 moreset_complement = TRUE;
-                moreset = Scm_GetStandardCharSet(SCM_CHARSET_DIGIT);
+                moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_DIGIT);
                 break;
             case 's':
                 moreset_complement = FALSE;
-                moreset = Scm_GetStandardCharSet(SCM_CHARSET_SPACE);
+                moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_SPACE);
                 break;
             case 'S':
                 moreset_complement = TRUE;
-                moreset = Scm_GetStandardCharSet(SCM_CHARSET_SPACE);
+                moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_SPACE);
                 break;
             case 'w':
                 moreset_complement = FALSE;
-                moreset = Scm_GetStandardCharSet(SCM_CHARSET_WORD);
+                moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_WORD);
                 break;
             case 'W':
                 moreset_complement = TRUE;
-                moreset = Scm_GetStandardCharSet(SCM_CHARSET_WORD);
+                moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_WORD);
                 break;
             default:
                 goto ordchar;
             }
             if (moreset_complement) {
-                moreset = Scm_CharSetComplement(SCM_CHARSET(Scm_CharSetCopy(SCM_CHARSET(moreset))));
+                moreset = Scm_CharSetComplement(SCM_CHAR_SET(Scm_CharSetCopy(SCM_CHAR_SET(moreset))));
             }
-            Scm_CharSetAdd(set, SCM_CHARSET(moreset));
+            Scm_CharSetAdd(set, SCM_CHAR_SET(moreset));
             continue;
         case '[':
-            moreset = read_predef_charset(input, &chars);
-            if (!SCM_CHARSETP(moreset)) goto err;
-            Scm_CharSetAdd(set, SCM_CHARSET(moreset));
+            moreset = read_predef_charset(input, &chars, error_p);
+            if (!SCM_CHAR_SET_P(moreset)) goto err;
+            Scm_CharSetAdd(set, SCM_CHAR_SET(moreset));
             continue;
         ordchar:
         default:
@@ -726,16 +740,18 @@ ScmObj Scm_CharSetRead(ScmPort *input, int *complement_p,
         return SCM_OBJ(set);
     }
   err:
-    if (error_p)
-        Scm_Error("Unclosed bracket in charset syntax [%A",
+    if (error_p) {
+        Scm_Error("Invalid charset syntax [%A",
                   Scm_ListToString(Scm_ReverseX(chars)));
+    }
     return SCM_FALSE;
 }
 
 /* Read posix [:alpha:] etc.  The first '[' is already read.
-   Return #f on error.  Set reverse list of read chars in *chars */
+   Return #f on error if errorp is FALSE.
+   Set reverse list of read chars in *chars */
 #define MAX_CHARSET_NAME_LEN  10
-ScmObj read_predef_charset(ScmPort *input, ScmObj *chars)
+ScmObj read_predef_charset(ScmPort *input, ScmObj *chars, int error_p)
 {
     int i;
     char name[MAX_CHARSET_NAME_LEN];
@@ -746,38 +762,40 @@ ScmObj read_predef_charset(ScmPort *input, ScmObj *chars)
         *chars = Scm_Cons(SCM_MAKE_CHAR(ch), *chars);
         if (!SCM_CHAR_ASCII_P(ch)) break;
         if (ch != ']') {
-            name[i] = ch;
+            name[i] = (char)ch;
             continue;
         }
         if (strncmp(name, ":alnum:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_ALNUM);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_ALNUM);
         } else if (strncmp(name, ":alpha:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_ALPHA);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_ALPHA);
         } else if (strncmp(name, ":blank:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_BLANK);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_BLANK);
         } else if (strncmp(name, ":cntrl:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_CNTRL);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_CNTRL);
         } else if (strncmp(name, ":digit:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_DIGIT);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_DIGIT);
         } else if (strncmp(name, ":graph:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_GRAPH);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_GRAPH);
         } else if (strncmp(name, ":lower:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_LOWER);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_LOWER);
         } else if (strncmp(name, ":print:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_PRINT);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_PRINT);
         } else if (strncmp(name, ":punct:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_PUNCT);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_PUNCT);
         } else if (strncmp(name, ":space:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_SPACE);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_SPACE);
         } else if (strncmp(name, ":upper:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_UPPER);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_UPPER);
         } else if (strncmp(name, ":xdigit:", 8) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHARSET_XDIGIT);
+            return Scm_GetStandardCharSet(SCM_CHAR_SET_XDIGIT);
         } else break;
     }
     /* here we got invalid charset name */
-    name[i] = '\0';
-    Scm_Error("invalid or unsupported POSIX charset '[%s]'", name);
+    if (error_p) {
+        name[i] = '\0';
+        Scm_Error("invalid or unsupported POSIX charset '[%s]'", name);
+    }
     return SCM_FALSE;
 }
 
@@ -789,7 +807,7 @@ ScmObj read_predef_charset(ScmPort *input, ScmObj *chars)
  * ASCII range, that all character sets agree on.
  */
 
-static ScmCharSet *predef_charsets[SCM_CHARSET_NUM_PREDEFINED_SETS] = {NULL};
+static ScmCharSet *predef_charsets[SCM_CHAR_SET_NUM_PREDEFINED_SETS] = {NULL};
 static ScmInternalMutex predef_charsets_mutex;
 
 static void install_charsets(void)
@@ -799,28 +817,28 @@ static void install_charsets(void)
     SCM_INTERNAL_MUTEX_LOCK(predef_charsets_mutex);
 
 #define CS(n)  predef_charsets[n]
-    for (i = 0; i < SCM_CHARSET_NUM_PREDEFINED_SETS; i++) {
-        CS(i) = SCM_CHARSET(Scm_MakeEmptyCharSet());
+    for (i = 0; i < SCM_CHAR_SET_NUM_PREDEFINED_SETS; i++) {
+        CS(i) = SCM_CHAR_SET(Scm_MakeEmptyCharSet());
     }
-    for (code = 0; code < SCM_CHARSET_SMALL_CHARS; code++) {
-        if (isalnum(code)) MASK_SET(CS(SCM_CHARSET_ALNUM), code);
-        if (isalpha(code)) MASK_SET(CS(SCM_CHARSET_ALPHA), code);
-        if (iscntrl(code)) MASK_SET(CS(SCM_CHARSET_CNTRL), code);
-        if (isdigit(code)) MASK_SET(CS(SCM_CHARSET_DIGIT), code);
-        if (isgraph(code)) MASK_SET(CS(SCM_CHARSET_GRAPH), code);
-        if (islower(code)) MASK_SET(CS(SCM_CHARSET_LOWER), code);
-        if (isprint(code)) MASK_SET(CS(SCM_CHARSET_PRINT), code);
-        if (ispunct(code)) MASK_SET(CS(SCM_CHARSET_PUNCT), code);
-        if (isspace(code)) MASK_SET(CS(SCM_CHARSET_SPACE), code);
-        if (isupper(code)) MASK_SET(CS(SCM_CHARSET_UPPER), code);
-        if (isxdigit(code)) MASK_SET(CS(SCM_CHARSET_XDIGIT), code);
+    for (code = 0; code < SCM_CHAR_SET_SMALL_CHARS; code++) {
+        if (isalnum(code)) MASK_SET(CS(SCM_CHAR_SET_ALNUM), code);
+        if (isalpha(code)) MASK_SET(CS(SCM_CHAR_SET_ALPHA), code);
+        if (iscntrl(code)) MASK_SET(CS(SCM_CHAR_SET_CNTRL), code);
+        if (isdigit(code)) MASK_SET(CS(SCM_CHAR_SET_DIGIT), code);
+        if (isgraph(code)) MASK_SET(CS(SCM_CHAR_SET_GRAPH), code);
+        if (islower(code)) MASK_SET(CS(SCM_CHAR_SET_LOWER), code);
+        if (isprint(code)) MASK_SET(CS(SCM_CHAR_SET_PRINT), code);
+        if (ispunct(code)) MASK_SET(CS(SCM_CHAR_SET_PUNCT), code);
+        if (isspace(code)) MASK_SET(CS(SCM_CHAR_SET_SPACE), code);
+        if (isupper(code)) MASK_SET(CS(SCM_CHAR_SET_UPPER), code);
+        if (isxdigit(code)) MASK_SET(CS(SCM_CHAR_SET_XDIGIT), code);
         /* Default word constituent chars #[\w].  NB: in future versions,
            a parameter might be introduced to customize this set. */
         if (isalnum(code)||code=='_')
-            MASK_SET(CS(SCM_CHARSET_WORD), code);
+            MASK_SET(CS(SCM_CHAR_SET_WORD), code);
         /* isblank() is not in posix.  for now, I hardcode it. */
         if (code == ' ' || code == '\t')
-            MASK_SET(CS(SCM_CHARSET_BLANK), code);
+            MASK_SET(CS(SCM_CHAR_SET_BLANK), code);
     }
 #undef CS
     SCM_INTERNAL_MUTEX_UNLOCK(predef_charsets_mutex);
@@ -828,7 +846,7 @@ static void install_charsets(void)
 
 ScmObj Scm_GetStandardCharSet(int id)
 {
-    if (id < 0 || id >= SCM_CHARSET_NUM_PREDEFINED_SETS)
+    if (id < 0 || id >= SCM_CHAR_SET_NUM_PREDEFINED_SETS)
         Scm_Error("bad id for predefined charset index: %d", id);
     if (predef_charsets[id] == NULL) {
         install_charsets();
