@@ -1,7 +1,7 @@
 /*
  * core.c - core kernel interface
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,22 +30,25 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: core.c,v 1.79 2007/04/01 04:21:38 shirok Exp $
+ *  $Id: core.c,v 1.84 2008-05-10 13:36:18 shirok Exp $
  */
 
-#include <stdlib.h>
-#include <unistd.h>
 #define LIBGAUCHE_BODY
 #include "gauche.h"
 #include "gauche/arch.h"
 #include "gauche/paths.h"
 #include "gauche/builtin-syms.h"
 
+/* GC_print_static_roots() is declared in private/gc_priv.h.  It is too much
+   hassle to include it with other GC internal baggages, so we just declare
+   it.  It's a private funciton of GC, so watch out the changes in GC. */
+extern void GC_print_static_roots(void);
+
 /*
  * out-of-memory handler.  this will be called by GC.
  */
 
-static GC_PTR oom_handler(size_t bytes)
+static void *oom_handler(size_t bytes)
 {
     Scm_Panic("out of memory (%d).  aborting...", bytes);
     return NULL;                /* dummy */
@@ -85,6 +88,7 @@ extern void Scm__InitVM(void);
 extern void Scm__InitRepl(void);
 extern void Scm__InitParameter(void);
 extern void Scm__InitAutoloads(void);
+extern void Scm__InitCollection(void);
 
 extern void Scm_Init_stdlib(ScmModule *);
 extern void Scm_Init_extlib(ScmModule *);
@@ -130,6 +134,7 @@ void Scm_Init(const char *signature)
 
     /* Initialize components.  The order is important, for some components
        rely on the other components to be initialized. */
+    Scm__InitVM();
     Scm__InitSymbol();
     Scm__InitModule();
     Scm__InitKeyword();
@@ -142,7 +147,6 @@ void Scm_Init(const char *signature)
     Scm__InitPort();
     Scm__InitWrite();
     Scm__InitCode();
-    Scm__InitVM();
     Scm__InitParameter();
     Scm__InitMacro();
     Scm__InitLoad();
@@ -151,7 +155,7 @@ void Scm_Init(const char *signature)
     Scm__InitSignal();
     Scm__InitSystem();
     Scm__InitRepl();
-
+    
     Scm_Init_stdlib(Scm_SchemeModule());
     Scm_Init_extlib(Scm_GaucheModule());
     Scm_Init_syslib(Scm_GaucheModule());
@@ -187,6 +191,11 @@ void Scm_GC()
     GC_gcollect();
 }
 
+void Scm_PrintStaticRoots()
+{
+    GC_print_static_roots();
+}
+
 /*
  * External API to register root set in dynamically loaded library.
  * Boehm GC doesn't do this automatically on some platforms.
@@ -200,10 +209,10 @@ void Scm_RegisterDL(void *data_start, void *data_end,
                     void *bss_start, void *bss_end)
 {
     if (data_start < data_end) {
-        GC_add_roots((GC_PTR)data_start, (GC_PTR)data_end);
+        GC_add_roots(data_start, data_end);
     }
     if (bss_start < bss_end) {
-        GC_add_roots((GC_PTR)bss_start, (GC_PTR)bss_end);
+        GC_add_roots(bss_start, bss_end);
     }
 }
 
@@ -227,14 +236,14 @@ void Scm_GCSentinel(void *obj, const char *name)
  */
 void Scm_RegisterFinalizer(ScmObj z, ScmFinalizerProc finalizer, void *data)
 {
-    GC_finalization_proc ofn; GC_PTR ocd;
+    GC_finalization_proc ofn; void *ocd;
     GC_REGISTER_FINALIZER_NO_ORDER(z, (GC_finalization_proc)finalizer,
                                    data, &ofn, &ocd);
 }
 
 void Scm_UnregisterFinalizer(ScmObj z)
 {
-    GC_finalization_proc ofn; GC_PTR ocd;
+    GC_finalization_proc ofn; void *ocd;
     GC_REGISTER_FINALIZER_NO_ORDER(z, (GC_finalization_proc)NULL, NULL,
                                    &ofn, &ocd);
 }
@@ -353,7 +362,7 @@ void Scm_Panic(const char *msg, ...)
 /* Use this for absolute emergency.  Newline is not attached to msg. */
 void Scm_Abort(const char *msg)
 {
-    int size = strlen(msg);
+    int size = (int)strlen(msg);
     write(2, msg, size); /* this may return an error, but we don't care */
     _exit(1);
 }
@@ -408,7 +417,7 @@ init_cond_features()
         const char *module;
     } init_features[] = {
         { "gauche", NULL },
-#ifdef __MINGW32__
+#ifdef GAUCHE_WINDOWS
         { "gauche.os.windows", NULL },
         { "gauche-windows", NULL }, /* for backward compatibility */
 #endif
