@@ -1,7 +1,7 @@
 /*
  * error.c - error handling
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,12 +30,9 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: error.c,v 1.77 2007/04/12 03:26:55 shirok Exp $
+ *  $Id: error.c,v 1.80 2008-05-10 13:36:18 shirok Exp $
  */
 
-#include <errno.h>
-#include <string.h>
-#include <ctype.h>
 #define LIBGAUCHE_BODY
 #include "gauche.h"
 #include "gauche/class.h"
@@ -43,6 +40,10 @@
 #include "gauche/vm.h"
 #include "gauche/builtin-syms.h"
 
+#include <errno.h>
+#include <string.h>
+#include <ctype.h>
+
 static void   message_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx);
 static ScmObj message_allocate(ScmClass *klass, ScmObj initargs);
 static ScmObj syserror_allocate(ScmClass *klass, ScmObj initargs);
@@ -113,7 +114,7 @@ static void message_set(ScmMessageCondition *obj, ScmObj val)
 
 static ScmClassStaticSlotSpec message_slots[] = {
     SCM_CLASS_SLOT_SPEC("message", message_get, message_set),
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 /*------------------------------------------------------------
@@ -284,12 +285,12 @@ static void porterror_port_set(ScmPortError *obj, ScmObj val)
 
 static ScmClassStaticSlotSpec syserror_slots[] = {
     SCM_CLASS_SLOT_SPEC("errno", syserror_number_get, syserror_number_set),
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 static ScmClassStaticSlotSpec sigerror_slots[] = {
     SCM_CLASS_SLOT_SPEC("signal", sigerror_signal_get, sigerror_signal_set),
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 static ScmClassStaticSlotSpec readerror_slots[] = {
@@ -298,12 +299,12 @@ static ScmClassStaticSlotSpec readerror_slots[] = {
     SCM_CLASS_SLOT_SPEC("column", readerror_dummy_get, readerror_dummy_set),
     SCM_CLASS_SLOT_SPEC("position", readerror_dummy_get, readerror_dummy_set),
     SCM_CLASS_SLOT_SPEC("span", readerror_dummy_get, readerror_dummy_set),
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 static ScmClassStaticSlotSpec porterror_slots[] = {
     SCM_CLASS_SLOT_SPEC("port", porterror_port_get, porterror_port_set),
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 /*------------------------------------------------------------
@@ -398,7 +399,7 @@ static void   conditions_set(ScmCompoundCondition *obj, ScmObj conds)
 
 static ScmClassStaticSlotSpec compound_slots[] = {
     SCM_CLASS_SLOT_SPEC("%conditions", conditions_get, conditions_set),
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 
@@ -548,29 +549,31 @@ void Scm_Error(const char *msg, ...)
 static ScmObj get_syserrmsg(int en)
 {
     ScmObj syserr;
-#ifndef __MINGW32__
+#if !defined(GAUCHE_WINDOWS)
     syserr = SCM_MAKE_STR_COPYING(strerror(en));
-#else  /*__MINGW32__*/
+#else  /*GAUCHE_WINDOWS*/
     LPTSTR msgbuf;
+    const char *xmsgbuf;
     FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,
 		  NULL,
 		  en,
 		  0,
 		  (LPTSTR)&msgbuf,
-		  0, NULL);
-    syserr = SCM_MAKE_STR_COPYING(msgbuf);
+                  0, NULL);
+    xmsgbuf = SCM_WCS2MBS(msgbuf);
+    syserr = SCM_MAKE_STR_COPYING(xmsgbuf);
     LocalFree(msgbuf);
-#endif /*__MINGW32__*/
+#endif /*GAUCHE_WINDOWS*/
     return syserr;
 }
 
 static int get_errno(void)
 {
-#ifndef __MINGW32__
+#if !defined(GAUCHE_WINDOWS)
     return errno;
-#else  /*__MINGW32__*/
+#else  /*GAUCHE_WINDOWS*/
     return GetLastError();
-#endif /*__MINGW32__*/
+#endif /*GAUCHE_WINDOWS*/
 }
 
 void Scm_SysError(const char *msg, ...)
