@@ -1,7 +1,7 @@
 /*
  * gauche.h - Gauche scheme system header
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,13 +30,21 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: gauche.h,v 1.492 2007/04/12 03:26:55 shirok Exp $
+ *  $Id: gauche.h,v 1.508 2008-06-02 01:13:13 shirok Exp $
  */
 
 #ifndef GAUCHE_H
 #define GAUCHE_H
 
+/* Read config.h _before_ other headers, for it may affect the behavior
+   of system header files.  Currently the only known instance of it is
+   sigwait() on Solaris---we need to define _POSIX_PTHREAD_SEMANTICS to
+   get pthread-compatible sigwait()---but we may encounter more of such
+   instances. */
+#include <gauche/config.h>
+
 #include <stdio.h>
+#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdarg.h>
@@ -45,14 +53,17 @@
 #include <signal.h>
 #include <string.h>
 #include <errno.h>
-#include <gauche/config.h>  /* read config.h _before_ gc.h */
 #include <gauche/int64.h>
 #include <gauche/float.h>
 
 #if defined(LIBGAUCHE_BODY)
+#if !defined(GC_DLL)
 #define GC_DLL    /* for gc.h to handle Win32 crazyness */
+#endif
+#if !defined(GC_BUILD)
 #define GC_BUILD  /* ditto */
-#endif 
+#endif
+#endif /* LIBGAUCHE_BODY */ 
 #include <gc.h>
 
 #ifndef SCM_DECL_BEGIN
@@ -78,21 +89,18 @@ SCM_DECL_BEGIN
 # endif
 #endif
 
-/* Ugly cliche for Win32. */
-#if defined(__CYGWIN__) || defined(__MINGW32__)
-# if defined(LIBGAUCHE_BODY)
-#  define SCM_EXTERN extern
-# else
-#  define SCM_EXTERN extern __declspec(dllimport)
-# endif
-#else  /*!(__CYGWIN__ || __MINGW32__)*/
-# define SCM_EXTERN extern
-#endif /*!(__CYGWIN__ || __MINGW32__)*/
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif /*HAVE_UNISTD_H*/
 
-/* For Mingw32, we need some tricks */
-#if defined(__MINGW32__)
-#include <gauche/mingw-compat.h>
-#endif /*__MINGW32__*/
+/* For Windows platforms, we need some compatibility tricks.
+   This defines GAUCHE_WINDOWS preprocessor symbol. */
+#if defined(__MINGW32__) || defined(MSVC)
+#include <gauche/win-compat.h>
+#endif /* MINGW32 || WINDOWS */
+
+/* Defines SCM_EXTERN magic. */
+#include <gauche/extern.h>
 
 /* Some useful macros */
 #ifndef FALSE
@@ -107,7 +115,7 @@ SCM_DECL_BEGIN
 #define SCM_DEBUG_HELPER      TRUE
 #endif
 
-#define SCM_INLINE_MALLOC_PRIMITIVES 1
+#define SCM_INLINE_MALLOC_PRIMITIVES 0
 
 #ifdef GAUCHE_USE_PTHREADS
 # include <gauche/pthread.h>
@@ -124,7 +132,7 @@ SCM_DECL_BEGIN
 /*
  * A word large enough to hold a pointer
  */
-typedef unsigned long ScmWord;
+typedef intptr_t ScmWord;
 
 /*
  * A byte
@@ -237,10 +245,10 @@ SCM_EXTERN int Scm_EqualM(ScmObj x, ScmObj y, int mode);
  */
 
 #define SCM_INTP(obj)        (SCM_TAG(obj) == 1)
-#define SCM_INT_VALUE(obj)   (((signed long int)(obj)) >> 2)
-#define SCM_MAKE_INT(obj)    SCM_OBJ(((long)(obj) << 2) + 1)
+#define SCM_INT_VALUE(obj)   (((signed long int)SCM_WORD(obj)) >> 2)
+#define SCM_MAKE_INT(obj)    SCM_OBJ(((intptr_t)(obj) << 2) + 1)
 
-#define SCM_UINTP(obj)       (SCM_INTP(obj)&&((signed long int)(obj)>=0))
+#define SCM_UINTP(obj)       (SCM_INTP(obj)&&((signed long int)SCM_WORD(obj)>=0))
 
 /*
  * CHARACTERS
@@ -254,7 +262,7 @@ SCM_EXTERN int Scm_EqualM(ScmObj x, ScmObj y, int mode);
 #define	SCM_CHAR(obj)           ((ScmChar)(obj))
 #define	SCM_CHARP(obj)          ((SCM_WORD(obj)&0x07L) == 2)
 #define	SCM_CHAR_VALUE(obj)     SCM_CHAR(SCM_WORD(obj) >> 3)
-#define	SCM_MAKE_CHAR(ch)       SCM_OBJ((long)((ch) << 3) + 2)
+#define	SCM_MAKE_CHAR(ch)       SCM_OBJ(SCM_WORD((ch) << 3) + 2)
 
 #define SCM_CHAR_INVALID        ((ScmChar)(-1)) /* indicate invalid char */
 #define SCM_CHAR_MAX            (0x1fffffff)
@@ -273,6 +281,9 @@ SCM_EXTERN ScmObj Scm_CharEncodingName(void);
 SCM_EXTERN const char **Scm_SupportedCharacterEncodings(void);
 SCM_EXTERN int Scm_SupportedCharacterEncodingP(const char *encoding);
 
+SCM_EXTERN void Scm__InstallCharconvHooks(ScmChar (*u2c)(int),
+                                          int (*c2u)(ScmChar));
+
 #if   defined(GAUCHE_CHAR_ENCODING_EUC_JP)
 #include "gauche/char_euc_jp.h"
 #elif defined(GAUCHE_CHAR_ENCODING_UTF_8)
@@ -293,30 +304,60 @@ SCM_EXTERN int Scm_SupportedCharacterEncodingP(const char *encoding);
 
 #define SCM_HOBJP(obj)  (SCM_PTRP(obj)&&SCM_TAG(SCM_OBJ(obj)->tag)==3)
 
-#define SCM_CPP_CAT(a, b)   a ## b
+#define SCM_CPP_CAT(a, b)   a##b
 #define SCM_CPP_CAT3(a, b, c)  a ## b ## c
 
-#define SCM_CLASS_DECL(klass) extern ScmClass klass
-#define SCM_CLASS_STATIC_PTR(klass)  (&klass)
+/* We use a pointer to the class structure (with low-bit tag) as
+   the generic type tag. */
 #define SCM_CLASS2TAG(klass)  ((ScmByte*)(klass) + 3)
 
 /* A common header for heap-allocated objects */
 typedef struct ScmHeaderRec {
     ScmByte *tag;                /* private.  should be accessed
-                                    only via macros. */
+                                    only via SCM_CLASS_OF and SCM_SET_CLASS
+                                    macros. */
 } ScmHeader;
 
 #define SCM_HEADER       ScmHeader hdr /* for declaration */
 
+/* Here comes the ugly part.  To understand the general idea, just ignore
+   GAUCHE_BROKEN_LINKER_WORKAROUND part; it's pretty simple.  Every heap
+   allocated object contains (pointer to its class + 3) in its tag field.  */
+#if !defined(GAUCHE_BROKEN_LINKER_WORKAROUND)
+
+# define SCM_CLASS_DECL(klass) extern ScmClass klass
+# define SCM_CLASS_STATIC_PTR(klass) (&klass)
+# define SCM_CLASS_STATIC_TAG(klass) SCM_CLASS2TAG(&klass)
+
 /* Extract the class pointer from the tag.
    You can use these only if SCM_HOBJP(obj) != FALSE */
-#define SCM_CLASS_OF(obj)      SCM_CLASS((SCM_OBJ(obj)->tag - 3))
-#define SCM_SET_CLASS(obj, k)  (SCM_OBJ(obj)->tag = (ScmByte*)(k) + 3)
+# define SCM_CLASS_OF(obj)      SCM_CLASS((SCM_OBJ(obj)->tag - 3))
+# define SCM_SET_CLASS(obj, k)  (SCM_OBJ(obj)->tag = (ScmByte*)(k) + 3)
 
-/* Check if classof(OBJ) equals to an extended class KLASS */
-#define SCM_XTYPEP(obj, klass) \
+/* Check if classof(OBJ) equals to an extended class KLASS.
+   We can check SCM_PTRP instead of SCM_HOBJP here, since a pair never
+   satisfies the second test. */
+# define SCM_XTYPEP(obj, klass) \
     (SCM_PTRP(obj)&&(SCM_OBJ(obj)->tag == SCM_CLASS2TAG(klass)))
 
+#else  /*GAUCHE_BROKEN_LINKER_WORKAROUND*/
+
+/* You don't want to understand these. */
+# define SCM_CLASS_DECL(klass) \
+    SCM_EXTERN ScmClass klass;                  \
+    extern ScmClass *SCM_CPP_CAT(_imp__, klass) 
+# define SCM_CLASS_STATIC_PTR(klass) ((ScmClass*)(&SCM_CPP_CAT(_imp__,klass)))
+# define SCM_CLASS_STATIC_TAG(klass) SCM_CLASS2TAG(SCM_CLASS_STATIC_PTR(klass))
+
+# define SCM_CLASS_OF(obj)      (*(ScmClass**)((SCM_OBJ(obj)->tag - 3)))
+# define SCM_SET_CLASS(obj, k)  (SCM_OBJ(obj)->tag = (ScmByte*)((k)->classPtr) + 3)
+
+# define SCM_XTYPEP(obj, klass) \
+    (SCM_HOBJP(obj)&&(SCM_CLASS_OF(obj) == klass))
+#endif /*GAUCHE_BROKEN_LINKER_WORKAROUND*/
+
+
+
 /* Check if classof(OBJ) is a subtype of an extended class KLASS */
 #define SCM_ISA(obj, klass) (SCM_XTYPEP(obj,klass)||Scm_TypeP(SCM_OBJ(obj),klass))
 
@@ -432,6 +473,15 @@ SCM_EXTERN int Scm_Apply(ScmObj proc, ScmObj args,
    ones does not capture exceptions. */
 SCM_EXTERN ScmObj Scm_EvalRec(ScmObj form, ScmObj env);
 SCM_EXTERN ScmObj Scm_ApplyRec(ScmObj proc, ScmObj args);
+SCM_EXTERN ScmObj Scm_ApplyRec0(ScmObj proc);
+SCM_EXTERN ScmObj Scm_ApplyRec1(ScmObj proc, ScmObj arg0);
+SCM_EXTERN ScmObj Scm_ApplyRec2(ScmObj proc, ScmObj arg0, ScmObj arg1);
+SCM_EXTERN ScmObj Scm_ApplyRec3(ScmObj proc, ScmObj arg0, ScmObj arg1,
+                                ScmObj arg2);
+SCM_EXTERN ScmObj Scm_ApplyRec4(ScmObj proc, ScmObj arg0, ScmObj arg1,
+                                ScmObj arg2, ScmObj arg3);
+SCM_EXTERN ScmObj Scm_ApplyRec5(ScmObj proc, ScmObj arg0, ScmObj arg1,
+                                ScmObj arg2, ScmObj arg3, ScmObj arg4);
 
 /* for compatibility */
 #define Scm_EvalCStringRec(f, e)  Scm_EvalRec(Scm_ReadFromCString(f), e)
@@ -477,6 +527,7 @@ SCM_EXTERN ScmObj Scm_MakeMacroAutoload(ScmSymbol *name,
 
 SCM_EXTERN ScmObj Scm_UnwrapSyntax(ScmObj form);
 
+SCM_EXTERN int    Scm_VMGetNumResults(ScmVM *vm);
 SCM_EXTERN ScmObj Scm_VMGetResult(ScmVM *vm);
 SCM_EXTERN ScmObj Scm_VMGetStackLite(ScmVM *vm);
 SCM_EXTERN ScmObj Scm_VMGetStack(ScmVM *vm);
@@ -501,6 +552,9 @@ typedef ScmObj (*ScmClassAllocateProc)(ScmClass *klass, ScmObj initargs);
    reflected to the class definition macros below */
 struct ScmClassRec {
     SCM_INSTANCE_HEADER;
+#if defined(GAUCHE_BROKEN_LINKER_WORKAROUND)
+    ScmClass **classPtr;
+#endif
     ScmClassPrintProc     print;
     ScmClassCompareProc   compare;
     ScmClassSerializeProc serialize;
@@ -641,8 +695,10 @@ SCM_CLASS_DECL(Scm_ForeignPointerClass);
 #define SCM_CLASS_OBJECT           (&Scm_ObjectClass)
 #define SCM_CLASS_FOREIGN_POINTER  (&Scm_ForeignPointerClass)
 
-SCM_EXTERN ScmClass *Scm_DefaultCPL[];
-SCM_EXTERN ScmClass *Scm_ObjectCPL[];
+/* NB: we can't use SCM_EXTERN because Windows DLL can't use the address of
+   dllimport-ed variables as constants. */
+extern ScmClass *Scm_DefaultCPL[];
+extern ScmClass *Scm_ObjectCPL[];
 
 #define SCM_CLASS_DEFAULT_CPL     (Scm_DefaultCPL)
 #define SCM_CLASS_OBJECT_CPL      (Scm_ObjectCPL)
@@ -654,9 +710,20 @@ SCM_EXTERN ScmClass *Scm_ObjectCPL[];
  *   SCM_DEFINE_BASE_CLASS
  */
 
+/* internal macro.  do not use directly */
+#if defined(GAUCHE_BROKEN_LINKER_WORKAROUND)
+#define SCM__CLASS_PTR_SLOT(cname)  (&SCM_CPP_CAT(_imp__, cname)),
+#define SCM__CLASS_PTR_BODY(cname) \
+    ; ScmClass *SCM_CPP_CAT(_imp__, cname) = &cname
+#else  /*!GAUCHE_BROKEN_LINKER_WORKAROUND*/
+#define SCM__CLASS_PTR_SLOT(cname)  /* none */
+#define SCM__CLASS_PTR_BODY(cname)  /* none */
+#endif /*!GAUCHE_BROKEN_LINKER_WORKAROUND*/
+
 #define SCM__DEFINE_CLASS_COMMON(cname, coreSize, flag, printer, compare, serialize, allocate, cpa) \
     ScmClass cname = {                           \
-        { SCM_CLASS2TAG(SCM_CLASS_CLASS), NULL },\
+        { SCM_CLASS_STATIC_TAG(Scm_ClassClass), NULL },\
+        SCM__CLASS_PTR_SLOT(cname)               \
         printer,                                 \
         compare,                                 \
         serialize,                               \
@@ -678,7 +745,7 @@ SCM_EXTERN ScmClass *Scm_ObjectCPL[];
         SCM_FALSE, /*redefined*/                 \
         SCM_INTERNAL_MUTEX_INITIALIZER,          \
         SCM_INTERNAL_COND_INITIALIZER,           \
-    }
+    } SCM__CLASS_PTR_BODY(cname)
     
 /* Define built-in class statically -- full-featured version */
 #define SCM_DEFINE_BUILTIN_CLASS(cname, printer, compare, serialize, allocate, cpa) \
@@ -1002,206 +1069,13 @@ SCM_EXTERN ScmObj Scm_Vsprintf(const char *fmt, va_list args, int sharedp);
  * READ
  */
 
-typedef struct ScmReadContextRec {
-    int flags;                  /* see below */
-    ScmHashTable *table;        /* used internally. */
-    ScmObj pending;             /* used internally. */
-} ScmReadContext;
-
-enum {
-    SCM_READ_SOURCE_INFO = (1L<<0),  /* preserving souce file information */
-    SCM_READ_CASE_FOLD   = (1L<<1),  /* case-fold read */
-    SCM_READ_LITERAL_IMMUTABLE = (1L<<2), /* literal should be read as immutable */
-    SCM_READ_DISABLE_CTOR = (1L<<3), /* disable #,() */
-    SCM_READ_RECURSIVELY = (1L<<4)   /* used internally. */
-};
-
-#define SCM_READ_CONTEXT_INIT(ctx) \
-   do { (ctx)->flags = 0; } while (0)
-
-/* An object to keep unrealized circular reference (e.g. #N=) during
- * 'read'.  It is replaced by the reference value before exitting 'read',
- * and it shouldn't leak out to the normal Scheme program, except the
- * code that handles it explicitly (like read-time constructor).
- */
-typedef struct ScmReadReferenceRec {
-    SCM_HEADER;
-    ScmObj value;               /* realized reference.  initially UNBOUND */
-} ScmReadReference;
-
-SCM_CLASS_DECL(Scm_ReadReferenceClass);
-#define SCM_CLASS_READ_REFERENCE  (&Scm_ReadReferenceClass)
-#define SCM_READ_REFERENCE(obj)   ((ScmReadReference*)(obj))
-#define SCM_READ_REFERENCE_P(obj) SCM_XTYPEP(obj, SCM_CLASS_READ_REFERENCE)
-#define SCM_READ_REFERENCE_REALIZED(obj) \
-   (!SCM_EQ(SCM_READ_REFERENCE(obj)->value, SCM_UNBOUND))
-
-SCM_EXTERN ScmObj Scm_Read(ScmObj port);
-SCM_EXTERN ScmObj Scm_ReadWithContext(ScmObj port, ScmReadContext *ctx);
-SCM_EXTERN ScmObj Scm_ReadList(ScmObj port, ScmChar closer);
-SCM_EXTERN ScmObj Scm_ReadListWithContext(ScmObj port, ScmChar closer,
-                                          ScmReadContext *ctx);
-SCM_EXTERN ScmObj Scm_ReadFromString(ScmString *string);
-SCM_EXTERN ScmObj Scm_ReadFromCString(const char *string);
-
-SCM_EXTERN void   Scm_ReadError(ScmPort *port, const char *fmt, ...);
-
-SCM_EXTERN ScmObj Scm_DefineReaderCtor(ScmObj symbol, ScmObj proc,
-                                       ScmObj finisher);
-    
-/*--------------------------------------------------------
- * WEAK VECTOR & WEAK BOX
- */
-
-typedef struct ScmWeakVectorRec {
-    SCM_HEADER;
-    int size;
-    void *pointers;  /* opaque */
-} ScmWeakVector;
-
-#define SCM_WEAK_VECTOR(obj)   ((ScmWeakVector*)(obj))
-#define SCM_WEAK_VECTOR_P(obj)  SCM_XTYPEP(obj, SCM_CLASS_WEAK_VECTOR)
-SCM_CLASS_DECL(Scm_WeakVectorClass);
-#define SCM_CLASS_WEAK_VECTOR  (&Scm_WeakVectorClass)
-    
-SCM_EXTERN ScmObj Scm_MakeWeakVector(int size);
-SCM_EXTERN ScmObj Scm_WeakVectorRef(ScmWeakVector *v, int index, ScmObj fallback);
-SCM_EXTERN ScmObj Scm_WeakVectorSet(ScmWeakVector *v, int index, ScmObj val);
-
-typedef struct ScmWeakBoxRec ScmWeakBox; /* opaque */
-
-SCM_EXTERN ScmWeakBox *Scm_MakeWeakBox(void *value);
-SCM_EXTERN int         Scm_WeakBoxEmptyP(ScmWeakBox *wbox);
-SCM_EXTERN void        Scm_WeakBoxSet(ScmWeakBox *wbox, void *value);
-SCM_EXTERN void       *Scm_WeakBoxRef(ScmWeakBox *wbox);
+#include <gauche/reader.h>
 
 /*--------------------------------------------------------
  * HASHTABLE
  */
 
-#if 0
-typedef struct ScmHashEntryRec ScmHashEntry;
-
-typedef ScmHashEntry *(*ScmHashAccessProc)(ScmHashTable *ht,
-                                           void *key, int op, void *val);
-typedef unsigned long (*ScmHashProc)(ScmHashTable *ht, void *key);
-typedef int (*ScmHashCmpProc)(ScmHashTable *ht, void *key, ScmHashEntry *e);
-
-struct ScmHashTableRec {
-    SCM_HEADER;
-    ScmHashEntry **buckets;
-    int numBuckets;
-    int numEntries;
-    int numBucketsLog2;
-    int type;
-    ScmHashAccessProc accessfn;
-    ScmHashProc hashfn;
-    ScmHashCmpProc cmpfn;
-    void *data;
-};
-
-SCM_CLASS_DECL(Scm_HashTableClass);
-#define SCM_CLASS_HASH_TABLE  (&Scm_HashTableClass)
-#define SCM_HASH_TABLE(obj)   ((ScmHashTable*)(obj))
-#define SCM_HASH_TABLE_P(obj)  SCM_ISA(obj, SCM_CLASS_HASH_TABLE)
-
-/* Hash types */
-enum {
-    /* ScmObj hashtables */
-    SCM_HASH_EQ,
-    SCM_HASH_EQV,
-    SCM_HASH_EQUAL,
-    SCM_HASH_STRING,
-    SCM_HASH_GENERAL,
-
-    /* Raw hashtables */
-    SCM_HASH_WORD,
-    SCM_HASH_MULTIWORD,
-    SCM_HASH_RAW
-};
-
-#define SCM_HASH_TABLE_RAW_P(ht) (SCM_HASH_TABLE(ht)->type >= SCM_HASH_WORD)
-
-/* 'weakness' type */
-enum {
-    SCM_HASH_WEAK_KEY    = 0x01,
-    SCM_HASH_WEAK_VALUE  = 0x02,
-    SCM_HASH_WEAK_BOTH   = (SCM_HASH_WEAK_KEY|SCM_HASH_WEAK_VALUE)
-};
-
-/* auxiliary structure; not an ScmObj. */
-struct ScmHashEntryRec {
-    void *key;
-    void *value;
-    struct ScmHashEntryRec *next;
-};
-
-typedef  struct ScmHashIterRec {
-    ScmHashTable *table;
-    int currentBucket;
-    ScmHashEntry *currentEntry;
-} ScmHashIter;
-
-/* Constructors */
-SCM_EXTERN ScmObj Scm_MakeHashTableSimple(int type, int initSize);
-SCM_EXTERN ScmObj Scm_MakeHashTableMultiWord(int keySize, int initSize);
-SCM_EXTERN ScmObj Scm_MakeHashTableFull(ScmClass *klass,
-                                        int type,
-                                        ScmHashProc hashfn,
-                                        ScmHashCmpProc cmpfn,
-                                        int initSize,
-                                        void *data);
-
-SCM_EXTERN ScmObj Scm_CopyHashTable(ScmHashTable *tab);
-
-/* Accessor API: the 'Raw' versions are base ones, to be used for
-   any type of hashtables.  The versions without 'Raw' do checks
-   to make sure the given hashtable is the ScmObj one. */
-SCM_EXTERN ScmHashEntry *Scm_HashTableGetRaw(ScmHashTable *hash, void *key);
-SCM_EXTERN ScmHashEntry *Scm_HashTableAddRaw(ScmHashTable *hash,
-                                             void *key, void *value);
-SCM_EXTERN ScmHashEntry *Scm_HashTablePutRaw(ScmHashTable *hash,
-                                             void *key, void *value);
-SCM_EXTERN ScmHashEntry *Scm_HashTableDeleteRaw(ScmHashTable *hash, void *key);
-
-SCM_EXTERN ScmHashEntry *Scm_HashTableGet(ScmHashTable *hash, ScmObj key);
-SCM_EXTERN ScmHashEntry *Scm_HashTableAdd(ScmHashTable *hash,
-					  ScmObj key, ScmObj value);
-SCM_EXTERN ScmHashEntry *Scm_HashTablePut(ScmHashTable *hash,
-					  ScmObj key, ScmObj value);
-SCM_EXTERN ScmHashEntry *Scm_HashTableDelete(ScmHashTable *hash, ScmObj key);
-
-SCM_EXTERN ScmObj Scm_HashTableKeys(ScmHashTable *table);
-SCM_EXTERN ScmObj Scm_HashTableValues(ScmHashTable *table);
-
-SCM_EXTERN void Scm_HashIterInitRaw(ScmHashTable *hash, ScmHashIter *iter);
-SCM_EXTERN void Scm_HashIterInit(ScmHashTable *hash, ScmHashIter *iter);
-SCM_EXTERN ScmHashEntry *Scm_HashIterNext(ScmHashIter *iter);
-
-/* Miscellaneous utils*/
-SCM_EXTERN ScmObj Scm_HashTableStat(ScmHashTable *table);
-
-/* Hash functions */
-SCM_EXTERN unsigned long Scm_EqHash(ScmObj obj);
-SCM_EXTERN unsigned long Scm_EqvHash(ScmObj obj);
-SCM_EXTERN unsigned long Scm_Hash(ScmObj obj);
-SCM_EXTERN unsigned long Scm_HashString(ScmString *str, unsigned long bound);
-
-/* Compatibility stuff
-   Use of these APIs are deprecated.  They'll go away in a few releases. */
-
-#define SCM_HASHTABLE       SCM_HASH_TABLE
-#define SCM_HASHTABLEP      SCM_HASH_TABLE_P
-#define SCM_CLASS_HASHTABLE SCM_CLASS_HASH_TABLE
-#define SCM_HASH_ADDRESS    SCM_HASH_EQ
-
-SCM_EXTERN ScmObj Scm_MakeHashTable(ScmHashProc hashfn,
-				    ScmHashCmpProc cmpfn,
-				    unsigned int initSize);
-
-#else
 #include <gauche/hash.h>
-#endif
 
 /*--------------------------------------------------------
  * TREEMAP
@@ -1209,6 +1083,12 @@ SCM_EXTERN ScmObj Scm_MakeHashTable(ScmHashProc hashfn,
 
 #include <gauche/treemap.h>
 
+/*--------------------------------------------------------
+ * WEAK VECTOR, WEAK BOX & WEAK HASH TABLES
+ */
+
+#include <gauche/weak.h>
+
 /*--------------------------------------------------------
  * CHAR-SET
  */
@@ -1242,9 +1122,8 @@ SCM_EXTERN ScmObj Scm_MakeModule(ScmSymbol *name, int error_if_exists);
 
 /* Flags for Scm_FindBinding and Scm_GlobalVariableRef */
 enum {
-    SCM_BINDING_STAY_IN_MODULE = (1L<<0), /* do not search parent/imported */
-    SCM_BINDING_KEEP_AUTOLOAD = (1L<<1)   /* do not resolve autoload 
-                                             (Scm_GlobalVaribleRef only) */
+    /* do not search parent/imported */
+    SCM_BINDING_STAY_IN_MODULE = (1L<<0)
 };
 
 SCM_EXTERN ScmGloc *Scm_FindBinding(ScmModule *module, ScmSymbol *symbol,
@@ -1338,18 +1217,19 @@ typedef ScmObj (*ScmTransformerProc)(ScmObj self, ScmObj form, ScmObj env,
 /* Base structure */
 struct ScmProcedureRec {
     SCM_INSTANCE_HEADER;
-    unsigned char required;     /* # of required args */
-    unsigned char optional;     /* 1 if it takes rest args */
-    unsigned char type;         /* procedure type  */
-    unsigned char locked;       /* setter locked? */
-    ScmObj info;                /* source code info */
-    ScmObj setter;              /* setter, if exists. */
-    ScmObj inliner;             /* inliner.  NB: for backward compatibility,
-                                   this may be initialized by NULL. */
+    unsigned int required : 10;    /* # of required args */
+    unsigned int optional : 1;     /* 1 if it takes rest args */
+    unsigned int type     : 3;     /* ScmProcedureType */
+    unsigned int locked   : 1;     /* setter locked? */
+    unsigned int currying : 1;     /* autocurrying */
+    ScmObj info;                   /* source code info */
+    ScmObj setter;                 /* setter, if exists. */
+    ScmObj inliner;                /* inliner.  NB: for backward compatibility,
+                                      this may be initialized by NULL. */
 };
 
 /* procedure type */
-enum {
+enum ScmProcedureType {
     SCM_PROC_SUBR,
     SCM_PROC_CLOSURE,
     SCM_PROC_GENERIC,
@@ -1361,6 +1241,7 @@ enum {
 #define SCM_PROCEDURE_REQUIRED(obj) SCM_PROCEDURE(obj)->required
 #define SCM_PROCEDURE_OPTIONAL(obj) SCM_PROCEDURE(obj)->optional
 #define SCM_PROCEDURE_TYPE(obj)     SCM_PROCEDURE(obj)->type
+#define SCM_PROCEDURE_CURRYING(obj) SCM_PROCEDURE(obj)->currying
 #define SCM_PROCEDURE_INFO(obj)     SCM_PROCEDURE(obj)->info
 #define SCM_PROCEDURE_SETTER(obj)   SCM_PROCEDURE(obj)->setter
 #define SCM_PROCEDURE_INLINER(obj)  SCM_PROCEDURE(obj)->inliner
@@ -1381,12 +1262,17 @@ SCM_CLASS_DECL(Scm_ProcedureClass);
     SCM_PROCEDURE(obj)->required = req,                 \
     SCM_PROCEDURE(obj)->optional = opt,                 \
     SCM_PROCEDURE(obj)->type = typ,                     \
+    SCM_PROCEDURE(obj)->locked = FALSE,                 \
+    SCM_PROCEDURE(obj)->currying = FALSE,               \
     SCM_PROCEDURE(obj)->info = inf,                     \
     SCM_PROCEDURE(obj)->setter = SCM_FALSE,             \
     SCM_PROCEDURE(obj)->inliner = SCM_FALSE
 
 #define SCM__PROCEDURE_INITIALIZER(klass, req, opt, typ, inf, inl)  \
-    { { klass }, (req), (opt), (typ), FALSE, (inf), SCM_FALSE, (inl) }
+    { { klass }, (req), (opt), (typ), FALSE, FALSE, (inf), SCM_FALSE, (inl) }
+
+SCM_EXTERN ScmObj Scm_CurryProcedure(ScmObj proc, ScmObj *given,
+                                     int ngiven, int foldlen);
 
 /* Closure - Scheme defined procedure */
 struct ScmClosureRec {
@@ -1416,7 +1302,7 @@ struct ScmSubrRec {
 
 #define SCM_DEFINE_SUBR(cvar, req, opt, inf, func, inliner, data)           \
     ScmSubr cvar = {                                                        \
-        SCM__PROCEDURE_INITIALIZER(SCM_CLASS2TAG(SCM_CLASS_PROCEDURE),      \
+        SCM__PROCEDURE_INITIALIZER(SCM_CLASS_STATIC_TAG(Scm_ProcedureClass),\
                                    req, opt, SCM_PROC_SUBR, inf, inliner),  \
         (func), (data)                                                      \
     }
@@ -1435,8 +1321,10 @@ SCM_EXTERN int    Scm_HasSetter(ScmObj proc);
 /* Generic - Generic function */
 struct ScmGenericRec {
     ScmProcedure common;
-    ScmObj methods;
-    ScmObj (*fallback)(ScmObj *args, int nargs, ScmGeneric *gf);
+    ScmObj methods;             /* list of methods */
+    int   maxReqargs;           /* maximum # of args required to select
+                                   applicable methods */
+    ScmObj (*fallback)(ScmObj *argv, int argc, ScmGeneric *gf);
     void *data;
     ScmInternalMutex lock;
 };
@@ -1449,10 +1337,10 @@ SCM_CLASS_DECL(Scm_GenericClass);
 
 #define SCM_DEFINE_GENERIC(cvar, cfunc, data)                           \
     ScmGeneric cvar = {                                                 \
-        SCM__PROCEDURE_INITIALIZER(SCM_CLASS2TAG(SCM_CLASS_GENERIC),    \
+        SCM__PROCEDURE_INITIALIZER(SCM_CLASS_STATIC_TAG(Scm_GenericClass),\
                                    0, 0, SCM_PROC_GENERIC, SCM_FALSE,   \
                                    NULL),                               \
-        SCM_NIL, cfunc, data                                            \
+        SCM_NIL, 0, cfunc, data                                         \
     }
 
 SCM_EXTERN void Scm_InitBuiltinGeneric(ScmGeneric *gf, const char *name,
@@ -1460,9 +1348,9 @@ SCM_EXTERN void Scm_InitBuiltinGeneric(ScmGeneric *gf, const char *name,
 SCM_EXTERN ScmObj Scm_MakeBaseGeneric(ScmObj name,
 				      ScmObj (*fallback)(ScmObj *, int, ScmGeneric*),
 				      void *data);
-SCM_EXTERN ScmObj Scm_NoNextMethod(ScmObj *args, int nargs, ScmGeneric *gf);
-SCM_EXTERN ScmObj Scm_NoOperation(ScmObj *args, int nargs, ScmGeneric *gf);
-SCM_EXTERN ScmObj Scm_InvalidApply(ScmObj *args, int nargs, ScmGeneric *gf);
+SCM_EXTERN ScmObj Scm_NoNextMethod(ScmObj *argv, int argc, ScmGeneric *gf);
+SCM_EXTERN ScmObj Scm_NoOperation(ScmObj *argv, int argc, ScmGeneric *gf);
+SCM_EXTERN ScmObj Scm_InvalidApply(ScmObj *argv, int argc, ScmGeneric *gf);
 
 /* Method - method
    A method can be defined either by C or by Scheme.  C-defined method
@@ -1472,7 +1360,7 @@ struct ScmMethodRec {
     ScmProcedure common;
     ScmGeneric *generic;
     ScmClass **specializers;    /* array of specializers, size==required */
-    ScmObj (*func)(ScmNextMethod *nm, ScmObj *args, int nargs, void * data);
+    ScmObj (*func)(ScmNextMethod *nm, ScmObj *argv, int argc, void * data);
     void *data;                 /* closure, or code */
     ScmEnvFrame *env;           /* environment (for Scheme created method) */
 };
@@ -1484,7 +1372,7 @@ SCM_CLASS_DECL(Scm_MethodClass);
 
 #define SCM_DEFINE_METHOD(cvar, gf, req, opt, specs, func, data)        \
     ScmMethod cvar = {                                                  \
-        SCM__PROCEDURE_INITIALIZER(SCM_CLASS2TAG(SCM_CLASS_METHOD),     \
+        SCM__PROCEDURE_INITIALIZER(SCM_CLASS_STATIC_TAG(Scm_MethodClass),\
                                    req, opt, SCM_PROC_METHOD,           \
                                    SCM_FALSE, NULL),                    \
         gf, specs, func, data, NULL                                     \
@@ -1498,9 +1386,10 @@ SCM_EXTERN void Scm_InitBuiltinMethod(ScmMethod *m);
 struct ScmNextMethodRec {
     ScmProcedure common;
     ScmGeneric *generic;
-    ScmObj methods;             /* list of applicable methods */
-    ScmObj *args;               /* original arguments */
-    int nargs;                  /* # of original arguments */
+    ScmObj methods;          /* list of applicable methods */
+    ScmObj *argv;            /* original arguments */
+    int argc;                /* # of original arguments */
+    int applyargs;           /* if TRUE, argv[argc-1] has a list of rest args */
 };
 
 SCM_CLASS_DECL(Scm_NextMethodClass);
@@ -1700,6 +1589,9 @@ SCM_EXTERN void   Scm_SetMasterSigmask(sigset_t *set);
 SCM_EXTERN ScmObj Scm_SignalName(int signum);
 SCM_EXTERN void   Scm_ResetSignalHandlers(sigset_t *mask);
 
+SCM_EXTERN void   Scm_GetSigmask(sigset_t *mask);
+SCM_EXTERN void   Scm_SetSigmask(sigset_t *mask);
+
 /*---------------------------------------------------
  * SYSTEM
  */
@@ -1733,6 +1625,7 @@ SCM_EXTERN void Scm_Abort(const char *msg);
 SCM_EXTERN void Scm_Panic(const char *msg, ...);
 
 SCM_EXTERN void Scm_GC(void);
+SCM_EXTERN void Scm_PrintStaticRoots(void);
 SCM_EXTERN void Scm_RegisterDL(void *data_start, void *data_end,
                                void *bss_start, void *bss_end);
 SCM_EXTERN void Scm_GCSentinel(void *obj, const char *name);
