@@ -1,7 +1,7 @@
 /*
  * class.h - Gauche object system private header
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: class.h,v 1.47 2007/03/22 11:20:31 shirok Exp $
+ *  $Id: class.h,v 1.51 2008-05-10 13:36:25 shirok Exp $
  */
 
 #ifndef GAUCHE_CLASS_H
@@ -74,16 +74,16 @@ struct ScmClassStaticSlotSpecRec {
     ScmSlotAccessor accessor;
 };
 
-#define SCM_CLASS_SLOT_SPEC(name, getter, setter)       \
-    { name, { {SCM_CLASS2TAG(SCM_CLASS_SLOT_ACCESSOR)}, \
-              NULL, NULL,                               \
-              (ScmNativeGetterProc)getter,              \
-              (ScmNativeSetterProc)setter,              \
-              SCM_UNBOUND,                              \
-              SCM_FALSE,                                \
-              SCM_FALSE,                                \
-              TRUE, -1,                                 \
-              SCM_FALSE, SCM_FALSE, SCM_FALSE,          \
+#define SCM_CLASS_SLOT_SPEC(name, getter, setter)               \
+    { name, { {SCM_CLASS_STATIC_TAG(Scm_SlotAccessorClass)},    \
+              NULL, NULL,                                       \
+              (ScmNativeGetterProc)getter,                      \
+              (ScmNativeSetterProc)setter,                      \
+              SCM_UNBOUND,                                      \
+              SCM_FALSE,                                        \
+              SCM_FALSE,                                        \
+              TRUE, -1,                                         \
+              SCM_FALSE, SCM_FALSE, SCM_FALSE,                  \
              } }
 
 #define SCM_CLASS_SLOT_SPEC_END()   { NULL }
@@ -111,11 +111,13 @@ SCM_EXTERN ScmObj Scm_ObjectAllocate(ScmClass *klass, ScmObj initargs);
 SCM_EXTERN ScmObj Scm_AllocateInstance(ScmClass *klass, int coresize);
 SCM_EXTERN ScmObj Scm_ComputeCPL(ScmClass *klass);
 SCM_EXTERN ScmObj Scm_ComputeApplicableMethods(ScmGeneric *gf,
-					       ScmObj *args,
-					       int nargs);
-SCM_EXTERN ScmObj Scm_SortMethods(ScmObj methods, ScmObj *args, int nargs);
+                                               ScmObj *argv,
+                                               int argc,
+                                               int applyargs);
+SCM_EXTERN ScmObj Scm_SortMethods(ScmObj methods, ScmObj *argv, int argc);
 SCM_EXTERN ScmObj Scm_MakeNextMethod(ScmGeneric *gf, ScmObj methods,
-				     ScmObj *args, int nargs, int copyArgs);
+                                     ScmObj *argv, int argc,
+                                     int copyargs, int applyargs);
 SCM_EXTERN ScmObj Scm_AddMethod(ScmGeneric *gf, ScmMethod *method);
 SCM_EXTERN ScmObj Scm_DeleteMethod(ScmGeneric *gf, ScmMethod *method);
 
@@ -123,11 +125,11 @@ SCM_EXTERN ScmObj Scm_VMSlotInitializeUsingAccessor(ScmObj obj,
                                                     ScmSlotAccessor *ca,
                                                     ScmObj initargs);
 SCM_EXTERN ScmObj Scm_VMSlotRefUsingAccessor(ScmObj obj,
-					     ScmSlotAccessor *acc,
-					     int boundp);
+                                             ScmSlotAccessor *acc,
+                                             int boundp);
 SCM_EXTERN ScmObj Scm_VMSlotSetUsingAccessor(ScmObj obj,
-					     ScmSlotAccessor *acc,
-					     ScmObj val);
+                                             ScmSlotAccessor *acc,
+                                             ScmObj val);
 
 SCM_EXTERN ScmObj Scm_VMClassOf(ScmObj obj);
 SCM_EXTERN ScmObj Scm_VMIsA(ScmObj obj, ScmClass *klass);
