@@ -1,7 +1,7 @@
 /*
  * hash.c - hash table implementation
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: hash.c,v 1.53 2007/04/17 14:43:05 shirok Exp $
+ *  $Id: hash.c,v 1.60 2008-05-10 13:36:19 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -41,14 +41,16 @@
  * Internal structures
  */
 
+
 /* The beginning of this structure must match ScmDictEntry. */
 typedef struct EntryRec {
     intptr_t key;
     intptr_t value;
     struct EntryRec *next;
+    u_long   hashval;
 } Entry;
 
-#define BUCKET(hc)   ((Entry**)hc->buckets)
+#define BUCKETS(hc)   ((Entry**)hc->buckets)
 
 #define DEFAULT_NUM_BUCKETS    4
 #define MAX_AVG_CHAIN_LIMITS   3
@@ -61,7 +63,6 @@ typedef struct EntryRec {
 typedef Entry *SearchProc(ScmHashCore *core, intptr_t key, ScmDictOp op);
 
 static unsigned int round2up(unsigned int val);
-static void check_scm_hashtable(ScmHashTable *table);
 
 /*============================================================
  * Hash functions
@@ -99,7 +100,7 @@ static void check_scm_hashtable(ScmHashTable *table);
     (result) = ((val)*2654435761UL)
 
 #define ADDRESS_HASH(result, val) \
-    (result) = ((SCM_WORD(val) >> 3)*2654435761UL)
+    (result) = (u_long)((SCM_WORD(val) >> 3)*2654435761UL)
 
 /* HASH2INDEX
    Map a hash value to bucket number.
@@ -111,38 +112,38 @@ static void check_scm_hashtable(ScmHashTable *table);
 /* Combining two hash values. */
 #define COMBINE(hv1, hv2)   ((hv1)*5+(hv2))
 
-unsigned long Scm_EqHash(ScmObj obj)
+u_long Scm_EqHash(ScmObj obj)
 {
-    unsigned long hashval;
+    u_long hashval;
     ADDRESS_HASH(hashval, obj);
     return hashval&HASHMASK;
 }
 
-unsigned long Scm_EqvHash(ScmObj obj)
+u_long Scm_EqvHash(ScmObj obj)
 {
-    unsigned long hashval;
+    u_long hashval;
     if (SCM_NUMBERP(obj)) {
         if (SCM_INTP(obj)) {
             SMALL_INT_HASH(hashval, SCM_INT_VALUE(obj));
         } else if (SCM_BIGNUMP(obj)) {
-            int i;
-            unsigned long u = 0;
+            u_int i;
+            u_long u = 0;
             for (i=0; i<SCM_BIGNUM_SIZE(obj); i++) {
                 u += SCM_BIGNUM(obj)->values[i];
             }
             SMALL_INT_HASH(hashval, u);
         } else if (SCM_FLONUMP(obj)) {
             /* TODO: I'm not sure this is a good hash. */
-            hashval = (unsigned long)(SCM_FLONUM_VALUE(obj)*2654435761UL);
+            hashval = (u_long)(SCM_FLONUM_VALUE(obj)*2654435761UL);
         } else if (SCM_RATNUMP(obj)) {
             /* Ratnum must be normalized, so we can simply combine
                hashvals of numerator and denominator. */
-            unsigned long h1 = Scm_EqvHash(SCM_RATNUM_NUMER(obj));
-            unsigned long h2 = Scm_EqvHash(SCM_RATNUM_DENOM(obj));
+            u_long h1 = Scm_EqvHash(SCM_RATNUM_NUMER(obj));
+            u_long h2 = Scm_EqvHash(SCM_RATNUM_DENOM(obj));
             hashval = COMBINE(h1, h2);
         } else {
             /* TODO: I'm not sure this is a good hash. */
-            hashval = (unsigned long)((SCM_COMPNUM_REAL(obj)+SCM_COMPNUM_IMAG(obj))*2654435761UL);
+            hashval = (u_long)((SCM_COMPNUM_REAL(obj)+SCM_COMPNUM_IMAG(obj))*2654435761UL);
         }
     } else {
         ADDRESS_HASH(hashval, obj);
@@ -151,18 +152,18 @@ unsigned long Scm_EqvHash(ScmObj obj)
 }
 
 /* General hash function */
-unsigned long Scm_Hash(ScmObj obj)
+u_long Scm_Hash(ScmObj obj)
 {
-    unsigned long hashval;
+    u_long hashval;
     if (!SCM_PTRP(obj)) {
-        SMALL_INT_HASH(hashval, (unsigned long)obj);
+        SMALL_INT_HASH(hashval, (u_long)SCM_WORD(obj));
         return hashval;
     } else if (SCM_NUMBERP(obj)) {
         return Scm_EqvHash(obj);
     } else if (SCM_STRINGP(obj)) {
         goto string_hash;
     } else if (SCM_PAIRP(obj)) {
-        unsigned long h = 0, h2;
+        u_long h = 0, h2;
         ScmObj cp;
         SCM_FOR_EACH(cp, obj) {
             h2 = Scm_Hash(SCM_CAR(cp));
@@ -173,7 +174,7 @@ unsigned long Scm_Hash(ScmObj obj)
         return h;
     } else if (SCM_VECTORP(obj)) {
         int i, siz = SCM_VECTOR_SIZE(obj);
-        unsigned long h = 0, h2;
+        u_long h = 0, h2;
         for (i=0; i<siz; i++) {
             h2 = Scm_Hash(SCM_VECTOR_ELEMENT(obj, i));
             h = COMBINE(h, h2);
@@ -190,7 +191,7 @@ unsigned long Scm_Hash(ScmObj obj)
         ScmObj r = Scm_ApplyRec(SCM_OBJ(&Scm_GenericObjectHash),
                                 SCM_LIST1(obj));
         if (SCM_INTP(r)) {
-            return (unsigned long)SCM_INT_VALUE(r);
+            return (u_long)SCM_INT_VALUE(r);
         }
         if (SCM_BIGNUMP(r)) {
             /* NB: Scm_GetUInteger clamps the result to [0, ULONG_MAX],
@@ -210,9 +211,9 @@ unsigned long Scm_Hash(ScmObj obj)
     }
 }
 
-unsigned long Scm_HashString(ScmString *str, unsigned long modulo)
+u_long Scm_HashString(ScmString *str, u_long modulo)
 {
-    unsigned long hashval;
+    u_long hashval;
     const char *p;
     const ScmStringBody *b = SCM_STRING_BODY(str);
     p = SCM_STRING_BODY_START(b);
@@ -230,12 +231,11 @@ unsigned long Scm_HashString(ScmString *str, unsigned long modulo)
  *
  * For the pre-defined simple hash tables, the calls to the hash and
  * compare functions are inlined in a single "access" function.
- * (In this case hashfn is only used for rehashing, and cmpfn is
- * never used).
+ * (In this case hashfn and cmpfn are never used.)
  * For the generic hash tables, the general_access function uses
  * the info in hashfn and cmpfn fields.
  *
- * The accessor function takes four arguments.
+ * The accessor function takes three arguments.
  *
  *     ScmHashCore *core   : hash table core
  *     intptr_t key        : key
@@ -255,13 +255,15 @@ unsigned long Scm_HashString(ScmString *str, unsigned long modulo)
  */
 static Entry *insert_entry(ScmHashCore *table,
                            intptr_t key,
+                           u_long   hashval,
                            int index)
 {
     Entry *e = SCM_NEW(Entry);
-    Entry **buckets = (Entry**)table->buckets;
+    Entry **buckets = BUCKETS(table);
     e->key = key;
     e->value = 0;
     e->next = buckets[index];
+    e->hashval = hashval;
     buckets[index] = e;
     table->numEntries++;
 
@@ -277,8 +279,7 @@ static Entry *insert_entry(ScmHashCore *table,
         
         Scm_HashIterInit(&iter, table);
         while ((f = (Entry*)Scm_HashIterNext(&iter)) != NULL) {
-            unsigned long hashval = table->hashfn(table, f->key);
-            index = HASH2INDEX(newsize, newbits, hashval);
+            index = HASH2INDEX(newsize, newbits, f->hashval);
             f->next = newb[index];
             newb[index] = f;
         }
@@ -289,6 +290,11 @@ static Entry *insert_entry(ScmHashCore *table,
     return e;
 }
 
+/* NB: Deleting entry E doesn't modify E's key and value, but cut
+   the "next" link for the sake of weak-gc robustness.  The hash core
+   iterator prefetches a pointer to the next entry, so deleting the
+   "current" entry of iteration is safe as far as other iterators
+   are running on the same hash table. */
 static Entry *delete_entry(ScmHashCore *table,
                            Entry *entry, Entry *prev,
                            int index)
@@ -312,10 +318,13 @@ static Entry *delete_entry(ScmHashCore *table,
         }                                               \
     } while (0)
 
-#define NOTFOUND(table, op, key, index)                                 \
-    do {                                                                \
-        if (op == SCM_DICT_CREATE) return insert_entry(table, key, index);\
-        else return NULL;                                               \
+#define NOTFOUND(table, op, key, hashval, index)                \
+    do {                                                        \
+        if (op == SCM_DICT_CREATE) {                            \
+           return insert_entry(table, key, hashval, index);     \
+        } else {                                                \
+           return NULL;                                         \
+        }                                                       \
     } while (0)
 
 /*
@@ -325,7 +334,7 @@ static Entry *address_access(ScmHashCore *table,
                              intptr_t key,
                              ScmDictOp op)
 {
-    unsigned long hashval, index;
+    u_long hashval, index;
     Entry *e, *p, **buckets = (Entry**)table->buckets;
 
     ADDRESS_HASH(hashval, key);
@@ -334,36 +343,41 @@ static Entry *address_access(ScmHashCore *table,
     for (e = buckets[index], p = NULL; e; p = e, e = e->next) {
         if (e->key == key) FOUND(table, op, e, p, index);
     }
-    NOTFOUND(table, op, key, index);
+    NOTFOUND(table, op, key, hashval, index);
 }
 
-static unsigned long address_hash(ScmHashCore *ht, intptr_t obj)
+static u_long address_hash(const ScmHashCore *ht, intptr_t obj)
 {
-    unsigned long hashval;
+    u_long hashval;
     ADDRESS_HASH(hashval, obj);
     return hashval;
 }
 
+static int address_cmp(const ScmHashCore *ht, intptr_t key, intptr_t k2)
+{
+    return (key == k2);
+}
+
 /*
  * Accessor function for equal and eqv-hash.
  * We assume KEY is ScmObj.
  */
-static unsigned long eqv_hash(ScmHashCore *table, intptr_t key)
+static u_long eqv_hash(const ScmHashCore *table, intptr_t key)
 {
     return Scm_EqvHash(SCM_OBJ(key));
 }
 
-static int eqv_cmp(ScmHashCore *table, intptr_t key, intptr_t k2)
+static int eqv_cmp(const ScmHashCore *table, intptr_t key, intptr_t k2)
 {
     return Scm_EqvP(SCM_OBJ(key), SCM_OBJ(k2));
 }
 
-static unsigned long equal_hash(ScmHashCore *table, intptr_t key)
+static u_long equal_hash(const ScmHashCore *table, intptr_t key)
 {
     return Scm_Hash(SCM_OBJ(key));
 }
 
-static int equal_cmp(ScmHashCore *table, intptr_t key, intptr_t k2)
+static int equal_cmp(const ScmHashCore *table, intptr_t key, intptr_t k2)
 {
     return Scm_EqualP(SCM_OBJ(key), SCM_OBJ(k2));
 }
@@ -374,7 +388,7 @@ static int equal_cmp(ScmHashCore *table, intptr_t key, intptr_t k2)
  */
 static Entry *string_access(ScmHashCore *table, intptr_t k, ScmDictOp op)
 {
-    unsigned long hashval, index;
+    u_long hashval, index;
     int size;
     const char *s;
     ScmObj key = SCM_OBJ(k);
@@ -382,8 +396,7 @@ static Entry *string_access(ScmHashCore *table, intptr_t k, ScmDictOp op)
     const ScmStringBody *keyb;
     
     if (!SCM_STRINGP(key)) {
-        Scm_Error("Got non-string key %S to the string hashtable %S",
-                  key, table);
+        Scm_Error("Got non-string key %S to the string hashtable.", key);
     }
     keyb = SCM_STRING_BODY(key);
     s = SCM_STRING_BODY_START(keyb);
@@ -402,12 +415,12 @@ static Entry *string_access(ScmHashCore *table, intptr_t k, ScmDictOp op)
             FOUND(table, op, e, p, index);
         }
     }
-    NOTFOUND(table, op, k, index);
+    NOTFOUND(table, op, k, hashval, index);
 }
 
-static unsigned long string_hash(ScmHashCore *table, intptr_t key)
+static u_long string_hash(const ScmHashCore *table, intptr_t key)
 {
-    unsigned long hashval;
+    u_long hashval;
     const char *p;
     const ScmStringBody *b = SCM_STRING_BODY(key);
     p = SCM_STRING_BODY_START(b);
@@ -415,15 +428,25 @@ static unsigned long string_hash(ScmHashCore *table, intptr_t key)
     return hashval;
 }
 
+static int string_cmp(const ScmHashCore *table, intptr_t k1, intptr_t k2)
+{
+    const ScmStringBody *b1 = SCM_STRING_BODY(k1);
+    const ScmStringBody *b2 = SCM_STRING_BODY(k2);
+    return ((SCM_STRING_BODY_SIZE(b1) == SCM_STRING_BODY_SIZE(b2))
+            && (memcmp(SCM_STRING_BODY_START(b1),
+                       SCM_STRING_BODY_START(b2),
+                       SCM_STRING_BODY_SIZE(b1)) == 0));
+}
+
 /*
  * Accessor function for multiword raw hashtable.
  * Key points to an array of N words.
  */
-static unsigned long multiword_hash(ScmHashCore *table, intptr_t key)
+static u_long multiword_hash(const ScmHashCore *table, intptr_t key)
 {
     ScmWord keysize = (ScmWord)table->data;
     ScmWord *keyarray = (ScmWord*)key;
-    unsigned long h = 0, h1;
+    u_long h = 0, h1;
     int i;
     for (i=0; i<keysize; i++) {
         ADDRESS_HASH(h1, keyarray[i]);
@@ -432,9 +455,10 @@ static unsigned long multiword_hash(ScmHashCore *table, intptr_t key)
     return h;
 }
 
+#if 0
 static Entry *multiword_access(ScmHashCore *table, intptr_t k, ScmDictOp op)
 {
-    unsigned long hashval, index;
+    u_long hashval, index;
     ScmWord keysize = (ScmWord)table->data;
     Entry *e, *p, **buckets;
     
@@ -446,8 +470,9 @@ static Entry *multiword_access(ScmHashCore *table, intptr_t k, ScmDictOp op)
         if (memcmp((void*)k, (void*)e->key, keysize*sizeof(ScmWord)) == 0)
             FOUND(table, op, e, p, index);
     }
-    NOTFOUND(table, op, k, index);
+    NOTFOUND(table, op, k, hashval, index);
 }
+#endif
 
 
 /*
@@ -456,7 +481,7 @@ static Entry *multiword_access(ScmHashCore *table, intptr_t k, ScmDictOp op)
  */
 static Entry *general_access(ScmHashCore *table, intptr_t key, ScmDictOp op)
 {
-    unsigned long hashval, index;
+    u_long hashval, index;
     Entry *e, *p, **buckets;
 
     hashval = table->hashfn(table, key);
@@ -466,7 +491,7 @@ static Entry *general_access(ScmHashCore *table, intptr_t key, ScmDictOp op)
     for (e = buckets[index], p = NULL; e; p = e, e = e->next) {
         if (table->cmpfn(table, key, e->key)) FOUND(table, op, e, p, index);
     }
-    NOTFOUND(table, op, key, index);
+    NOTFOUND(table, op, key, hashval, index);
 }
 
 /*============================================================
@@ -474,14 +499,14 @@ static Entry *general_access(ScmHashCore *table, intptr_t key, ScmDictOp op)
  */
 
 static void hash_core_init(ScmHashCore *table,
-                           SearchProc accessfn,
-                           ScmHashProc hashfn,
-                           ScmHashCompareProc cmpfn,
+                           SearchProc  *accessfn,
+                           ScmHashProc *hashfn,
+                           ScmHashCompareProc *cmpfn,
                            unsigned int initSize,
                            void *data)
 {
     Entry **b;
-    int i;
+    u_int i;
     
     if (initSize != 0) initSize = round2up(initSize);
     else initSize = DEFAULT_NUM_BUCKETS;
@@ -500,35 +525,70 @@ static void hash_core_init(ScmHashCore *table,
     for (i=0; i<initSize; i++) table->buckets[i] = NULL;
 }
 
+/* choose appropriate procedures for predefined hash types. */
+int  hash_core_predef_procs(ScmHashType type,
+                            SearchProc  **accessfn,
+                            ScmHashProc **hashfn,
+                            ScmHashCompareProc **cmpfn)
+{
+    switch (type) {
+    case SCM_HASH_EQ:
+    case SCM_HASH_WORD:
+        *accessfn = address_access;
+        *hashfn = address_hash;
+        *cmpfn  = address_cmp;
+        return TRUE;
+    case SCM_HASH_EQV:
+        *accessfn = general_access;
+        *hashfn = eqv_hash;
+        *cmpfn  = eqv_cmp;
+        return TRUE;
+    case SCM_HASH_EQUAL:
+        *accessfn = general_access;
+        *hashfn = equal_hash;
+        *cmpfn  = equal_cmp;
+        return TRUE;
+    case SCM_HASH_STRING:
+        *accessfn = string_access;
+        *hashfn = string_hash;
+        *cmpfn  = string_cmp;
+        return TRUE;
+    default:
+        return FALSE;
+    }
+}
+
 void Scm_HashCoreInitSimple(ScmHashCore *core,
                             ScmHashType type,
                             unsigned int initSize,
                             void *data)
 {
-    switch (type) {
-    case SCM_HASH_EQ:
-        hash_core_init(core, address_access, address_hash, NULL,
-                       initSize, data);
-        break;
-    case SCM_HASH_EQV:
-        hash_core_init(core, general_access, eqv_hash, eqv_cmp,
-                       initSize, data);
-        break;
-    case SCM_HASH_EQUAL:
-        hash_core_init(core, general_access, equal_hash, equal_cmp,
-                       initSize, data);
-        break;
-    case SCM_HASH_STRING:
-        hash_core_init(core, string_access, string_hash, NULL,
-                       initSize, data);
-        break;
-    case SCM_HASH_WORD:
-        hash_core_init(core, address_access, address_hash, NULL,
-                       initSize, data);
-        break;
-    default:    
+    SearchProc  *accessfn;
+    ScmHashProc *hashfn;
+    ScmHashCompareProc *cmpfn;
+
+    if (hash_core_predef_procs(type, &accessfn, &hashfn, &cmpfn) == FALSE) {
         Scm_Error("[internal error]: wrong TYPE argument passed to Scm_HashCoreInitSimple: %d", type);
     }
+    hash_core_init(core, accessfn, hashfn, cmpfn, initSize, data);
+}
+
+void Scm_HashCoreInitGeneral(ScmHashCore *core,
+                             ScmHashProc *hashfn,
+                             ScmHashCompareProc *cmpfn,
+                             unsigned int initSize,
+                             void *data)
+{
+    hash_core_init(core, general_access, hashfn,
+                   cmpfn, initSize, data);
+}
+
+int Scm_HashCoreTypeToProcs(ScmHashType type,
+                            ScmHashProc **hashfn,
+                            ScmHashCompareProc **cmpfn)
+{
+    SearchProc *accessfn;       /* dummy */
+    return hash_core_predef_procs(type, &accessfn, hashfn, cmpfn);
 }
 
 void Scm_HashCoreCopy(ScmHashCore *dst, const ScmHashCore *src)
@@ -569,7 +629,6 @@ void Scm_HashCoreCopy(ScmHashCore *dst, const ScmHashCore *src)
 void Scm_HashCoreClear(ScmHashCore *table)
 {
     int i;
-    Entry *e;
     for (i=0; i<table->numBuckets; i++) {
         table->buckets[i] = NULL;
     }
@@ -588,6 +647,11 @@ int Scm_HashCoreNumEntries(ScmHashCore *table)
     return table->numEntries;
 }
 
+/*
+ * NB: It is important to keep the pointer to the "next" entry,
+ * not the "current", since the current entry may be deleted,
+ * erasing its next pointer.
+ */
 void Scm_HashIterInit(ScmHashIter *iter, ScmHashCore *table)
 {
     int i;
@@ -716,7 +780,7 @@ ScmObj Scm_HashTableStat(ScmHashTable *table)
     ScmHashCore *c = SCM_HASH_TABLE_CORE(table);
     ScmVector *v = SCM_VECTOR(Scm_MakeVector(c->numBuckets, SCM_NIL));
     ScmObj *vp;
-    Entry** b = BUCKET(c);
+    Entry** b = BUCKETS(c);
     int i;
     
     SCM_APPEND1(h, t, SCM_MAKE_KEYWORD("num-entries"));
@@ -809,9 +873,9 @@ ScmHashEntry *Scm_HashTableAdd(ScmHashTable *ht, ScmObj key, ScmObj value)
     ScmDictEntry *e = Scm_HashCoreSearch(SCM_HASH_TABLE_CORE(ht),
                                          (intptr_t)key, SCM_DICT_CREATE);
     if (sizeof(intptr_t) != sizeof(void*)) {
-        Scm_Error("[internal] Scm_HashTableGet is obsoleted on this platform.  You should use the new hashtable API.");
+        Scm_Error("[internal] Scm_HashTableAdd is obsoleted on this platform.  You should use the new hashtable API.");
     }
-    if (!e->value) SCM_DICT_SET_VALUE(e, value);
+    if (!e->value) (void)SCM_DICT_SET_VALUE(e, value);
     return (ScmHashEntry*)e;
 }
 
@@ -820,9 +884,9 @@ ScmHashEntry *Scm_HashTablePut(ScmHashTable *ht, ScmObj key, ScmObj value)
     ScmDictEntry *e = Scm_HashCoreSearch(SCM_HASH_TABLE_CORE(ht),
                                          (intptr_t)key, SCM_DICT_CREATE);
     if (sizeof(intptr_t) != sizeof(void*)) {
-        Scm_Error("[internal] Scm_HashTableGet is obsoleted on this platform.  You should use the new hashtable API.");
+        Scm_Error("[internal] Scm_HashTablePut is obsoleted on this platform.  You should use the new hashtable API.");
     }
-    SCM_DICT_SET_VALUE(e, value);
+    (void)SCM_DICT_SET_VALUE(e, value);
     return (ScmHashEntry*)e;
 }
 
@@ -886,6 +950,8 @@ ScmObj Scm_MakeHashTable(ScmHashProc *hashfn,
         return Scm_MakeHashTableFull(SCM_CLASS_HASH_TABLE, SCM_HASH_GENERAL,
                                      hashfn, cmpfn, initSize, NULL);
     }
+#else
+    return SCM_UNDEFINED;
 #endif
 }
 
