@@ -1,7 +1,7 @@
 /*
  * load.c - load a program
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,28 +30,26 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: load.c,v 1.113 2007/04/12 03:26:55 shirok Exp $
+ *  $Id: load.c,v 1.123 2008-05-21 10:46:37 shirok Exp $
  */
 
-#include <stdlib.h>
-#include <unistd.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <string.h>
-#include <ctype.h>
-#include <fcntl.h>
 #define LIBGAUCHE_BODY
 #include "gauche.h"
 #include "gauche/arch.h"
 #include "gauche/port.h"
 #include "gauche/builtin-syms.h"
 
+#include <ctype.h>
+#include <fcntl.h>
+
 #define LOAD_SUFFIX ".scm"      /* default load suffix */
 
 /*
  * Load file.
  */
 
+typedef struct dlobj_rec dlobj;
+
 /* Static parameters */
 static struct {
     /* Load path list */
@@ -72,7 +70,7 @@ static struct {
 
     /* Dynamic linking */
     ScmObj dso_suffixes;
-    ScmObj dso_list;              /* List of dynamically loaded objects. */
+    dlobj *dso_list;              /* List of dynamically loaded objects. */
     ScmInternalMutex dso_mutex;
 } ldinfo = { (ScmGloc*)&ldinfo, };  /* trick to put ldinfo in .data section */
 
@@ -117,7 +115,7 @@ static void load_packet_init(ScmLoadPacket *packet)
 struct load_packet {
     ScmPort *port;
     ScmModule *prev_module;
-    ScmReadContext ctx;
+    ScmReadContext *ctx;
     ScmObj prev_port;
     ScmObj prev_history;
     ScmObj prev_next;
@@ -154,7 +152,7 @@ static ScmObj load_after(ScmObj *args, int nargs, void *data)
 static ScmObj load_cc(ScmObj result, void **data)
 {
     struct load_packet *p = (struct load_packet*)(data[0]);
-    ScmObj expr = Scm_ReadWithContext(SCM_OBJ(p->port), &(p->ctx));
+    ScmObj expr = Scm_ReadWithContext(SCM_OBJ(p->port), p->ctx);
 
     if (!SCM_EOFP(expr)) {
         Scm_VMPushCC(load_cc, data, 1);
@@ -198,11 +196,8 @@ ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
     p->prev_next = vm->load_next;
     p->prev_situation = vm->evalSituation;
 
-    SCM_READ_CONTEXT_INIT(&(p->ctx));
-    p->ctx.flags = SCM_READ_LITERAL_IMMUTABLE | SCM_READ_SOURCE_INFO;
-    if (SCM_VM_RUNTIME_FLAG_IS_SET(vm, SCM_CASE_FOLD)) {
-        p->ctx.flags |= SCM_READ_CASE_FOLD;
-    }
+    p->ctx = Scm_MakeReadContext(NULL);
+    p->ctx->flags = SCM_READ_LITERAL_IMMUTABLE | SCM_READ_SOURCE_INFO;
     
     vm->load_next = next_paths;
     vm->load_port = SCM_OBJ(port);
@@ -318,10 +313,10 @@ ScmObj Scm_FindFile(ScmString *filename, ScmObj *paths,
     } else if (*ptr == '/'
                || (*ptr == '.' && *(ptr+1) == '/')
                || (*ptr == '.' && *(ptr+1) == '.' && *(ptr+2) == '/')
-#if defined(__CYGWIN__) || defined(__MINGW32__)
+#if defined(__CYGWIN__) || defined(GAUCHE_WINDOWS)
 	       /* support for wicked legacy DOS drive letter */
 	       || (isalpha(*ptr) && *(ptr+1) == ':')
-#endif /* __CYGWIN__ || __MINGW32__ */
+#endif /* __CYGWIN__ || GAUCHE_WINDOWS */
 	       ) {
         use_load_paths = FALSE;
     }
@@ -494,11 +489,11 @@ ScmObj Scm_GetDynLoadPath(void)
 static ScmObj break_env_paths(const char *envname)
 {
     const char *e = getenv(envname);
-#ifndef __MINGW32__
+#ifndef GAUCHE_WINDOWS
     char delim = ':';
-#else  /*__MINGW32__*/
+#else  /*GAUCHE_WINDOWS*/
     char delim = ';';
-#endif /*__MINGW32__*/
+#endif /*GAUCHE_WINDOWS*/
 
     if (e == NULL) {
 	return SCM_NIL;
@@ -566,7 +561,7 @@ ScmObj Scm_AddLoadPath(const char *cpath, int afterp)
 }
 
 /*------------------------------------------------------------------
- * Dynamic link
+ * Dynamic linking
  */
 
 /* The API to load object file dynamically differ among platforms.
@@ -613,12 +608,30 @@ ScmObj Scm_AddLoadPath(const char *cpath, int afterp)
 
 typedef void (*ScmDynLoadInitFn)(void);
 
+enum dlobj_state {
+    DLOBJ_NONE,             /* dlopen and dlsym haven't completed. */
+    DLOBJ_LOADED,           /* dlopened and initfn found, but not init'ed */
+    DLOBJ_INITIALIZED       /* initialized and ready to use */
+};
+
+struct dlobj_rec {
+    dlobj *next;                /* chain */
+    const char *path;           /* pathname for DSO, including suffix */
+    enum dlobj_state state;     /* state of loading */
+    void *handle;               /* whatever dl_open returned */
+    ScmVM *loader;              /* VM who's loading this.  NULL once done. */
+    ScmDynLoadInitFn initfn;    /* initialization fn */
+    ScmInternalMutex mutex;
+    ScmInternalCond  cv;
+};
+
+
 /* NB: we rely on dlcompat library for dlopen instead of using dl_darwin.c
    for now; Boehm GC requires dlopen when compiled with pthread, so there's
    not much point to avoid dlopen here. */
 #if defined(HAVE_DLOPEN)
 #include "dl_dlopen.c"
-#elif defined(__MINGW32__)
+#elif defined(GAUCHE_WINDOWS)
 #include "dl_win.c"
 #else
 #include "dl_dummy.c"
@@ -651,7 +664,35 @@ static const char *get_dynload_initfn(const char *filename)
     return name;
 }
 
-#if 0
+static dlobj *make_dlobj(const char *path)
+{
+    dlobj *z = SCM_NEW(dlobj);
+    z->next = NULL;
+    z->path = path;
+    z->loader = Scm_VM();
+    z->state = DLOBJ_NONE;
+    z->initfn = NULL;
+    (void)SCM_INTERNAL_MUTEX_INIT(z->mutex);
+    (void)SCM_INTERNAL_COND_INIT(z->cv);
+    return z;
+}
+
+static dlobj *find_or_add_dlobj(dlobj *newobj)
+{
+    dlobj *z = NULL;
+    (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.dso_mutex);
+    for (z = ldinfo.dso_list; z; z = z->next) {
+        if (strcmp(z->path, newobj->path) == 0) break;
+    }
+    if (z == NULL) {
+        newobj->next = ldinfo.dso_list;
+        ldinfo.dso_list = z = newobj;
+    }
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.dso_mutex);
+    return z;
+}
+
+#if 0  /* Turned off libtool *.la support.  Not sure if we need this. */
 /* Aux fn to get a parameter value from *.la file line */
 static const char *get_la_val(const char *start)
 {
@@ -726,14 +767,8 @@ static ScmObj find_so_from_la(ScmString *lafile)
 ScmObj Scm_DynLoad(ScmString *filename, ScmObj initfn, int export_)
 {
     ScmObj reqname, truename, load_paths = Scm_GetDynLoadPath();
-    void *handle;
-    ScmDynLoadInitFn func;
-    const char *cpath, *initname, *err = NULL;
-    enum  {
-        DLERR_NONE,             /* no error */
-        DLERR_DLOPEN,           /* failure in dlopen */
-        DLERR_NOINITFN,         /* failure in finding initfn */
-    } errtype = DLERR_NONE;
+    const char *cpath, *initname;
+    dlobj *newdlo, *dlo;
 
     truename = Scm_FindFile(filename, &load_paths, ldinfo.dso_suffixes, TRUE);
     if (SCM_FALSEP(truename)) {
@@ -742,7 +777,7 @@ ScmObj Scm_DynLoad(ScmString *filename, ScmObj initfn, int export_)
     reqname = truename;         /* save requested name */
     cpath = Scm_GetStringConst(SCM_STRING(truename));
 
-#if 0
+#if 0  /* Turned off libtool *.la support.  Not sure if we need this. */
     if ((suff = strrchr(cpath, '.')) && strcmp(suff, ".la") == 0) {
         truename = find_so_from_la(SCM_STRING(truename));
         if (SCM_FALSEP(truename)) {
@@ -765,66 +800,103 @@ ScmObj Scm_DynLoad(ScmString *filename, ScmObj initfn, int export_)
         initname = get_dynload_initfn(Scm_GetStringConst(SCM_STRING(reqname)));
     }
 
-    (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.dso_mutex);
-    if (!SCM_FALSEP(Scm_Member(truename, ldinfo.dso_list, SCM_CMP_EQUAL))) {
-        /* already loaded */
-        goto cleanup;
+    newdlo = make_dlobj(cpath);
+    dlo = find_or_add_dlobj(newdlo);
+
+    if (dlo != newdlo) {
+        /* somebody has tried to load this DSO. */
+        (void)SCM_INTERNAL_MUTEX_LOCK(dlo->mutex);
+        while (dlo->state != DLOBJ_INITIALIZED) {
+            if (dlo->loader == NULL) {
+                /* loading is abandoned.  we take it over. */
+                dlo->loader = newdlo->loader;  /* this is our VM */
+                break;
+            } else {
+                (void)SCM_INTERNAL_COND_WAIT(dlo->cv, dlo->mutex);
+            }
+        }
+        (void)SCM_INTERNAL_MUTEX_UNLOCK(dlo->mutex);
+
+        if (dlo->state == DLOBJ_INITIALIZED) {
+            /* it is already loaded. we just return. */
+            return SCM_TRUE;
+        }
+        /* FALLTHROUGH */
     }
+
+    /* At this moment, dlo is owned by our thread (either we're the
+       first one to load, or we took over the abandoned dlo).  Note that
+       only the loader can touch dlo->state; so it is safe to inspect
+       it here without locking it. */
     SCM_UNWIND_PROTECT {
         ScmVM *vm = Scm_VM();
-        if (SCM_VM_RUNTIME_FLAG_IS_SET(vm, SCM_LOAD_VERBOSE)) {
-            int len = Scm_Length(vm->load_history);
-            SCM_PUTZ(";;", 2, SCM_CURERR);
-            while (len-- > 0) SCM_PUTC(' ', SCM_CURERR);
-            Scm_Printf(SCM_CURERR, "Dynamically Loading %s...\n", cpath);
+
+        switch (dlo->state) {
+        case DLOBJ_NONE:
+            /* First, we dl_open the DSO. */
+            if (SCM_VM_RUNTIME_FLAG_IS_SET(vm, SCM_LOAD_VERBOSE)) {
+                int len = Scm_Length(vm->load_history);
+                SCM_PUTZ(";;", 2, SCM_CURERR);
+                while (len-- > 0) SCM_PUTC(' ', SCM_CURERR);
+                Scm_Printf(SCM_CURERR, "Dynamically Loading %s...\n", cpath);
+            }
+            dlo->handle = dl_open(cpath);
+            if (dlo->handle == NULL) {
+                const char *err = dl_error();
+                if (err == NULL) {
+                    Scm_Error("failed to link %S dynamically", filename);
+                } else {
+                    Scm_Error("failed to link %S dynamically: %s", filename, err);
+                }
+                /*NOTREACHED*/
+            }
+            /* locate initfn.  initname always has '_'.  Whether the actual
+               symbol dl_sym returns has '_' or not depends on the platform,
+               so we first try without '_', then '_'. */
+            dlo->initfn = dl_sym(dlo->handle, initname+1);
+            if (dlo->initfn == NULL) {
+                dlo->initfn = (void(*)(void))dl_sym(dlo->handle, initname);
+                if (dlo->initfn == NULL) {
+                    dl_close(dlo->handle);
+                    dlo->handle = NULL;
+                    Scm_Error("dynamic linking of %S failed: couldn't find initialization function %s", filename, initname);
+                    /*NOTREACHED*/
+                }
+            }
+            dlo->state = DLOBJ_LOADED;
+            /*FALLTHROUGH*/
+        case DLOBJ_LOADED:
+            SCM_ASSERT(dlo->initfn != NULL);
+            /* Call initialization function.  note that there can be arbitrary
+               complex stuff done within func(), including evaluation of
+               Scheme procedures and/or calling dynamic-load for other
+               object.  There's a chance that, with some contrived example,
+               func() can trigger the dynamic loading of the same file we're
+               loading right now.  However, if the code follows the Gauche's
+               standard module structure, such circular dependency is detected
+               by Scm_Load, so we don't worry about it here. */
+            dlo->initfn();
+            /* Alright.  Everything seems fine. */
+            dlo->state = DLOBJ_INITIALIZED;
+            /*FALLTHROUGH*/
+        default:
+            /*Nothing to do*/
+            break;
         }
     } SCM_WHEN_ERROR {
-        (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.dso_mutex);
+        (void)SCM_INTERNAL_MUTEX_LOCK(dlo->mutex);
+        dlo->loader = NULL;
+        (void)SCM_INTERNAL_COND_BROADCAST(dlo->cv);
+        (void)SCM_INTERNAL_MUTEX_UNLOCK(dlo->mutex);
         SCM_NEXT_HANDLER;
     } SCM_END_PROTECT;
-    handle = dl_open(cpath);
-    if (handle == NULL) {
-        err = dl_error();
-        errtype = DLERR_DLOPEN;
-        goto cleanup;
-    }
-    /* initname always has '_'.  We first try without '_' */
-    func = dl_sym(handle, initname+1);
-    if (func == NULL) {
-        func = (void(*)(void))dl_sym(handle, initname);
-        if (func == NULL) {
-            dl_close(handle);
-            errtype = DLERR_NOINITFN;
-            goto cleanup;
-        }
-    }
-    /* TODO: if the module initialization function fails,
-       there's no safe way to unload the module, and we
-       can't load the same module again.  We're stuck to
-       the broken module.  This has to be addressed. */
-    SCM_UNWIND_PROTECT {
-        func();
-    } SCM_WHEN_ERROR {
-        (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.dso_mutex);
-        SCM_NEXT_HANDLER;
-    } SCM_END_PROTECT;
-    ldinfo.dso_list = Scm_Cons(truename, ldinfo.dso_list);
-  cleanup:
-    (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.dso_mutex);
-    switch (errtype) {
-    case DLERR_DLOPEN:
-        if (err == NULL) {
-            Scm_Error("failed to link %S dynamically", filename);
-        } else {
-            Scm_Error("failed to link %S dynamically: %s", filename, err);
-        }
-        /*NOTREACHED*/
-    case DLERR_NOINITFN:
-        Scm_Error("dynamic linking of %S failed: couldn't find initialization function %s", filename, initname);
-        /*NOTREACHED*/
-    case DLERR_NONE:
-        break;
-    }
+
+    /* All is done. */
+    (void)SCM_INTERNAL_MUTEX_LOCK(dlo->mutex);
+    dlo->loader = NULL;
+    (void)SCM_INTERNAL_COND_BROADCAST(dlo->cv);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(dlo->mutex);
+
     return SCM_TRUE;
 }
 
@@ -894,7 +966,7 @@ int Scm_Require(ScmObj feature, int flags, ScmLoadPacket *packet)
         }
         
         for (;;) {
-            q = Scm_Assoc(SCM_CDR(p), ldinfo.waiting, SCM_CMP_EQ);
+            q = Scm_Assq(SCM_CDR(p), ldinfo.waiting);
             if (SCM_FALSEP(q)) break;
             SCM_ASSERT(SCM_PAIRP(q));
             p = Scm_Assoc(SCM_CDR(q), ldinfo.providing, SCM_CMP_EQUAL);
@@ -1041,18 +1113,34 @@ void Scm_DefineAutoload(ScmModule *where,
 }
 
 
-ScmObj Scm_LoadAutoload(ScmAutoload *adata)
+ScmObj Scm_ResolveAutoload(ScmAutoload *adata, int flags)
 {
-    int error = FALSE;
+    int circular = FALSE;
     ScmModule *prev_module;
     ScmVM *vm = Scm_VM();
     
-    /* check if some other thread already loaded this before attempt to lock */
-    if (adata->loaded) {
-        return adata->value;
+    /* shortcut in case if somebody else already did the job. */
+    if (adata->loaded) return adata->value;
+
+    /* check to see if this autoload is recursive.  if so, we just return
+       SCM_UNBOUND and let the caller handle the issue (NB: it isn't
+       necessarily an error.  For example, define-method searches if
+       a generic function of the same name is already defined; if the
+       name is set autoload and define-method is in the file that's being
+       autoloaded, define-method finds the name is an autoload that points
+       the currently autoloaded file.)
+       we have to be careful to exclude the case that when one thread is
+       resolving autoload another thread enters here and sees this autoload
+       is already being resolved.
+     */
+    if ((adata->locker == NULL || adata->locker == vm)
+        && !SCM_FALSEP(Scm_Assoc(SCM_OBJ(adata->path),
+                                 ldinfo.providing,
+                                 SCM_CMP_EQUAL))) {
+        return SCM_UNBOUND;
     }
 
-    /* obtain the right to load this autoload */
+    /* obtain the lock to load this autoload */
     (void)SCM_INTERNAL_MUTEX_LOCK(adata->mutex);
     do {
         if (adata->loaded) break;
@@ -1060,7 +1148,7 @@ ScmObj Scm_LoadAutoload(ScmAutoload *adata)
             adata->locker = vm;
         } else if (adata->locker == vm) {
             /* bad circular dependency */
-            error = TRUE;
+            circular = TRUE;
         } else if (adata->locker->state == SCM_VM_TERMINATED) {
             /* the loading thread have died prematurely.
                let's take over the task. */
@@ -1076,10 +1164,13 @@ ScmObj Scm_LoadAutoload(ScmAutoload *adata)
         return adata->value;
     }
     
-    if (error) {
+    if (circular) {
+        /* Since we have already checked recursive loading, it isn't normal
+           if we reach here.  Right now I have no idea how this happens, but
+           just in case we raise an error. */
         adata->locker = NULL;
-        SCM_INTERNAL_COND_SIGNAL(adata->cv);
-        Scm_Error("Circular autoload dependency involving %S::%S\n",
+        SCM_INTERNAL_COND_BROADCAST(adata->cv);
+        Scm_Error("Attempted to trigger the same autoload %S#%S recursively.  Maybe circular autoload dependency?\n",
                   adata->module, adata->name);
     }
 
@@ -1123,13 +1214,13 @@ ScmObj Scm_LoadAutoload(ScmAutoload *adata)
     } SCM_WHEN_ERROR {
         adata->locker = NULL;
         vm->module = prev_module;
-        SCM_INTERNAL_COND_SIGNAL(adata->cv);
+        SCM_INTERNAL_COND_BROADCAST(adata->cv);
         SCM_NEXT_HANDLER;
     } SCM_END_PROTECT;
 
     adata->loaded = TRUE;
     adata->locker = NULL;
-    SCM_INTERNAL_COND_SIGNAL(adata->cv);
+    SCM_INTERNAL_COND_BROADCAST(adata->cv);
     return adata->value;
 }
 
@@ -1208,5 +1299,5 @@ void Scm__InitLoad(void)
     ldinfo.waiting = SCM_NIL;
     ldinfo.dso_suffixes = SCM_LIST2(SCM_MAKE_STR(".la"),
                                     SCM_MAKE_STR("." SHLIB_SO_SUFFIX));
-    ldinfo.dso_list = SCM_NIL;
+    ldinfo.dso_list = NULL;
 }
