@@ -1,7 +1,7 @@
 /*
  * macro.c - macro implementation
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: macro.c,v 1.66 2007/03/02 07:39:13 shirok Exp $
+ *  $Id: macro.c,v 1.70 2008-05-10 13:36:19 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -219,7 +219,10 @@ ScmObj Scm_MakeMacroTransformerOld(ScmSymbol *name, ScmProcedure *proc)
 
 static ScmMacro *resolve_macro_autoload(ScmAutoload *adata)
 {
-    ScmObj mac = Scm_LoadAutoload(adata);
+    ScmObj mac = Scm_ResolveAutoload(adata, 0);
+    if (SCM_UNBOUNDP(mac)) {
+        Scm_Error("tried to autoload macro %S, but it caused circular autoload.", adata->name);
+    }
     if (!SCM_MACROP(mac)) {
         Scm_Error("tried to autoload macro %S, but it yields non-macro object: %S", adata->name, mac);
     }
@@ -281,8 +284,8 @@ typedef struct {
 } PatternContext;
 
 #define PVREF_P(pvref)         SCM_PVREF_P(pvref)
-#define PVREF_LEVEL(pvref)     SCM_PVREF_LEVEL(pvref)
-#define PVREF_COUNT(pvref)     SCM_PVREF_COUNT(pvref)
+#define PVREF_LEVEL(pvref)     (int)SCM_PVREF_LEVEL(pvref)
+#define PVREF_COUNT(pvref)     (int)SCM_PVREF_COUNT(pvref)
 
 /* add pattern variable pvar.  called when compiling a pattern */
 static inline ScmObj add_pvar(PatternContext *ctx,
@@ -948,16 +951,17 @@ ScmObj Scm_VMMacroExpand(ScmObj expr, ScmObj env, int oncep)
             /* local syntactic binding */
             mac = SCM_MACRO(sym);
         } else {
+            ScmGloc *g = NULL;
             if (SCM_IDENTIFIERP(sym)) {
-                sym = SCM_OBJ(SCM_IDENTIFIER(sym)->name);
+                g = Scm_FindBinding(SCM_IDENTIFIER(sym)->module,
+                                    SCM_IDENTIFIER(sym)->name, 0);
+            } else if (SCM_SYMBOLP(sym)) {
+                g = Scm_FindBinding(Scm_VM()->module,
+                                    SCM_SYMBOL(sym), 0);
             }
-            if (SCM_SYMBOLP(sym)) {
-                ScmGloc *g = Scm_FindBinding(Scm_VM()->module,
-                                             SCM_SYMBOL(sym), 0);
-                if (g) {
-                    ScmObj gv = SCM_GLOC_GET(g);
-                    if (SCM_MACROP(gv)) mac = SCM_MACRO(gv);
-                }
+            if (g) {
+                ScmObj gv = SCM_GLOC_GET(g);
+                if (SCM_MACROP(gv)) mac = SCM_MACRO(gv);
             }
         }
     }
