@@ -1,7 +1,7 @@
 /*
  * main.c - interpreter main program
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,22 +30,23 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: main.c,v 1.100 2007/04/12 03:26:55 shirok Exp $
+ *  $Id: main.c,v 1.104 2008-05-10 13:36:19 shirok Exp $
  */
 
-#include <unistd.h>
-#include <string.h>
-#include <stdlib.h>
-#include <sys/stat.h>
-#include <signal.h>
-#include <ctype.h>
-
 #define GAUCHE_API_0_9        /* temporary compatibility stuff */
 #include "gauche.h"
 
+#include <signal.h>
+#include <ctype.h>
+
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
+#ifdef MSVC
+int optind = 1;
+char *optarg = NULL;
+int getopt(int argc, char **argv, const char *spec);
+#endif /*MSVC*/
 
 /* options */
 int load_initfile = TRUE;       /* if false, not to load init files */
@@ -230,9 +231,6 @@ static void sig_setup(void)
     sigdelset(&set, SIGSTOP);
 #endif
     sigdelset(&set, SIGSEGV);
-//#ifdef SIGPROF
-//    sigdelset(&set, SIGPROF);
-//#endif /*SIGPROF*/
 #ifdef SIGBUS
     sigdelset(&set, SIGBUS);
 #endif /*SIGBUS*/
@@ -369,11 +367,11 @@ int main(int argc, char **argv)
         if (argv[optind][0] == '\0') Scm_Error("bad script name");
         if (argv[optind][0] == '/') {
             scriptfile = argv[optind];
-#if defined(__CYGWIN__) || defined(__MINGW32__)
+#if defined(__CYGWIN__) || defined(GAUCHE_WINDOWS)
 	} else if (isalpha(argv[optind][0]) && argv[optind][1] == ':') {
 	    /* support of wicked legacy DOS drive letter */
 	    scriptfile = argv[optind];
-#endif /* __CYGWIN__ || __MINGW32__ */
+#endif /* __CYGWIN__ || GAUCHE_WINDOWS */
         } else {
             if (stat(argv[optind], &statbuf) == 0) {
                 ScmDString ds;
@@ -473,6 +471,7 @@ int main(int argc, char **argv)
         mainproc = Scm_SymbolValue(Scm_UserModule(),
                                    SCM_SYMBOL(SCM_INTERN("main")));
         if (SCM_PROCEDUREP(mainproc)) {
+#if 0 /* Temporarily turned off due to the bug that loses stack traces. */
             int r = Scm_Apply(mainproc, SCM_LIST1(av), &epak);
             if (r > 0) {
                 ScmObj res = epak.results[0];
@@ -482,6 +481,14 @@ int main(int argc, char **argv)
                 Scm_ReportError(epak.exception);
                 exit_code = 70;  /* EX_SOFTWARE, see SRFI-22. */
             }
+#else
+            ScmObj r = Scm_ApplyRec(mainproc, SCM_LIST1(av));
+            if (SCM_INTP(r)) {
+                exit_code = SCM_INT_VALUE(r);
+            } else {
+                exit_code = 70;
+            }
+#endif
         }
     } else {
         /* We're in interactive mode. (use gauche.interactive) */
@@ -506,3 +513,63 @@ int main(int argc, char **argv)
     Scm_Exit(exit_code);
     return 0;
 }
+
+#if defined(MSVC)
+/* getopt emulation.  this is NOT a complete implementation of getopt;
+   we know how it is used in main.c, so we don't need full-spec. */
+int getopt(int argc, char **argv, const char *spec)
+{
+    static int clusterind = 0;
+
+    char optchar, *optspec;
+    
+    do {
+        if (optind >= argc) return -1;
+        if (clusterind > 0) {
+            /* remaining options */
+            if (argv[optind][clusterind] == '\0') {
+                clusterind = 0;
+                optind++;
+                continue;
+            } else {
+                optchar = argv[optind][clusterind];
+            }
+        } else {
+            if (argv[optind][0] != '-') return -1;
+            if (argv[optind][1] == '\0'
+                || (argv[optind][1] == '-' && argv[optind][2] == '\0')) {
+                optind++;
+                return -1;
+            }
+            /* found an option. */
+            optchar = argv[optind][1];
+            clusterind = 2;
+        }
+
+        /* now we have option char in optchar.  we know spec[0] == '+' so
+           we skip it. */
+        if ((optspec = strchr(spec+1, optchar)) == NULL) return '?';
+        
+        if (optspec[1] != ':') return optchar;
+
+        /* look for the argument. */
+        if (argv[optind][clusterind] == '\0') {
+            if (optind == argc-1) {
+                optarg = NULL;
+                return '?';
+            } else {
+                optarg = argv[++optind];
+                optind++;
+                clusterind = 0;
+                return optchar;
+            }
+        } else {
+            optarg = &(argv[optind][clusterind]);
+            optind++;
+            clusterind = 0;
+            return optchar;
+        }
+    } while (0);
+    return -1;
+}
+#endif /*MSVC*/
