@@ -1,7 +1,7 @@
 /*
  * module.c - module implementation
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: module.c,v 1.68 2007/04/09 02:09:45 shirok Exp $
+ *  $Id: module.c,v 1.71 2008-05-10 13:36:19 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -148,7 +148,7 @@ static ScmModule *lookup_module_create(ScmSymbol *name, int *created)
                            (intptr_t)name,
                            SCM_DICT_CREATE);
     if (e->value == 0) {
-        SCM_DICT_SET_VALUE(e, make_module(name));
+        (void)SCM_DICT_SET_VALUE(e, make_module(name));
         *created = TRUE;
     } else {
         *created = FALSE;
@@ -268,9 +268,9 @@ ScmObj Scm_GlobalVariableRef(ScmModule *module,
     
     if (g == NULL) return SCM_UNBOUND;
     val = SCM_GLOC_GET(g);
-    if (!(flags & SCM_BINDING_KEEP_AUTOLOAD)
-        && SCM_AUTOLOADP(val)) {
-        val = Scm_LoadAutoload(SCM_AUTOLOAD(val));
+    if (SCM_AUTOLOADP(val)) {
+        /* NB: Scm_ResolveAutoload may return SCM_UNBOUND */
+        val = Scm_ResolveAutoload(SCM_AUTOLOAD(val), 0);
     }
     return val;
 }
@@ -407,7 +407,7 @@ ScmObj Scm_ExportSymbols(ScmModule *module, ScmObj list)
         } else {
             g = SCM_GLOC(Scm_MakeGloc(s, module));
             g->exported = TRUE;
-            SCM_DICT_SET_VALUE(e, SCM_OBJ(g));
+            (void)SCM_DICT_SET_VALUE(e, SCM_OBJ(g));
             syms = Scm_Cons(SCM_OBJ(s), syms);
         }
     }
