@@ -1,7 +1,7 @@
 ;;;
 ;;; object.scm - object system
 ;;;  
-;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: objlib.scm,v 1.7 2007/03/16 12:14:32 shirok Exp $
+;;;  $Id: objlib.scm,v 1.11 2008-05-10 13:36:20 shirok Exp $
 ;;;
 
 ;; This module is not meant to be `use'd.   It is just to hide
@@ -518,23 +518,19 @@
 (define-method x->string ((obj <string>)) obj)
 (define-method x->string ((obj <number>)) (number->string obj))
 (define-method x->string ((obj <symbol>)) (symbol->string obj))
-(define-method x->string ((obj <char>)) (string obj))
-(define-method x->string ((obj <top>))
-  (with-output-to-string (lambda () (display obj))))
+(define-method x->string ((obj <char>))   (string obj))
+(define-method x->string ((obj <top>))    (write-to-string obj display))
 
 (define-method x->integer ((obj <integer>)) obj)
-(define-method x->integer ((obj <real>))
-  (inexact->exact (round obj)))
-(define-method x->integer ((obj <number>)) 0) ;complex numbers to 0
-(define-method x->integer ((obj <char>)) (char->integer obj))
-(define-method x->integer ((obj <top>))
-  (x->integer (x->number obj)))
+(define-method x->integer ((obj <real>))    (round->exact obj))
+(define-method x->integer ((obj <number>))  0) ;complex numbers to 0
+(define-method x->integer ((obj <char>))    (char->integer obj))
+(define-method x->integer ((obj <top>))     (x->integer (x->number obj)))
 
 (define-method x->number  ((obj <number>)) obj)
-(define-method x->number  ((obj <string>))
-  (or (string->number obj) 0))
-(define-method x->number  ((obj <char>)) (char->integer obj))
-(define-method x->number  ((obj <top>)) 0)
+(define-method x->number  ((obj <string>)) (or (string->number obj) 0))
+(define-method x->number  ((obj <char>))   (char->integer obj))
+(define-method x->number  ((obj <top>))    0)
 
 ;;----------------------------------------------------------------
 ;; Generic accessor
@@ -614,6 +610,22 @@
 (define-method object-apply ((type <condition-meta>) obj)
   (condition-has-type? obj type))
 
+;; Regexp printer.  It doesn't need speed, and it's easier to write in Scheme.
+;; This is defined here since we can only use it after define-method support.
+;; TODO: eventually we need a converter from internal AST to string regexp
+;; so that we can print the regexp which is created procedurally (via
+;; regexp-compile), hence we have complete read/write invariance.
+(define-method write-object ((obj <regexp>) out)
+  (cond [(regexp->string obj)
+         => (lambda (s)
+              (format out "#/~a/~a"
+                      (regexp-replace-all*
+                       s #/\// "\\\\/" #/[\x00-\x1f\x7f]/
+                       (lambda (m)
+                         (format "\\x~2,'0x" (char->integer (string-ref (m 0) 0)))))
+                      (if (regexp-case-fold? obj) "i" "")))]
+        [else (format out "#<regexp>")]))
+
 ;;;
 ;;; Make exported symbol visible from outside
 ;;;
