@@ -1,7 +1,7 @@
 /*
  * paths.c - get 'known' pathnames, such as the system's library directory.
  *
- *   Copyright (c) 2005-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2005-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: paths.c,v 1.2 2007/03/02 07:39:14 shirok Exp $
+ *  $Id: paths.c,v 1.5 2008-05-10 13:36:20 shirok Exp $
  *
  */
 
@@ -38,11 +38,11 @@
  * doesn't use ScmObj, so the function works on bare C strings.
  */
 
-#include <string.h>
-#include "gauche/config.h"
+#define LIBGAUCHE_BODY
+#include "gauche.h"
 #include "gauche/arch.h"
 
-#if defined(__MINGW32__)
+#if defined(GAUCHE_WINDOWS)
 #include "getdir_win.c"
 #elif defined(GAUCHE_MACOSX_FRAMEWORK)
 #include "getdir_darwin.c"
@@ -57,12 +57,12 @@ void maybe_prepend_install_dir(const char *orig, char *buf, int buflen,
 {
     if (*orig == '@') {
         int len = get_install_dir(buf, buflen, errfn);
-        if (len + strlen(orig) > buflen) {
+        if (len + (int)strlen(orig) > buflen) {
             errfn("Pathname too long: %s", orig);
         }
         strcat(buf, orig+1);
     } else {
-        if (strlen(orig) >= buflen-1) {
+        if ((int)strlen(orig) >= buflen-1) {
             errfn("Pathname too long: %s", orig);
         }
         strcpy(buf, orig);
