@@ -1,7 +1,7 @@
 /*
  * port.c - port implementation
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,19 +30,19 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: port.c,v 1.139 2007/04/16 03:47:13 shirok Exp $
+ *  $Id: port.c,v 1.148 2008-05-10 13:36:20 shirok Exp $
  */
 
-#include <unistd.h>
-#include <string.h>
-#include <fcntl.h>
-#include <errno.h>
-#include <ctype.h>
 #define LIBGAUCHE_BODY
 #include "gauche.h"
 #include "gauche/class.h"
 #include "gauche/port.h"
 
+#include <string.h>
+#include <fcntl.h>
+#include <errno.h>
+#include <ctype.h>
+
 #undef MAX
 #undef MIN
 #define MAX(a, b) ((a)>(b)? (a) : (b))
@@ -55,6 +55,7 @@
 static void port_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx);
 static void port_finalize(ScmObj obj, void* data);
 static void register_buffered_port(ScmPort *port);
+static void unregister_buffered_port(ScmPort *port);
 static void bufport_flush(ScmPort*, int, int);
 static void file_closer(ScmPort *p);
 
@@ -82,7 +83,7 @@ SCM_DEFINE_BASE_CLASS(Scm_LimitedLengthPortClass,
 
 /* Cleaning up:
  *   The underlying file descriptor/stream may be closed when the port
- *   is explicitly closed by close-port, or implicitly destroyed by
+ *   is explicitly closed by close-port, or implicitly destroyed by the
  *   garbage collector.  To keep consistency, Scheme ports should never
  *   share the same file descriptor.  However, C code and Scheme port
  *   may share the same file descriptor for efficiency (e.g. stdios).
@@ -93,9 +94,11 @@ static void port_cleanup(ScmPort *port)
     if (SCM_PORT_CLOSED_P(port)) return;
     switch (SCM_PORT_TYPE(port)) {
     case SCM_PORT_FILE:
-        if (SCM_PORT_DIR(port) == SCM_PORT_OUTPUT
-            && !SCM_PORT_ERROR_OCCURRED_P(port)) {
-            bufport_flush(port, 0, TRUE);
+        if (SCM_PORT_DIR(port) == SCM_PORT_OUTPUT) {
+            if (!SCM_PORT_ERROR_OCCURRED_P(port)) {
+                bufport_flush(port, 0, TRUE);
+            }
+            unregister_buffered_port(port);
         }
         if (port->ownerp && port->src.buf.closer) port->src.buf.closer(port);
         break;
@@ -134,7 +137,10 @@ static ScmPort *make_port(ScmClass *klass, int dir, int type)
     port->closed = FALSE;
     port->error = FALSE;
     port->ownerp = FALSE;
-    port->flags = 0;
+    port->flags =
+        SCM_VM_RUNTIME_FLAG_IS_SET(Scm_VM(), SCM_CASE_FOLD)
+        ? SCM_PORT_CASE_FOLD
+        : 0;
     port->name = SCM_FALSE;
     (void)SCM_INTERNAL_FASTLOCK_INIT(port->lock);
     port->lockOwner = NULL;
@@ -172,10 +178,6 @@ void Scm_ClosePort(ScmPort *port)
  * Locking ports
  */
 
-void Scm_PortLock(ScmPort *port)
-{
-}
-
 /*
  * External routine to access port exclusively
  */
@@ -239,6 +241,42 @@ int Scm_PortFileNo(ScmPort *port)
     }
 }
 
+/* Duplicates the file descriptor of the source port, and set it to
+   the destination port.  Both source and destination port must be
+   file ports. */
+#if !defined(GAUCHE_WINDOWS)
+void Scm_PortFdDup(ScmPort *dst, ScmPort *src)
+{
+    int r, srcfd, dstfd;
+
+    if (SCM_PORT_TYPE(dst) != SCM_PORT_FILE)
+        Scm_Error("file port required, but got %S", dst);
+    if (SCM_PORT_TYPE(src) != SCM_PORT_FILE)
+        Scm_Error("file port required, but got %S", src);
+    if (src->direction != dst->direction)
+        Scm_Error("port direction mismatch: got %S and %S",
+                  src, dst);
+        
+    srcfd = (int)(intptr_t)src->src.buf.data;
+    dstfd = (int)(intptr_t)dst->src.buf.data;
+
+    if (dst->direction == SCM_PORT_INPUT) {
+        /* discard the current buffer */
+        ScmVM *vm = Scm_VM();
+        PORT_LOCK(dst, vm);
+        dst->src.buf.current = dst->src.buf.buffer;
+        dst->src.buf.end = dst->src.buf.buffer;
+        PORT_UNLOCK(dst);
+    } else {
+        /* flush the current buffer */
+        Scm_Flush(dst);
+    }
+    SCM_SYSCALL(r, dup2(srcfd, dstfd));
+    if (r < 0) Scm_SysError("dup2 failed");
+    dst->src.buf.data = (void*)(intptr_t)r;
+}
+#endif /*!defined(GAUCHE_WINDOWS)*/
+
 /* Low-level function to find if the file descriptor is ready or not.
    DIR specifies SCM_PORT_INPUT or SCM_PORT_OUTPUT.
    If the system doesn't have select(), this function returns
@@ -652,7 +690,7 @@ static void register_buffered_port(ScmPort *port)
     int need_gc  = FALSE;
     
   retry:
-    h = i = PORT_HASH(port);
+    h = i = (int)PORT_HASH(port);
     c = 0;
     /* search an available entry by quadratic hash */
     (void)SCM_INTERNAL_MUTEX_LOCK(active_buffered_ports.mutex);
@@ -684,6 +722,27 @@ static void register_buffered_port(ScmPort *port)
     }
 }
 
+/* This should be called when the output buffered port is explicitly closed.
+   The ports collected by GC are automatically unregistered. */
+static void unregister_buffered_port(ScmPort *port)
+{
+    int i, h, c;
+    ScmObj p;
+    
+    h = i = (int)PORT_HASH(port);
+    c = 0;
+    (void)SCM_INTERNAL_MUTEX_LOCK(active_buffered_ports.mutex);
+    do {
+        p = Scm_WeakVectorRef(active_buffered_ports.ports, i, SCM_FALSE);
+        if (!SCM_FALSEP(p) && SCM_EQ(SCM_OBJ(port), p)) {
+            Scm_WeakVectorSet(active_buffered_ports.ports, i, SCM_FALSE);
+            break;
+        }
+        i -= ++c; while (i<0) i+=PORT_VECTOR_SIZE;
+    } while (i != h);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(active_buffered_ports.mutex);
+}
+
 /* Flush all ports.  Note that it is possible that this routine can be
    called recursively if one of the flushing routine calls Scm_Exit.
    In order to avoid infinite loop, I have to delete the entries of already
@@ -791,7 +850,7 @@ ScmObj Scm_GetBufferingMode(ScmPort *port)
 static int file_filler(ScmPort *p, int cnt)
 {
     int nread = 0, r;
-    int fd = (int)p->src.buf.data;
+    int fd = (int)(intptr_t)p->src.buf.data;
     char *datptr = p->src.buf.end;
     SCM_ASSERT(fd >= 0);
     while (nread == 0) {
@@ -815,7 +874,7 @@ static int file_flusher(ScmPort *p, int cnt, int forcep)
 {
     int nwrote = 0, r;
     int datsiz = SCM_PORT_BUFFER_AVAIL(p);
-    int fd = (int)p->src.buf.data;
+    int fd = (int)(intptr_t)p->src.buf.data;
     char *datptr = p->src.buf.buffer;
     
     SCM_ASSERT(fd >= 0);
@@ -836,26 +895,26 @@ static int file_flusher(ScmPort *p, int cnt, int forcep)
 
 static void file_closer(ScmPort *p)
 {
-    int fd = (int)p->src.buf.data;
+    int fd = (int)(intptr_t)p->src.buf.data;
     SCM_ASSERT(fd >= 0);
     close(fd);
 }
 
 static int file_ready(ScmPort *p)
 {
-    int fd = (int)p->src.buf.data;
+    int fd = (int)(intptr_t)p->src.buf.data;
     SCM_ASSERT(fd >= 0);
     return Scm_FdReady(fd, SCM_PORT_DIR(p));
 }
 
 static int file_filenum(ScmPort *p)
 {
-    return (int)p->src.buf.data;
+    return (int)(intptr_t)p->src.buf.data;
 }
 
 static off_t file_seeker(ScmPort *p, off_t offset, int whence)
 {
-    return lseek((int)p->src.buf.data, offset, whence);
+    return lseek((int)(intptr_t)p->src.buf.data, offset, whence);
 }
 
 ScmObj Scm_OpenFilePort(const char *path, int flags, int buffering, int perm)
@@ -870,12 +929,12 @@ ScmObj Scm_OpenFilePort(const char *path, int flags, int buffering, int perm)
     if (buffering < SCM_PORT_BUFFER_FULL || buffering > SCM_PORT_BUFFER_NONE) {
         Scm_Error("bad buffering flag: %d", buffering);
     }
-#if defined(__MINGW32__)
+#if defined(GAUCHE_WINDOWS)
     /* Force binary mode if not specified */
     if (!(flags & (O_TEXT|O_BINARY))) {
 	flags |= O_BINARY;
     }
-#endif /*__MINGW32__*/
+#endif /*GAUCHE_WINDOWS*/
     fd = open(path, flags, perm);
     if (fd < 0) return SCM_FALSE;
     bufrec.mode = buffering;
@@ -887,7 +946,7 @@ ScmObj Scm_OpenFilePort(const char *path, int flags, int buffering, int perm)
     bufrec.ready = file_ready;
     bufrec.filenum = file_filenum;
     bufrec.seeker = file_seeker;
-    bufrec.data = (void*)fd;
+    bufrec.data = (void*)(intptr_t)fd;
     p = Scm_MakeBufferedPort(SCM_CLASS_PORT, SCM_MAKE_STR_COPYING(path),
                              dir, TRUE, &bufrec);
     return p;
@@ -915,7 +974,7 @@ ScmObj Scm_MakePortWithFd(ScmObj name, int direction,
     bufrec.ready = file_ready;
     bufrec.filenum = file_filenum;
     bufrec.seeker = NULL;
-    bufrec.data = (void*)fd;
+    bufrec.data = (void*)(intptr_t)fd;
     
     p = Scm_MakeBufferedPort(SCM_CLASS_PORT, name, direction, ownerp, &bufrec);
     return p;
@@ -1006,25 +1065,26 @@ ScmObj Scm_GetRemainingInputString(ScmPort *port, int flags)
         if (cp - sp >= nbytes
             && memcmp(cp - nbytes, cbuf, nbytes) == 0) {
             cp -= nbytes;       /* we can reuse buffer */
-            return Scm_MakeString(cp, ep-cp, -1, flags);
+            return Scm_MakeString(cp, (int)(ep-cp), -1, flags);
         } else {
             /* we need to copy */
-            return get_remaining_input_string_aux(cp, ep-cp, cbuf,
+            return get_remaining_input_string_aux(cp, (int)(ep-cp), cbuf,
                                                   nbytes, flags);
         }
     } else if (port->scrcnt > 0) {
         sp = port->src.istr.start;
-        if (cp - sp >= port->scrcnt
+        if (cp - sp >= (int)port->scrcnt
             && memcmp(cp - port->scrcnt, port->scratch, port->scrcnt) == 0) {
             cp -= port->scrcnt; /* we can reuse buffer */
-            return Scm_MakeString(cp, ep-cp, -1, flags);
+            return Scm_MakeString(cp, (int)(ep-cp), -1, flags);
         } else {
             /* we need to copy */
-            return get_remaining_input_string_aux(cp, ep-cp, port->scratch,
+            return get_remaining_input_string_aux(cp, (int)(ep-cp),
+                                                  port->scratch,
                                                   port->scrcnt, flags);
         }
     } else {
-        return Scm_MakeString(cp, ep-cp, -1, flags);
+        return Scm_MakeString(cp, (int)(ep-cp), -1, flags);
     }
 }
 
@@ -1050,7 +1110,7 @@ ScmObj Scm__GetRemainingInputStringCompat(ScmPort *port)
  */
 
 /* To create a procedural port, fill in the ScmPortVTable function
-   pointers and pass it to Scm_MakeVirutalPort.  You don't need to
+   pointers and pass it to Scm_MakeVirtualPort.  You don't need to
    provide all the functions; put NULL if you think you don't
    provide the functionality.
 */
@@ -1133,10 +1193,14 @@ ScmObj Scm_MakeVirtualPort(ScmClass *klass, int direction,
    'coding' magic comment.   It is primarily used when loading source
    code, but can be used separately. */
 
+static ScmPort *(*coding_aware_port_hook)(ScmPort *src,
+                                          const char *srcencoding) = NULL;
+
 /* gauche.charconv sets the pointer */
-ScmPort *(*Scm_CodingAwarePortHook)(ScmPort *src,
-                                    const char *srcencoding)
-    = NULL;
+void Scm__InstallCodingAwarePortHook(ScmPort *(*f)(ScmPort*, const char*))
+{
+    coding_aware_port_hook = f;
+}
 
 #define CODING_MAGIC_COMMENT_LINES 2 /* maximum number of lines to be
                                         looked at for the 'encoding' magic
@@ -1248,7 +1312,7 @@ static void coding_port_recognize_encoding(ScmPort *port,
         }
     }
     data->pbuf = Scm_DStringGetz(&ds);
-    data->pbufsize = strlen(data->pbuf);
+    data->pbufsize = (int)strlen(data->pbuf);
     
     /* Look for the magic comment */
     encoding = look_for_encoding(data->pbuf);
@@ -1258,18 +1322,18 @@ static void coding_port_recognize_encoding(ScmPort *port,
         return;
     }
 
-    if (Scm_CodingAwarePortHook == NULL) {
+    if (coding_aware_port_hook == NULL) {
         /* Require gauche.charconv.
            NB: we don't need mutex here, for loading the module is
            serialized in Scm_Require. */
         Scm_Require(SCM_MAKE_STR("gauche/charconv"),
                     SCM_LOAD_PROPAGATE_ERROR, NULL);
-        if (Scm_CodingAwarePortHook == NULL) {
+        if (coding_aware_port_hook == NULL) {
             Scm_PortError(port, SCM_PORT_ERROR_OTHER,
                           "couldn't load gauche.charconv module");
         }
     }
-    data->source = Scm_CodingAwarePortHook(data->source, encoding);
+    data->source = coding_aware_port_hook(data->source, encoding);
 }
 
 static int coding_filler(ScmPort *p, int cnt)
@@ -1554,10 +1618,20 @@ void Scm__InitPort(void)
                                     SCM_PORT_BUFFER_FULL, TRUE);
     scm_stdout = Scm_MakePortWithFd(SCM_MAKE_STR("(stdout)"),
                                     SCM_PORT_OUTPUT, 1,
-                                    SCM_PORT_BUFFER_LINE, TRUE);
+                                    isatty(1)
+                                    ? SCM_PORT_BUFFER_LINE
+                                    : SCM_PORT_BUFFER_FULL,
+                                    TRUE);
     scm_stderr = Scm_MakePortWithFd(SCM_MAKE_STR("(stderr)"),
                                     SCM_PORT_OUTPUT, 2,
                                     SCM_PORT_BUFFER_NONE, TRUE);
+
+    /* The root VM is initialized with bogus standard ports; we need to
+       reset them. */
+    Scm_VM()->curin  = SCM_PORT(scm_stdin);
+    Scm_VM()->curout = SCM_PORT(scm_stdout);
+    Scm_VM()->curerr = SCM_PORT(scm_stderr);
+    
     key_full   = Scm_MakeKeyword(SCM_STRING(SCM_MAKE_STR("full")));
     key_modest = Scm_MakeKeyword(SCM_STRING(SCM_MAKE_STR("modest")));
     key_line   = Scm_MakeKeyword(SCM_STRING(SCM_MAKE_STR("line")));
