@@ -1,7 +1,7 @@
 /*
  * portapi.c - port common API
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: portapi.c,v 1.31 2007/03/02 07:39:14 shirok Exp $
+ *  $Id: portapi.c,v 1.33 2008-05-10 13:36:20 shirok Exp $
  */
 
 /* This file is included _twice_ by port.c to define safe- and unsafe-
@@ -108,7 +108,7 @@ void Scm_PutbUnsafe(ScmByte b, ScmPort *p)
     switch (SCM_PORT_TYPE(p)) {
     case SCM_PORT_FILE:
         if (p->src.buf.current >= p->src.buf.end) {
-            SAFE_CALL(p, bufport_flush(p, p->src.buf.current - p->src.buf.buffer, FALSE));
+            SAFE_CALL(p, bufport_flush(p, (int)(p->src.buf.current - p->src.buf.buffer), FALSE));
         }
         SCM_ASSERT(p->src.buf.current < p->src.buf.end);
         *p->src.buf.current++ = b;
@@ -152,7 +152,7 @@ void Scm_PutcUnsafe(ScmChar c, ScmPort *p)
     case SCM_PORT_FILE:
         nb = SCM_CHAR_NBYTES(c);
         if (p->src.buf.current+nb > p->src.buf.end) {
-            SAFE_CALL(p, bufport_flush(p, p->src.buf.current - p->src.buf.buffer, FALSE));
+            SAFE_CALL(p, bufport_flush(p, (int)(p->src.buf.current - p->src.buf.buffer), FALSE));
         }
         SCM_ASSERT(p->src.buf.current+nb <= p->src.buf.end);
         SCM_CHAR_PUT(p->src.buf.current, c);
@@ -245,7 +245,7 @@ void Scm_PutzUnsafe(const char *s, int siz, ScmPort *p)
     SHORTCUT(p, Scm_PutzUnsafe(s, siz, p); return);
     LOCK(p);
     CLOSE_CHECK(p);
-    if (siz < 0) siz = strlen(s);
+    if (siz < 0) siz = (int)strlen(s);
     switch (SCM_PORT_TYPE(p)) {
     case SCM_PORT_FILE:
         SAFE_CALL(p, bufport_write(p, s, siz));
@@ -416,7 +416,7 @@ int Scm_PeekbUnsafe(ScmPort *p)
 /* shift scratch buffer content */
 static inline void shift_scratch(ScmPort *p, int off)
 {
-    int i;
+    u_int i;
     for (i=0; i<p->scrcnt; i++) {
         p->scratch[i] = p->scratch[i+off];
     }
@@ -483,6 +483,7 @@ int Scm_GetbUnsafe(ScmPort *p)
             Scm_PortError(p, SCM_PORT_ERROR_INPUT,
                           "bad port type for input: %S", p);
         }
+        p->bytes++;
     }
     UNLOCK(p);
     return b;
@@ -555,6 +556,7 @@ int Scm_GetcUnsafe(ScmPort *p)
         }
         first = (unsigned char)*p->src.buf.current++;
         nb = SCM_CHAR_NFOLLOWS(first);
+        p->bytes++;
         if (nb > 0) {
             if (p->src.buf.current + nb > p->src.buf.end) {
                 /* The buffer doesn't have enough bytes to consist a char.
@@ -591,6 +593,7 @@ int Scm_GetcUnsafe(ScmPort *p)
                 SCM_CHAR_GET(p->src.buf.current-1, c);
                 p->src.buf.current += nb;
             }
+            p->bytes += nb;
         } else {
             c = first;
             if (c == '\n') p->line++;
@@ -604,6 +607,7 @@ int Scm_GetcUnsafe(ScmPort *p)
         }
         first = (unsigned char)*p->src.istr.current++;
         nb = SCM_CHAR_NFOLLOWS(first);
+        p->bytes++;
         if (nb > 0) {
             if (p->src.istr.current + nb > p->src.istr.end) {
                 /* TODO: make this behavior customizable */
@@ -613,6 +617,7 @@ int Scm_GetcUnsafe(ScmPort *p)
             }
             SCM_CHAR_GET(p->src.istr.current-1, c);
             p->src.istr.current += nb;
+            p->bytes += nb;
         } else {
             c = first;
             if (c == '\n') p->line++;
@@ -626,8 +631,7 @@ int Scm_GetcUnsafe(ScmPort *p)
         return c;
     default:
         UNLOCK(p);
-        Scm_PortError(p, SCM_PORT_ERROR_INPUT,
-                      "bad port type for input: %S", p);
+        Scm_PortError(p, SCM_PORT_ERROR_INPUT, "bad port type for input: %S", p);
     }
     return 0;/*dummy*/
 }
@@ -650,7 +654,7 @@ static int getz_scratch_unsafe(char *buf, int buflen, ScmPort *p)
 #endif
 {
     int i, n = 0;
-    if (p->scrcnt >= buflen) {
+    if (p->scrcnt >= (u_int)buflen) {
         memcpy(buf, p->scratch, buflen);
         p->scrcnt -= buflen;
         shift_scratch(p, buflen);
@@ -712,21 +716,23 @@ int Scm_GetzUnsafe(char *buf, int buflen, ScmPort *p)
     switch (SCM_PORT_TYPE(p)) {
     case SCM_PORT_FILE:
         SAFE_CALL(p, siz = bufport_read(p, buf, buflen));
+        p->bytes += siz;
         UNLOCK(p);
         if (siz == 0) return EOF;
         else return siz;
     case SCM_PORT_ISTR:
         r = GETZ_ISTR(p, buf, buflen);
+        p->bytes += r;
         UNLOCK(p);
         return r;
     case SCM_PORT_PROC:
         SAFE_CALL(p, r = p->src.vt.Getz(buf, buflen, p));
+        p->bytes += r;
         UNLOCK(p);
         return r;
     default:
         UNLOCK(p);
-        Scm_PortError(p, SCM_PORT_ERROR_INPUT,
-                      "bad port type for input: %S", p);
+        Scm_PortError(p, SCM_PORT_ERROR_INPUT, "bad port type for input: %S", p);
     }
     return -1;                  /* dummy */
 }
