@@ -1,7 +1,7 @@
 /*
  * prof.c - profiler
  *
- *   Copyright (c) 2005-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2005-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,11 +30,9 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: prof.c,v 1.9 2007/03/02 07:39:14 shirok Exp $
+ *  $Id: prof.c,v 1.11 2008-05-10 13:36:20 shirok Exp $
  */
 
-#include <stdlib.h>
-#include <unistd.h>
 #define LIBGAUCHE_BODY
 #include "gauche.h"
 #include "gauche/vm.h"
@@ -338,6 +336,7 @@ void Scm_ProfilerStart(void)
 int  Scm_ProfilerStop(void)
 {
     Scm_Error("profiler is not supported.");
+    return 0;
 }
 
 void Scm_ProfilerReset(void)
