@@ -1,7 +1,7 @@
 /*
  * read.c - reader
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: read.c,v 1.92 2007/04/12 03:26:55 shirok Exp $
+ *  $Id: read.c,v 1.99 2008-05-10 13:36:20 shirok Exp $
  */
 
 #include <stdio.h>
@@ -67,12 +67,12 @@ static ScmObj read_charset(ScmPort *port);
 static ScmObj read_sharp_comma(ScmPort *port, ScmReadContext *ctx);
 static ScmObj process_sharp_comma(ScmPort *port, ScmObj key, ScmObj args,
                                   ScmReadContext *ctx, int has_ref);
+static ScmObj read_shebang(ScmPort *port, ScmReadContext *ctx);
 static ScmObj read_reference(ScmPort *port, ScmChar ch, ScmReadContext *ctx);
 static ScmObj maybe_uvector(ScmPort *port, char c, ScmReadContext *ctx);
 
 /* Special hook for SRFI-4 syntax */
-ScmObj (*Scm_ReadUvectorHook)(ScmPort *port, const char *tag,
-                              ScmReadContext *ctx) = NULL;
+static ScmObj (*read_uvector_hook)(ScmPort *, const char*, ScmReadContext *) = NULL;
 
 /* Table of 'read-time constructor' in SRFI-10 */
 static struct {
@@ -80,6 +80,12 @@ static struct {
     ScmInternalMutex mutex;
 } readCtorData = { NULL };
 
+/* Table of hash-bang directive */
+static struct {
+    ScmHashTable *table;
+    ScmInternalMutex mutex;
+} hashBangData = { NULL };
+
 /*----------------------------------------------------------------
  * Entry points
  *   Note: Entire read operation are done while locking the input port.
@@ -113,19 +119,16 @@ ScmObj Scm_ReadWithContext(ScmObj port, ScmReadContext *ctx)
 
 ScmObj Scm_Read(ScmObj port)
 {
-    ScmReadContext ctx;
-    read_context_init(Scm_VM(), &ctx);
-    return Scm_ReadWithContext(port, &ctx);
+    return Scm_ReadWithContext(port, Scm_MakeReadContext(NULL));
 }
 
 /* Convenience functions */
 ScmObj Scm_ReadFromString(ScmString *str)
 {
     ScmObj inp = Scm_MakeInputStringPort(str, TRUE), r;
-    ScmReadContext ctx;
-    read_context_init(Scm_VM(), &ctx);
-    r = read_item(SCM_PORT(inp), &ctx);
-    read_context_flush(&ctx);
+    ScmReadContext *ctx = Scm_MakeReadContext(NULL);
+    r = read_item(SCM_PORT(inp), ctx);
+    read_context_flush(ctx);
     return r;
 }
 
@@ -134,10 +137,9 @@ ScmObj Scm_ReadFromCString(const char *cstr)
     ScmObj s = SCM_MAKE_STR_IMMUTABLE(cstr);
     ScmObj inp = Scm_MakeInputStringPort(SCM_STRING(s), TRUE);
     ScmObj r;
-    ScmReadContext ctx;
-    read_context_init(Scm_VM(), &ctx);
-    r = read_item(SCM_PORT(inp), &ctx);
-    read_context_flush(&ctx);
+    ScmReadContext *ctx = Scm_MakeReadContext(NULL);
+    r = read_item(SCM_PORT(inp), ctx);
+    read_context_flush(ctx);
     return r;
 }
 
@@ -167,21 +169,42 @@ ScmObj Scm_ReadListWithContext(ScmObj port, ScmChar closer, ScmReadContext *ctx)
 
 ScmObj Scm_ReadList(ScmObj port, ScmChar closer)
 {
-    ScmReadContext ctx;
-    read_context_init(Scm_VM(), &ctx);
-    return Scm_ReadListWithContext(port, closer, &ctx);
+    ScmReadContext *ctx = Scm_MakeReadContext(NULL);
+    return Scm_ReadListWithContext(port, closer, ctx);
 }
 
-static void read_context_init(ScmVM *vm, ScmReadContext *ctx)
+/*----------------------------------------------------------------
+ * Read context
+ */
+
+ScmReadContext *Scm_MakeReadContext(ScmReadContext *proto)
 {
-    ctx->flags = SCM_READ_SOURCE_INFO;
-    if (SCM_VM_RUNTIME_FLAG_IS_SET(vm, SCM_CASE_FOLD)) {
-        ctx->flags |= SCM_READ_CASE_FOLD;
+    ScmVM *vm = Scm_VM();
+    ScmReadContext *ctx = SCM_NEW(ScmReadContext);
+    SCM_SET_CLASS(ctx, SCM_CLASS_READ_CONTEXT);
+    if (proto == NULL) {
+        ctx->flags = SCM_READ_SOURCE_INFO;
+        ctx->table = NULL;
+        ctx->pending = SCM_NIL;
+    } else {
+        /* TODO: we haven't thought well of the case when prototype
+           is given.  Should we copy the #n= table as well?  How should
+           we handle pending read references? */
+        ctx->flags = proto->flags;
+        ctx->table = NULL;
+        ctx->pending = SCM_NIL;
     }
-    ctx->table = NULL;
-    ctx->pending = SCM_NIL;
+    return ctx;
 }
 
+static void read_context_print(ScmObj obj, ScmPort *port,
+                               ScmWriteContext *ctx)
+{
+    Scm_Printf(port, "#<read-context %p>", obj);
+}
+
+SCM_DEFINE_BUILTIN_CLASS_SIMPLE(Scm_ReadContextClass, read_context_print);
+
 /*----------------------------------------------------------------
  * Error
  */
@@ -250,7 +273,7 @@ static ScmObj ref_val(ScmObj ref)
 static ScmObj ref_register(ScmReadContext *ctx, ScmObj obj, int refnum)
 {
     SCM_ASSERT(ctx->table);
-    Scm_HashTablePut(ctx->table, SCM_MAKE_INT(refnum), obj);
+    Scm_HashTableSet(ctx->table, SCM_MAKE_INT(refnum), obj, 0);
     return obj;
 }
 
@@ -424,12 +447,8 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
                 Scm_UngetcUnsafe(c1, port);
                 return read_number(port, c, ctx);
             case '!':
-                /* allow `#!' magic of executable */
-                for (;;) {
-                    c = Scm_GetcUnsafe(port);
-                    if (c == '\n') return SCM_UNDEFINED;
-                    if (c == EOF) return SCM_EOF;
-                }
+                /* #! is either a script shebang or a reader directive */
+                return read_shebang(port, ctx);
             case '/':
                 /* #/.../ literal regexp */
                 return read_regexp(port);
@@ -722,48 +741,88 @@ static ScmObj read_string(ScmPort *port, int incompletep,
     for (;;) {
         FETCH(c);
         switch (c) {
-          case EOF: goto eof_exit;
-          case '"': {
-              int flags = ((incompletep? SCM_STRING_INCOMPLETE : 0)
-                           | SCM_STRING_IMMUTABLE);
-              return Scm_DStringGet(&ds, flags);
-          }
-          case '\\': {
-            int c1 = Scm_GetcUnsafe(port);
-            switch (c1) {
-              case EOF: goto eof_exit;
-              case 'n': ACCUMULATE('\n'); break;
-              case 'r': ACCUMULATE('\r'); break;
-              case 'f': ACCUMULATE('\f'); break;
-              case 't': ACCUMULATE('\t'); break;
-              case '\\': ACCUMULATE('\\'); break;
-              case '0': ACCUMULATE(0); break;
-              case 'x': {
-                  int cc = read_string_xdigits(port, 2, 'x', incompletep);
-                  ACCUMULATE(cc);
-                  break;
-              }
-              case 'u': {
-                  int cc = read_string_xdigits(port, 4, 'u', incompletep);
-                  ACCUMULATE(Scm_UcsToChar(cc));
-                  break;
-              }
-              case 'U': {
-                  int cc = read_string_xdigits(port, 8, 'U', incompletep);
-                  ACCUMULATE(Scm_UcsToChar(cc));
-                  break;
-              }
-              default:
-                ACCUMULATE(c1); break;
+        case EOF: goto eof_exit;
+        case '"': {
+            int flags = ((incompletep? SCM_STRING_INCOMPLETE : 0)
+                         | SCM_STRING_IMMUTABLE);
+            return Scm_DStringGet(&ds, flags);
+        }
+        backslash:
+        case '\\': {
+            FETCH(c);
+            switch (c) {
+            case EOF: goto eof_exit;
+            case 'n': ACCUMULATE('\n'); break;
+            case 'r': ACCUMULATE('\r'); break;
+            case 'f': ACCUMULATE('\f'); break;
+            case 't': ACCUMULATE('\t'); break;
+            case '\\': ACCUMULATE('\\'); break;
+            case '0': ACCUMULATE(0); break;
+            case 'x': {
+                int cc = read_string_xdigits(port, 2, 'x', incompletep);
+                ACCUMULATE(cc);
+                break;
+            }
+            case 'u': {
+                int cc = read_string_xdigits(port, 4, 'u', incompletep);
+                ACCUMULATE(Scm_UcsToChar(cc));
+                break;
+            }
+            case 'U': {
+                int cc = read_string_xdigits(port, 8, 'U', incompletep);
+                ACCUMULATE(Scm_UcsToChar(cc));
+                break;
+            }
+                /* R6RS-style line continuation handling*/
+            case ' ':
+            case '\t': {
+                for (;;) {
+                    FETCH(c);
+                    if (c == EOF) goto eof_exit;
+                    if (c == '\r') goto cont_cr;
+                    if (c == '\n') goto cont_lf;
+                    if (c != ' ' && c != '\t') goto cont_err;
+                }
+            }
+                /*FALLTHROUGH*/
+            cont_cr:
+            case '\r': {
+                FETCH(c);
+                if (c == EOF)  goto eof_exit;
+                if (c == '\\') goto backslash;
+                if (c != '\n' && c != ' ' && c != '\t') {
+                    ACCUMULATE(c);
+                    break;
+                }
+                /*FALLTHROUGH*/
+            }
+            cont_lf:
+            case '\n': {
+                for (;;) {
+                    FETCH(c);
+                    if (c == EOF) goto eof_exit;
+                    if (c == '\\') goto backslash;
+                    if (c != ' ' && c != '\t') {
+                        ACCUMULATE(c);
+                        break;
+                    }
+                }
+                break;
+            }
+            default:
+                ACCUMULATE(c); break;
             }
             break;
-          }
-          default: ACCUMULATE(c); break;
+        }       
+        default: ACCUMULATE(c); break;
         }
     }
  eof_exit:
     Scm_ReadError(port, "EOF encountered in a string literal: %S",
                   Scm_DStringGet(&ds, 0));
+ cont_err:
+    Scm_ReadError(port, "Invalid line continuation sequence in a string literal: %S...",
+                  Scm_DStringGet(&ds, 0));
     /* NOTREACHED */
     return SCM_FALSE; 
 }
@@ -829,7 +888,7 @@ static ScmObj read_char(ScmPort *port, ScmReadContext *ctx)
         /* handle #\uxxxx or #\uxxxxxxxx*/
         if ((cname[0] == 'u') && isxdigit(cname[1])) {
             int code;
-            if (namesize == 5 || namesize == 9) {
+            if (namesize >= 5 && namesize <= 9) {
                 code = Scm_ReadXdigitsFromString(cname+1, namesize-1, NULL);
                 if (code >= 0) return SCM_MAKE_CHAR(Scm_UcsToChar(code));
             }
@@ -860,7 +919,7 @@ static ScmObj read_word(ScmPort *port, ScmChar initial, ScmReadContext *ctx,
                         int temp_case_fold)
 {
     int c = 0;
-    int case_fold = temp_case_fold || (ctx->flags & SCM_READ_CASE_FOLD);
+    int case_fold = temp_case_fold || SCM_PORT_CASE_FOLD(port);
     ScmDString ds;
     Scm_DStringInit(&ds);
     if (initial != SCM_CHAR_INVALID) {
@@ -947,7 +1006,7 @@ static ScmObj read_escaped_symbol(ScmPort *port, ScmChar delim)
 /* gauche extension :  #/regexp/ */
 static ScmObj read_regexp(ScmPort *port)
 {
-    ScmChar c = 0;
+    ScmChar c = 0, c1 = 0;
     ScmDString ds;
     Scm_DStringInit(&ds);
     for (;;) {
@@ -956,12 +1015,14 @@ static ScmObj read_regexp(ScmPort *port)
             Scm_ReadError(port, "unterminated literal regexp");
         }
         if (c == '\\') {
-            SCM_DSTRING_PUTC(&ds, c);
-            c = Scm_GetcUnsafe(port);
-            if (c == SCM_CHAR_INVALID) {
+            /* NB: We "eat" a backslash before '/', since it is only dealt
+               with the reader and nothing to do with regexp parser itself. */
+            c1 = Scm_GetcUnsafe(port);
+            if (c1 == SCM_CHAR_INVALID) {
                 Scm_ReadError(port, "unterminated literal regexp");
             }
-            SCM_DSTRING_PUTC(&ds, c);
+            if (c1 != '/') SCM_DSTRING_PUTC(&ds, c);
+            SCM_DSTRING_PUTC(&ds, c1);
         } else if (c == '/') {
             /* Read one more char to see if we have a flag */
             int flags = 0;
@@ -987,7 +1048,7 @@ static ScmObj read_charset(ScmPort *port)
 
 static ScmObj read_reference(ScmPort *port, ScmChar ch, ScmReadContext *ctx)
 {
-    ScmHashEntry *e = NULL;
+    ScmObj e = SCM_UNBOUND;
     int refnum = Scm_DigitToInt(ch, 10);
 
     for (;;) {
@@ -1008,14 +1069,15 @@ static ScmObj read_reference(ScmPort *port, ScmChar ch, ScmReadContext *ctx)
     if (ch == '#') {
         /* #digit# - back reference */
         if (ctx->table == NULL
-            || (e = Scm_HashTableGet(ctx->table, Scm_MakeInteger(refnum))) == NULL) {
+            || SCM_UNBOUNDP(e = Scm_HashTableRef(ctx->table,
+                                                 Scm_MakeInteger(refnum),
+                                                 SCM_UNBOUND))) {
             Scm_ReadError(port, "invalid reference number in #%d#", refnum);
         }
-        if (SCM_READ_REFERENCE_P(e->value)
-            && SCM_READ_REFERENCE_REALIZED(e->value)) {
-            return SCM_READ_REFERENCE(e->value)->value;
+        if (SCM_READ_REFERENCE_P(e) && SCM_READ_REFERENCE_REALIZED(e)) {
+            return SCM_READ_REFERENCE(e)->value;
         } else {
-            return e->value;
+            return e;
         }
     } else {
         /* #digit= - register */
@@ -1026,8 +1088,11 @@ static ScmObj read_reference(ScmPort *port, ScmChar ch, ScmReadContext *ctx)
             ctx->table =
                 SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQV, 0));
         }
-        if (Scm_HashTableGet(ctx->table, Scm_MakeInteger(refnum)) != NULL) {
-            Scm_ReadError(port, "duplicate back-reference number in #%d=", refnum);
+        if (!SCM_UNBOUNDP(Scm_HashTableRef(ctx->table,
+                                           Scm_MakeInteger(refnum),
+                                           SCM_UNBOUND))) {
+            Scm_ReadError(port, "duplicate back-reference number in #%d=",
+                          refnum);
         }
         ref_register(ctx, ref, refnum);
         val = read_item(port, ctx);
@@ -1048,7 +1113,7 @@ ScmObj Scm_DefineReaderCtor(ScmObj symbol, ScmObj proc, ScmObj finisher)
     }
     pair = Scm_Cons(proc, finisher);
     (void)SCM_INTERNAL_MUTEX_LOCK(readCtorData.mutex);
-    Scm_HashTablePut(readCtorData.table, symbol, pair);
+    Scm_HashTableSet(readCtorData.table, symbol, pair, 0);
     (void)SCM_INTERNAL_MUTEX_UNLOCK(readCtorData.mutex);
     return SCM_UNDEFINED;
 }
@@ -1079,19 +1144,17 @@ static ScmObj read_sharp_comma(ScmPort *port, ScmReadContext *ctx)
 static ScmObj process_sharp_comma(ScmPort *port, ScmObj key, ScmObj args,
                                   ScmReadContext *ctx, int has_ref)
 {
-    ScmHashEntry *e;
-    ScmObj r;
+    ScmObj r, e;
 
     if (ctx->flags & SCM_READ_DISABLE_CTOR) return SCM_FALSE;
 
     (void)SCM_INTERNAL_MUTEX_LOCK(readCtorData.mutex);
-    e = Scm_HashTableGet(readCtorData.table, key);
+    e = Scm_HashTableRef(readCtorData.table, key, SCM_FALSE);
     (void)SCM_INTERNAL_MUTEX_UNLOCK(readCtorData.mutex);
 
-    if (e == NULL) Scm_ReadError(port, "unknown #,-key: %S", key);
-    SCM_ASSERT(SCM_PAIRP(e->value));
-    r = Scm_ApplyRec(SCM_CAR(e->value), args);
-    if (has_ref) ref_push(ctx, r, SCM_CDR(e->value));
+    if (!SCM_PAIRP(e)) Scm_ReadError(port, "unknown #,-key: %S", key);
+    r = Scm_ApplyRec(SCM_CAR(e), args);
+    if (has_ref) ref_push(ctx, r, SCM_CDR(e));
     return r;
 }
 
@@ -1101,12 +1164,68 @@ static ScmObj reader_ctor(ScmObj *args, int nargs, void *data)
     return Scm_DefineReaderCtor(args[0], args[1], optarg);
 }
 
+/*----------------------------------------------------------------
+ * #!-support
+ */
+
+ScmObj Scm_DefineReaderDirective(ScmObj symbol, ScmObj proc)
+{
+    (void)SCM_INTERNAL_MUTEX_LOCK(hashBangData.mutex);
+    Scm_HashTableSet(hashBangData.table, symbol, proc, 0);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(hashBangData.mutex);
+    return SCM_UNDEFINED;
+}
+
+static ScmObj read_shebang(ScmPort *port, ScmReadContext *ctx)
+{
+    /* If '#!' appears in the beginning of the input port, and it is
+       followed by '/' or a space, then we consider it as the
+       beginning of shebang and discards entire line.  Otherwise, we
+       take this as #!<identifier> directive as specified in R6RS, and
+       calls appropriate handler.
+
+       R6RS is actually not very clean at this corner.  It requires
+       distinct modes to parse a script (which always begins with
+       shebang) and to parse R6RS program (in which '#!'  always
+       introduces #!<identifier> token).  There's no way for just one
+       parser that strictly covers both situation.
+    */
+    int c2 = Scm_GetcUnsafe(port);
+    if (port->bytes == 3 && (c2 == '/' || c2 == ' ')) {
+        /* shebang */
+        for (;;) {
+            c2 = Scm_GetcUnsafe(port);
+            if (c2 == '\n') return SCM_UNDEFINED;
+            if (c2 == EOF) return SCM_EOF;
+        }
+        /*NOTREACHED*/
+    } else {
+        ScmObj id = read_symbol(port, c2, ctx), e, r;
+        if (SCM_EOFP(id)) return SCM_EOF; /* we may want warn? */
+        
+        (void)SCM_INTERNAL_MUTEX_LOCK(hashBangData.mutex);
+        e = Scm_HashTableRef(hashBangData.table, id, SCM_FALSE);
+        (void)SCM_INTERNAL_MUTEX_UNLOCK(hashBangData.mutex);
+        if (SCM_FALSEP(e)) {
+            Scm_Warn("Ignoring unrecognized hash-bang directive: #!%S", id);
+            return SCM_UNDEFINED;
+        }
+        /* Reader directive may return zero or one value. */
+        r = Scm_ApplyRec3(e, id, SCM_OBJ(port), SCM_OBJ(ctx));
+        if (Scm_VMGetNumResults(Scm_VM()) == 0) return SCM_UNDEFINED;
+        else                                    return r;
+    }
+}
+
+
+
+
 /*----------------------------------------------------------------
  * Uvector
  */
 
-/* Uvector support is implemented by extention.  When the extention
-   is loaded, it sets up the pointer Scm_ReadUvectorHook. */
+/* Uvector support is implemented by gauche.uvector.  When it
+   is loaded, it sets up the pointer read_uvector_hook. */
 
 static ScmObj maybe_uvector(ScmPort *port, char ch, ScmReadContext *ctx)
 {
@@ -1150,18 +1269,24 @@ static ScmObj maybe_uvector(ScmPort *port, char ch, ScmReadContext *ctx)
         *bufp = '\0';
         Scm_ReadError(port, "invalid uniform vector tag: %s", buf);
     }
-    if (Scm_ReadUvectorHook == NULL) {
+    if (read_uvector_hook == NULL) {
         /* Require srfi-4 (gauche/uvector)
            NB: we don't need mutex here, for the loading of srfi-4 is
            serialized in Scm_Require. */
         Scm_Require(SCM_MAKE_STR("gauche/uvector"),
                     SCM_LOAD_PROPAGATE_ERROR, NULL);
-        if (Scm_ReadUvectorHook == NULL)
+        if (read_uvector_hook == NULL)
             Scm_ReadError(port, "couldn't load srfi-4 module");
     }
-    return Scm_ReadUvectorHook(port, tag, ctx);
+    return read_uvector_hook(port, tag, ctx);
 }
 
+void Scm__InstallReadUvectorHook(ScmObj (*f)(ScmPort*, const char*, ScmReadContext *))
+{
+    read_uvector_hook = f;
+}
+
+
 /*----------------------------------------------------------------
  * Initialization
  */
@@ -1175,5 +1300,9 @@ void Scm__InitRead(void)
                          Scm_MakeSubr(reader_ctor, NULL, 2, 1,
                                       SCM_SYM_DEFINE_READER_CTOR),
                          SCM_FALSE);
+
+    hashBangData.table =
+        SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQ, 0));
+    (void)SCM_INTERNAL_MUTEX_INIT(hashBangData.mutex);
 }
 
