@@ -1,7 +1,7 @@
 /*
  * signal.c - signal handling
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,11 +30,9 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: signal.c,v 1.51 2007/03/02 07:39:14 shirok Exp $
+ *  $Id: signal.c,v 1.56 2008-06-02 01:13:13 shirok Exp $
  */
 
-#include <stdlib.h>
-#include <signal.h>
 #define LIBGAUCHE_BODY
 #include "gauche.h"
 #include "gauche/vm.h"
@@ -61,20 +59,21 @@
  *  the system to handle the signal unless the Scheme program installs
  *  the handler.   Such signals are the ones that can't be caught, or
  *  are ignored by default.  SIGPWR and SIGXCPU are also left to the system
- *  since GC uses it in the Linux/pthread environment.
+ *  since GC uses it in the Linux/pthread environment.  
+ *
+ *  About the signal behavior on windows, see "Note on windows port" below.
  */
-
-#ifndef __MINGW32__
-  #ifdef GAUCHE_USE_PTHREADS
-  #define SIGPROCMASK pthread_sigmask
-  #else
-  #define SIGPROCMASK sigprocmask
-  #endif
-#else  /* __MINGW32__ */
-  /* This isn't correct (we need some mechanism to block the signal),
-     but just for the time being ... */
-  #define SIGPROCMASK(mode, set, omask)  (0)
-#endif /* __MINGW32__ */
+#if !defined(GAUCHE_WINDOWS)
+# ifdef GAUCHE_USE_PTHREADS
+#  define SIGPROCMASK pthread_sigmask
+# else
+#  define SIGPROCMASK sigprocmask
+# endif
+#else  /* GAUCHE_WINDOWS */
+/* emulation routine is defined below */
+# define SIGPROCMASK sigprocmask_win
+static int sigprocmask_win(int how, const sigset_t *set, sigset_t *oldset);
+#endif /* GAUCHE_WINDOWS */
 
 /* Master signal handler vector. */
 static struct sigHandlersRec {
@@ -543,9 +542,18 @@ static SCM_DEFINE_SUBR(through_sighandler_stub, 1, 0,
 #endif
 
 /*
- * An emulation stub for Windows/MinGW
+ * An emulation stub for Windows
+ *
+ * Note on windows port:
+ *  Windows does provide signal() function to conform C standard, but
+ *  its use is so limited that it's effectively useless.  You can only
+ *  trap SIGABRT, SIGFPE, SIGILL, SIGSEGV and SIGTERM, and you cannot
+ *  send signal to other processes.  Emulating POSIX signal behavior is
+ *  not an easy task (see http://cygwin.com/cgi-bin/cvsweb.cgi/src/winsup/cygwin/how-signals-work.txt?cvsroot=src ).
+ *  So, although we provide a signal interface, DO NOT USE IT.  It's better
+ *  to expose Windows native IPC and build an abstraction on top if it.
  */
-#ifdef __MINGW32__
+#if defined(GAUCHE_WINDOWS)
 int sigaction(int signum, const struct sigaction *act,
 	      struct sigaction *oact)
 {
@@ -558,7 +566,12 @@ int sigaction(int signum, const struct sigaction *act,
 	return 0;
     }
 }
-#endif /* __MINGW32__ */
+
+int sigprocmask_win(int how, const sigset_t *set, sigset_t *oldset)
+{
+    return 0;
+}
+#endif /* GAUCHE_WINDOWS */
 
 /*
  * set-signal-handler!
@@ -686,7 +699,7 @@ void Scm_SetMasterSigmask(sigset_t *set)
     struct sigdesc *desc = sigDesc;
     struct sigaction acton, actoff;
 
-    acton.sa_handler = (void(*)())sig_handle;
+    acton.sa_handler = (void(*)(int))sig_handle;
     acton.sa_mask = *set;
     acton.sa_flags = 0;
     actoff.sa_handler = SIG_DFL;
@@ -734,6 +747,24 @@ void Scm_SetMasterSigmask(sigset_t *set)
  * Other signal-related operations
  */
 
+/*
+ * Convenience routines hiding platform-dependent stuff
+ */
+void Scm_GetSigmask(sigset_t *mask)
+{
+    if (SIGPROCMASK(SIG_SETMASK, NULL, mask) != 0) {
+        Scm_SysError("sigprocmask failed");
+    }
+}
+
+void Scm_SetSigmask(sigset_t *mask)
+{
+    if (SIGPROCMASK(SIG_SETMASK, mask, NULL) != 0) {
+        Scm_SysError("sigprocmask failed");
+    }
+}
+
+
 /*
  * set signal mask
  */
@@ -750,7 +781,7 @@ ScmObj Scm_SysSigmask(int how, ScmSysSigset *newmask)
         }
     }
     if (SIGPROCMASK(how, newset, &(oldmask->set)) != 0) {
-        Scm_Error("sigprocmask failed");
+        Scm_SysError("sigprocmask failed");
     }
     return SCM_OBJ(oldmask);
 }
@@ -782,7 +813,7 @@ void Scm_ResetSignalHandlers(sigset_t *mask)
  */
 static void scm_sigsuspend(sigset_t *mask)
 {
-#ifndef __MINGW32__
+#if !defined(GAUCHE_WINDOWS)
     sigset_t omask;
     ScmVM *vm = Scm_VM();
     for (;;) {
@@ -797,9 +828,9 @@ static void scm_sigsuspend(sigset_t *mask)
     sigsuspend(mask);
     SIGPROCMASK(SIG_SETMASK, &omask, NULL);
     SCM_SIGCHECK(vm);
-#else  /*__MINGW32__*/
-    Scm_Error("sigsuspend not supported on MinGW port");
-#endif /*__MINGW32__*/
+#else  /* GAUCHE_WINDOWS */
+    Scm_Error("sigsuspend not supported on Windows port");
+#endif /* GAUCHE_WINDOWS */
 }
 
 ScmObj Scm_SigSuspend(ScmSysSigset *mask)
