@@ -1,7 +1,7 @@
 /*
  * write.c - writer
  *
- *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,16 +30,16 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: write.c,v 1.68 2007/04/12 03:26:55 shirok Exp $
+ *  $Id: write.c,v 1.71 2008-05-10 13:36:24 shirok Exp $
  */
 
-#include <stdio.h>
-#include <ctype.h>
 #define LIBGAUCHE_BODY
 #include "gauche.h"
 #include "gauche/port.h"
 #include "gauche/builtin-syms.h"
 
+#include <ctype.h>
+
 static void write_walk(ScmObj obj, ScmPort *port, ScmWriteContext *ctx);
 static void write_ss(ScmObj obj, ScmPort *port, ScmWriteContext *ctx);
 static void write_ss_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx);
@@ -665,11 +665,11 @@ static void format_sexp(ScmPort *out, ScmObj arg,
         const char *s = Scm_GetStringContent(tmpstr, NULL, NULL, NULL), *e;
         if (dots && maxcol > 4) {
             e = Scm_StringPosition(tmpstr, maxcol-4);
-            Scm_PutzUnsafe(s, e-s, out);
+            Scm_PutzUnsafe(s, (int)(e-s), out);
             Scm_PutzUnsafe(" ...", 4, out);
         } else {
             e = Scm_StringPosition(tmpstr, maxcol);
-            Scm_PutzUnsafe(s, e-s, out);
+            Scm_PutzUnsafe(s, (int)(e-s), out);
         }
     } else {
         format_pad(out, tmpstr, mincol, colinc, padchar, rightalign);
@@ -772,6 +772,9 @@ static void format_proc(ScmPort *out, ScmString *fmt, ScmObj args, int sharedp)
         for (;;) {
             ch = Scm_GetcUnsafe(fmtstr);
             switch (ch) {
+            case EOF:
+                Scm_Error("incomplete format string: %S", fmt);
+                break;
             case '%':
                 Scm_PutcUnsafe('\n', out);
                 break;
@@ -1097,7 +1100,7 @@ static void vprintf_proc(ScmPort *out, const char *fmt, ScmObj args,
                     SCM_DSTRING_PUTB(&argbuf, c);
                     SCM_DSTRING_PUTB(&argbuf, 0);
                     snprintf(buf, SPBUFSIZ, Scm_DStringGetz(&argbuf),
-                             (void*)Scm_GetUInteger(val));
+                             (void*)(intptr_t)Scm_GetUInteger(val));
                     Scm_PutzUnsafe(buf, -1, out);
                     break;
                 }
@@ -1222,7 +1225,7 @@ void Scm_Vprintf(ScmPort *out, const char *fmt, va_list ap, int sharedp)
             case 'p':
                 {
                     void *val = va_arg(ap, void *);
-                    SCM_APPEND1(h, t, Scm_MakeIntegerU((unsigned long)val));
+                    SCM_APPEND1(h, t, Scm_MakeIntegerU((u_long)(intptr_t)val));
                     break;
                 }
             case 'S':; case 'A':
