@@ -104,6 +104,47 @@
              :if-exists #f)
            (call-with-input-file "tmp2.o" read)))
 
+;;-------------------------------------------------------------------
+(test-section "port-fd-dup!")
+
+(cond-expand
+ (gauche.os.windows #f)
+ (else
+  (test* "port-fd-dup!" '("foo" "bar")
+         (let* ((p1 (open-output-file "tmp1.o"))
+                (p2 (open-output-file "tmp2.o")))
+           (display "foo\n" p1)
+           (port-fd-dup! p1 p2)
+           (display "bar\n" p1)
+           (close-output-port p1)
+           (close-output-port p2)
+           (list (call-with-input-file "tmp1.o" read-line)
+                 (call-with-input-file "tmp2.o" read-line))))
+
+  (test* "port-fd-dup!" '("foo" "bar")
+         (let* ((p1 (open-input-file "tmp1.o"))
+                (p2 (open-input-file "tmp2.o"))
+                (s1 (read-line p1)))
+           (port-fd-dup! p1 p2)
+           (list s1 (read-line p1))))
+
+  (test* "port-fd-dup!" *test-error*
+         (let* ((p1 (open-output-file "tmp1.o"))
+                (p2 (open-input-file "tmp2.o")))
+           (guard (e (else
+                      (close-output-port p1)
+                      (close-input-port p2)
+                      (raise e)))
+             (port-fd-dup! p1 p2))))
+
+  (test* "port-fd-dup!" *test-error*
+         (let* ((p1 (open-input-file "tmp2.o")))
+           (guard (e (else
+                      (close-input-port p1)
+                      (raise e)))
+             (port-fd-dup! (open-input-string "") p1))))
+  )) ; !gauche.os.windows
+
 ;;-------------------------------------------------------------------
 (test-section "input ports")
 
@@ -622,6 +663,11 @@
 (test* "format ~@*" "1 2 5"
        (format #f "~a ~a ~4@*~a" 1 2 3 4 5))
 
+(test* "format incomplete tilde sequence" *test-error*
+       (format #f "~"))
+(test* "format incomplete tilde sequence" *test-error*
+       (format #f "~123"))
+
 ;;-------------------------------------------------------------------
 (test-section "nested multi-line comments")
 
