@@ -2,7 +2,7 @@
 ;; testing regexp
 ;;
 
-;; $Id: regexp.scm,v 1.22 2006/03/10 07:28:16 shirok Exp $
+;; $Id: regexp.scm,v 1.27 2007-12-23 06:58:48 shirok Exp $
 
 (use gauche.test)
 (use srfi-1)
@@ -66,6 +66,80 @@
 (test-regexp-parse "(?<name>)(?<name>)\\k<name>"
                    '(0 #f (1 name) (2 name) (alt (backref . 2) (backref . 1))))
 
+;;-------------------------------------------------------------------------
+(test-section "regexp-writer")
+
+(define-syntax test-regexp-writer
+  (syntax-rules ()
+    ((_ exp pat)
+     (test* exp exp (write-to-string (string->regexp pat))))))
+
+(test-regexp-writer "#/abc/" "abc")
+(test-regexp-writer "#/a[\\d]/" "a[\\d]")
+(test-regexp-writer "#/a[a-z]/" "a[a-z]")
+(test-regexp-writer "#/a\\/b/"  "a/b")
+(test-regexp-writer "#/\\\\/"  "\\\\")
+(test-regexp-writer "#/ /"  "\x20")
+(test-regexp-writer "#/\\x20/"  "\\x20")
+
+(test* "#/abc/i" "#/abc/i"
+       (write-to-string (string->regexp "abc" :case-fold #t)))
+
+;;-------------------------------------------------------------------------
+(test-section "compile")
+
+(define-syntax test-regexp-compile
+  (syntax-rules ()
+    ((_ pat)
+     (test* #`"regexp-compile \",|pat|\"" #t
+           (regexp? (regexp-compile (regexp-parse pat)))))))
+
+(test-regexp-compile "a")
+(test-regexp-compile "ab")
+(test-regexp-compile "(?:ab)")
+(test-regexp-compile "(a)")
+(test-regexp-compile "a?")
+(test-regexp-compile "a*")
+(test-regexp-compile "a+")
+(test-regexp-compile "a{3,5}")
+(test-regexp-compile "a{3}")
+(test-regexp-compile "a|b")
+(test-regexp-compile "[ab]")
+(test-regexp-compile "[^ab]")
+(test-regexp-compile ".")
+(test-regexp-compile "^")
+(test-regexp-compile "$")
+(test-regexp-compile "\\b")
+(test-regexp-compile "\\B")
+(test-regexp-compile "(?>a)")
+(test-regexp-compile "a*+")
+(test-regexp-compile "a++")
+(test-regexp-compile "a?+")
+(test-regexp-compile "(?i:a)")
+(test-regexp-compile "(?-i:a)")
+(test-regexp-compile "(?=a)")
+(test-regexp-compile "(?!a)")
+(test-regexp-compile "(?<=ab)")
+(test-regexp-compile "(?<!ab)")
+(test-regexp-compile "(?<name>a)")
+(test-regexp-compile "(?(?=)y)")
+(test-regexp-compile "(?(?=)y|n)")
+(test-regexp-compile "(?(?<=)y)")
+(test-regexp-compile "(?(?<=)y|n)")
+(test-regexp-compile "()(?(1)y)")
+(test-regexp-compile "()(?(1)y|n)")
+(test-regexp-compile "()\\1")
+(test-regexp-compile "(?<name>)\\k<name>")
+(test-regexp-compile "(?<name>)(?<name>)\\k<name>")
+
+;;-------------------------------------------------------------------------
+(test-section "boundary")
+
+(test* "submatch #f" #f (rxmatch-start #f))
+(test* "submatch #f" #f (rxmatch-end #f))
+(test* "submatch #f" #f (rxmatch-substring #f))
+(test* "num-match #f" 0 (rxmatch-num-matches #f))
+
 ;;-------------------------------------------------------------------------
 (test-section "basics")
 
@@ -742,6 +816,9 @@
 (test* "(?<foo>a)" "a"
        (let1 m (#/(?<foo>a)/ "a")
          (m 'foo)))
+(test* "(?<foo>a)" #f
+       (let1 m (#/(?<foo>a)?/ "b")
+         (m 'foo)))
 (test* "(?<foo>a)(?<bar>.*)" '("a" "bcd")
        (let1 m (#/(?<foo>a)(?<bar>.*)/ "abcd")
          (list (m 'foo) (m 'bar))))
@@ -884,6 +961,12 @@
 (test* "regexp-replace" "abc|\\0|ghi"
        (regexp-replace #/def|DEF/ "abcdefghi" "|\\\\0|"))
 
+(test* "regexp-replace" "abc|def|ghi"
+       (regexp-replace #/(?<match>def|DEF)/ "abcdefghi" "|\\k<match>|"))
+
+(test* "regexp-replace" *test-error*
+       (regexp-replace #/(?<match>def|DEF)/ "abcdefghi" "|\\k<matchee>|"))
+
 (test* "regexp-replace" "abraabra**brabra**brabrabracadabrabrabra"
        (regexp-replace #/a((bra)+)cadabra/
                        "abraabraabrabracadabrabrabrabracadabrabrabra"
@@ -894,6 +977,11 @@
                            "abraabraabrabracadabrabrabrabracadabrabrabra"
                            "**\\1**"))
 
+(test* "regexp-replace-all" "abraabra**brabra**br**brabra**brabra"
+       (regexp-replace-all #/a(?<match>(bra)+)cadabra/
+                           "abraabraabrabracadabrabrabrabracadabrabrabra"
+                           "**\\k<match>**"))
+
 (test* "regexp-replace" "abfedhi"
        (regexp-replace #/c(.*)g/ "abcdefghi" 
                        (lambda (m)
